#!/usr/bin/perl

# Script:
# Download Tarball - TrueCrypt v7.1 for 32-bit Linuxes
#
# (21:59 02/07/2015.  Also extracts the tarball's contents, so the tarball
# doesn't have to be copied and extracted every time the Puppy Setup Kit is run.)


use strict;
use warnings;
use local::lib '/root/zzzzz-tmp/Apollias_Perl_Modules';



use ApolliaLib::Common::Loader;


$Launched_Script->Title("Download Tarball - TrueCrypt v7.1 for 32-bit Linuxes");

$Launched_Script->Version("1");
$Launched_Script->ApolliaLib_Version("1");

$Launched_Script->Description("Downloads and unzips the maybe not fully libre TrueCrypt v7.1 for 32-bit Linuxes.  Please note that some very odd news about TrueCrypt came out in May 2014, so, please only use TrueCrypt at your own risk.");


#$ApolliaLib->Shouldnt_Launch_New_Terminal_Window();		# Useful for debugging.

$ApolliaLib->Should_Note_Successful_Script_Completion();
$ApolliaLib->If_Script_Has_Been_Successfully_Completed_Before__Warn_and_Ask_if_User_Wants_to_Continue();
#$ApolliaLib->Dont_Rerun_If_Script_Has_Been_Successfully_Completed_Before();



use ApolliaLib::Puppy_Setup_Kit::Loader;


$Puppy_Setup_Kit->Puppy_Distro_is_("Any_32_bit_Pup");


Typical_Script_Launch();

Typical_Postlaunch_Phase_for_Puppy_Setup_Kit_Script();


#####################################################
#
#	Start of most stuff unique to this script
#


#####################################
#
#	Additional modules
#
#

use ApolliaLib::Puppy_Setup_Kit::Downloader ( '$Puppy_Setup_Kit_Downloader' );

use ApolliaLib::Checksums::SHA512 (
	"SHA512_Check"
	);
	
#
#
#	End of additional modules
#
#####################################################



#####################################
#
#	Start of variable declarations
#

my $human_readable_software_name_and_maybe_desc;

my $Minimum_RAM_Required_in_MB_for_file;

my $URL_for_file;


my $realm_for_file;
my $relative_destination_folders;
my $filename_of_file_to_download;


my $should_proceed;
my $download_verified;

my $uncapitalized_human_readable_software_name_and_maybe_desc;
my $optional_extra_params_for_wget;

my $destination_folder;
my $full_path_to_downloaded_file;
my $full_path_to_SHA512_checksum_for_file;

my $folder_to_run_shell_command_in;
my $command_line;

my $sha512_check_result;
#
#
#	End of variable declarations
#
#####################################################


$human_readable_software_name_and_maybe_desc = "The maybe not fully libre TrueCrypt v7.1 for 32-bit Linuxes - please note that some very odd news about TrueCrypt came out in May 2014, so, please only use TrueCrypt at your own risk";


$Minimum_RAM_Required_in_MB_for_file=5;
	# Make sure you round up.


$URL_for_file="https://github.com/DrWhax/truecrypt-archive/blob/master/truecrypt-7.1-linux-x86.tar.gz?raw=true";

#$optional_extra_params_for_wget = "--output-document='lupu-528.004.iso'";


$realm_for_file="Weir";

$relative_destination_folders="TrueCrypt";
	# Don't put a slash at the beginning or end.

$filename_of_file_to_download="truecrypt-7.1-linux-x86.tar.gz";


# Uncomment these if you don't like the universal defaults already set 
# in $Puppy_Setup_Kit or $ApolliaLib_Settings:

#$Puppy_Setup_Kit->{'should_use_temporary_workspace_for_downloads'}=false;

#$Puppy_Setup_Kit->{'should_allow_downloading_direct_to_physical_disk_if_RAM_disk_is_too_full'}=false;


#$Puppy_Setup_Kit->{'should_allow_downloads'}=false;

$ApolliaLib_Settings->{'should_tell_wget_NOT_to_check_certificate'}=true;
# 02:59 09/22/2014.  Wget refuses to work while using https with github.com
# if you check the certificate.



##############################
#
# Here, we make sure we can proceed at all.  
#
# We abort here if the file already exists,
# or if there's not enough RAM and downloading directly to the physical disk is forbidden.


#!!!!
$Puppy_Setup_Kit_Downloader->Make_Sure_Download_Is_Feasible(
	$realm_for_file, $relative_destination_folders, $filename_of_file_to_download, $human_readable_software_name_and_maybe_desc, $Minimum_RAM_Required_in_MB_for_file, $URL_for_file
	);



##############################
#
# Now, we ask the user (or don't ask, depending on what arguments this script received at launch)
# if they want to download the file.

$uncapitalized_human_readable_software_name_and_maybe_desc=lcfirst($human_readable_software_name_and_maybe_desc);


$should_proceed
	= Ask_or_Dont_Ask_Whether_To_Do_Action
	(
		$Puppy_Setup_Kit->should_or_should_not_install_without_asking, 
		
		"download and unzip $uncapitalized_human_readable_software_name_and_maybe_desc"
	);
	
	

if ($should_proceed)
{
	
	#@@@@
	$download_verified=false;
	#@@@@


	$download_verified = $Puppy_Setup_Kit_Downloader->Download_File($realm_for_file, $relative_destination_folders, $filename_of_file_to_download, $human_readable_software_name_and_maybe_desc, $URL_for_file, $optional_extra_params_for_wget);

	
	if ($download_verified )
	{	
		$destination_folder=$Puppy_Setup_Kit->Realm_Path($realm_for_file) . $relative_destination_folders . "/";
		
		$full_path_to_downloaded_file = $destination_folder . $filename_of_file_to_download;
		
		$full_path_to_SHA512_checksum_for_file=$Puppy_Setup_Kit->Realm_Path($realm_for_file) . "SHA512 Checksums/" . $relative_destination_folders . "/" . "truecrypt-7.1-setup-x86.sha512";
		
		
		print "\n\n";
		
		#--------------------
		#
		# Tarball to extract:
		
		$folder_to_run_shell_command_in=$destination_folder;

		$command_line="tar -xvkf '$filename_of_file_to_download'";

		Install_Program_Via_Shell_Command( $should_proceed, $folder_to_run_shell_command_in, $command_line, "extract TrueCrypt 7.1 tarball", "Extracting TrueCrypt 7.1 tarball!");
		
		
		
		#------------------
		# Now we check the SHA512 checksum of the extracted file.
		#
		# If it doesn't match, the user is warned, but the file is left alone.
		
		$sha512_check_result = SHA512_Check($full_path_to_SHA512_checksum_for_file, "and run check from this folder:", $destination_folder);
		
		if ( $sha512_check_result )
		{	
			
			print "\n\nExtracted file's SHA512 checksum appears correct!";

		}
		else
		{
			print "\n\nWarning: extracted file's SHA512 checksum appears to be wrong!";


		}
	
	
		
		$Launched_Script->Was_Successfully_Completed();
	}

}


#!!!!
End_Script();