#!/bin/sh

# 	Script:                    Launch Puppy Setup Kit                             
#
#   		             (For Any Puppy Setup Kit Blueprint)                          
# 
#	Version 1.0
#	by Apollia of Astroblahhh.Com.  http://astroblahhh.com/
#
#	Completed Sept. 11, 2014.
#
#
#
#------------------------------------------------------------------------------
#
# Not a standalone script - this is intended to be run via a "source" command
# inside a Launch script which will set the variables $PUPPY_DISTRO_ABBREV, 
# $SETUP_KIT_PARENTDIRS, and $SPECIFIC_BLUEPRINT_TO_FOLLOW that this script
# needs set for it.
#
# You shouldn't alter anything in this script - there are no configurable settings.
#
#------------------------------------------------------------------------------
#
#
#
# Under the GNU General Public License v3.0.
#
# Copyright (C) 2014  Apollia
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# Contact info:     http://astroblahhh.com/contact-apollia.shtml
#
# ----


#
#
#                  Nothing in this script should be changed.
#
#
####################################################################################################
####################################################################################################
####################################################################################################



##########################################
#
#
# 	Start of function 
#
#
#	Escape_Single_Quotes_for_Bash_and_Enclose_in_Single_Quotes()
#
#

Escape_Single_Quotes_for_Bash_and_Enclose_in_Single_Quotes()
{	
	local quotes_escaped

	quotes_escaped=$(echo "$1" | sed -e "s/'/'\"'\"'/g")

	echo "'$quotes_escaped'"
}

#
#
#	End of function 
#
#	Escape_Single_Quotes_for_Bash_and_Enclose_in_Single_Quotes()
#
#
###########################################################



##########################################################
#
#	Now, we set a bunch of variables related to the script's
#	path and filename.
#


Get_Script_Path_and_Filename()
{
	THIS_SCRIPT_FILE_NAME="$(basename "${BASH_SOURCE[0]}" )"

	THIS_SCRIPT_FOLDER="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

	full_path_to_this_script="$THIS_SCRIPT_FOLDER/$THIS_SCRIPT_FILE_NAME"

	escaped_enclosed_full_path_to_this_script=$(Escape_Single_Quotes_for_Bash_and_Enclose_in_Single_Quotes "$full_path_to_this_script")
}



Get_Script_Path_and_Filename



##########################################################
#
#	Then, we announce this script's name, path, and what it is.
#


Announce_Script_Now_Running()
{
	echo -e "\n\n*(*)*\n\nBash Script Now Running:\n"
	
	echo "	$THIS_SCRIPT_FILE_NAME

Full path:
	$full_path_to_this_script"

}


Announce_Script_Now_Running




##########################################################
#
#	Should abort if a Puppy Setup Kit was already installed.
#
#


Check_for_Already_Installed_Blueprint()
{
	if [ "$should_abort_if_a_setup_kit_was_already_installed" == "true" ]
	then 
		path_to_note_that_a_blueprint_was_already_installed="/root/zzzzz-tmp/Apollia's Puppy Setup Kit Settings/A Puppy Setup Kit Blueprint Was Already Installed.txt"
		
		if [ -a "$path_to_note_that_a_blueprint_was_already_installed" ]
		# If that file exists...
		then
		
			Xdialog --title "Aborting - A Puppy Setup Kit blueprint was already installed!" --msgbox "A Puppy Setup Kit blueprint was already installed!  Aborting." 30 70
		
			exit
		fi
	fi
}

Check_for_Already_Installed_Blueprint


##########################################################
#
#	Should check to see if a Puppy Setup Kit installation
#	is in progress or was partly completed, and abort if
#	so.

path_to_note_that_a_blueprint_install_is_in_progress="/root/zzzzz-tmp/Apollia's Puppy Setup Kit Settings/A Puppy Setup Kit Blueprint Install Is In Progress.txt"

Check_for_In_Progress_Blueprint_Installation()
{
		
	if [ -a "$path_to_note_that_a_blueprint_install_is_in_progress" ]
		# If that file exists...
		then
			Xdialog --title "Aborting - A partly completed Puppy Setup Kit install detected!" --msgbox "A partly completed Puppy Setup Kit install was detected!  Aborting.\n\n\nIf you wish to continue, you can delete the following file:\n\n/root/zzzzz-tmp/Apollia's Puppy Setup Kit Settings/A Puppy Setup Kit Install Is In Progress.txt\n\n...and then rerun the setup kit launcher." 30 100
		exit
	fi
}

Check_for_In_Progress_Blueprint_Installation


##########################################################
#
#	Now, we probably ask the user if they're sure they
#	want to install this blueprint.


#Xdialog --title "should_confirm_launch_with_xmessage_popup" --msgbox "should_confirm_launch_with_xmessage_popup $should_confirm_launch_with_xmessage_popup " 30 100


if "$should_confirm_launch_with_xmessage_popup" != "false" ]
	then
	xmessage -bg yellow -center -buttons Yes,No "Are you sure you want to launch the below Puppy Setup Kit blueprint?

$BLUEPRINT_TO_FOLLOW"

	answer=$?
	if [ $answer -eq 101 ] ; then
		echo "User clicked 'Yes'"
		# Have to check again for an ongoing installation...
		Check_for_In_Progress_Blueprint_Installation
		
		# And also check again for an already installed blueprint.
		Check_for_Already_Installed_Blueprint
		
	fi

	if [ $answer -eq 102 ] ; then
		echo "User clicked 'No'"
		exit
	fi

fi




##########################################################
#
#	Now, we output the "A Puppy Setup Kit Install Is In Progress.txt" note.
#
#	If all goes well, this note will get deleted after the setup kit installation
#	is complete.  Otherwise, the note will just remain there, and its presence
#	will halt future attempts at setup until the note is manually deleted.


echo -n "Installation started around:  $(date)" > "$path_to_note_that_a_blueprint_install_is_in_progress"

chmod a-rwx,u+r "$path_to_note_that_a_blueprint_install_is_in_progress"
	# Takes away read/write/execute permissions from all, then gives owner read permissions.



##########################################################
#
#	Next, we run the Prepare for Launch script.
#
#


cd "$THIS_SCRIPT_FOLDER"

source "./Prepare for Launch.sh"



##########################################################
#
#	Now that everything is prepared, it's time to run the blueprint.
#
#

path_to_blueprint="/$mountpoint_parentdir/$disk/$SETUP_KIT_PARENTDIRS/Woodland/$PUPPY_DISTRO_ABBREV/Apollia's Puppy Setup Kit/Blueprints/00-Launchers/Start Setup/$BLUEPRINT_TO_FOLLOW.sh"

escaped_path_to_blueprint=$(Escape_Single_Quotes_for_Bash_and_Enclose_in_Single_Quotes "$path_to_blueprint")

command_line="sh $escaped_path_to_blueprint"

eval "$command_line"


##########################################################
#
#	At this point, the Puppy Setup Kit should be finished installing.
#	If so, there should be a file at:
#
#	/root/zzzzz-tmp/Apollia's Puppy Setup Kit Settings/Puppy Setup Kit Launcher Can Create Note Stating That a Puppy Setup Kit Blueprint Was Already Installed.txt
#
#	That file should be created at the end of every blueprint in the Start Setup folder.
#
#
#	So, now we can delete the installation-in-progress note.
#	
#

if [ -a "/root/zzzzz-tmp/Apollia's Puppy Setup Kit Settings/Puppy Setup Kit Launcher Can Create Note Stating That a Puppy Setup Kit Blueprint Was Already Installed.txt" ]
then

	if [ -a "/root/zzzzz-tmp/Apollia's Puppy Setup Kit Settings/A Puppy Setup Kit Blueprint Install Is In Progress.txt" ]
		# If that file exists...
	then
		cd "/root/zzzzz-tmp/Apollia's Puppy Setup Kit Settings"

		rm "A Puppy Setup Kit Blueprint Install Is In Progress.txt"
	fi


	##########################################################
	#
	#	And here, we create a note stating that a Puppy Setup Kit
	#	was already installed.
	#
	#	The existence of this file will block future attempts to run
	#	a Puppy Setup Kit launcher, unless that launcher has
	#	$should_abort_if_a_setup_kit_was_already_installed set to false.
	#

	echo -n "Finished installing $PUPPY_DISTRO_ABBREV - $BLUEPRINT_TO_FOLLOW around:  $(date)" > "$SETTINGS_FOLDER/A Puppy Setup Kit Blueprint Was Already Installed.txt"

	chmod a-rwx,u+r "$SETTINGS_FOLDER/A Puppy Setup Kit Blueprint Was Already Installed.txt"
		# Takes away read/write/execute permissions from all, then gives owner read permissions.
	
	rm "/root/zzzzz-tmp/Apollia's Puppy Setup Kit Settings/Puppy Setup Kit Launcher Can Create Note Stating That a Puppy Setup Kit Blueprint Was Already Installed.txt"
fi


#
#
#  End of script.
#
############################