# ApolliaLib :: Checksums :: SHA512.pm
#
# Stuff related to SHA512 checksums.


use strict;
use warnings;


package ApolliaLib::Checksums::SHA512;



################################################################
#
#                         Exported Stuff
#
# This bit specifies which parts of this module are made available to
# scripts that use the following line:
#
# 	use ApolliaLib::Checksums::SHA512;
#
#
# I pasted much of the below from:
#
# 	http://perldoc.perl.org/perlmod.html


BEGIN 
{
	require Exporter;

	#set the version for version checking
	our $VERSION = 1.00;

	#Inherit from Exporter to export functions and variables
	our @ISA = qw(Exporter);

	#Functions and variables which are exported by default
	our @EXPORT = qw(  
		SHA512_Check
		Create_SHA512_Checksum
		);

	#Functions and variables which can be optionally exported
	our @EXPORT_OK = qw(
	
		);
}



#
#                End of Exported Stuff section. 
#
#
################################################################



##################################################
#
#	Needed imports from other modules
#

use ApolliaLib::Common::Misc qw (
	true
	false
	Escape_Single_Quotes_for_Bash
	Display_and_Do_Command_Line_and_Result
	necho
	);

use ApolliaLib::Common::End_Script qw (
	End_Script_and_Wait_for_Return
	);

use ApolliaLib::Common::Settings (
	'$ApolliaLib_Settings'
	);
	
use File::Path ("mkpath");

use ApolliaLib::Common::Path_String_Handling (
	"Get_Filename_and_Parentdirs",
	"Get_Filename__Parentdirs__and_File_Suffix"

	);
	
#
#	End of Needed imports from other modules
#
##################################################




################################################################
#
#	Start of stuff unique to this module
#


sub Create_SHA512_Checksum
{
	my $full_path_to_file_to_make_checksum_for=$_[0];
	
	#----
	my $command_line;
	#my $escaped_full_path_to_file_to_make_checksum_for;

	my $filename;
	my $parentdirs;

	#my $escaped_parentdirs;
	my $escaped_filename;
	
	#@@@@	#$escaped_full_path_to_file_to_make_checksum_for=Escape_Single_Quotes_for_Bash($full_path_to_file_to_make_checksum_for);
	
	($filename, $parentdirs) = Get_Filename_and_Parentdirs( $full_path_to_file_to_make_checksum_for );
	
	#$escaped_parentdirs=Escape_Single_Quotes_for_Bash($parentdirs);
	
	$escaped_filename=Escape_Single_Quotes_for_Bash($filename);
	
	
	
	##############################
	
	chdir $parentdirs;
	
	$command_line = "sha512sum '$escaped_filename' > '$escaped_filename.sha512'";

	Display_and_Do_Command_Line_and_Result($command_line);
	
}



sub SHA512_Check
{	
	my $full_path_to_SHA512_checksum_for_file=$_[0];
	
	# Arg 1 - unused, just for readability.  I usually put "and run check from this folder:"
	
	
	my $folder_of_file_to_check=$_[2];
	
	#----
	my $escaped_full_path_to_SHA512_checksum_for_file;
	my $command_line;
	my $sha512_check_result;
	
	#@@@@
	$escaped_full_path_to_SHA512_checksum_for_file=Escape_Single_Quotes_for_Bash($full_path_to_SHA512_checksum_for_file);
	
	
	
	##############################
	
	chdir $folder_of_file_to_check;
	
	
	
	$command_line = "sha512sum --check '$escaped_full_path_to_SHA512_checksum_for_file'";
	
	$sha512_check_result=Display_and_Do_Command_Line_and_Result($command_line);
		
	if ( $sha512_check_result =~ /OK$/ )
	{	
		
		#!!!!
		return true;
	}
	
	
	#!!!!
	return false;
	
}




1;