# ApolliaLib :: Common :: Arguments.pm
#
# Deals with arguments sent to a script via the command line or drag & drop.
# (See also: Dragdrop.pm)


use strict;
use warnings;


package ApolliaLib::Common::Arguments;



################################################################
#
#                         Exported Stuff
#
# This bit specifies which parts of this module are made available to
# scripts that use the following line:
#
# 	use ApolliaLib::Common::Arguments;
#
#
# I pasted much of the below from:
#
# 	http://perldoc.perl.org/perlmod.html


BEGIN 
{
	require Exporter;

	#set the version for version checking
	our $VERSION = 1.00;

	#Inherit from Exporter to export functions and variables
	our @ISA = qw(Exporter);

	#Functions and variables which are exported by default
	our @EXPORT = qw(  
		Remove_Now_Unneeded_Clutter_from_ARGV
		Determine_Automaticness_Using_Argv
		Determine_Script_Ending_Type_Using_Argv
		
		Is_Recognized_ApolliaLib_Window_Managing_Arg
		Is_Recognized_ApolliaLib_Script_Endingtype_Arg
		Is_Recognized_ApolliaLib_Automaticness_Arg
		Is_Recognized_ApolliaLib_Arg
		Get_Nonignored_Args_from_ARGV
		Make_String_of_Escaped_Args_out_of_Array
		);

	#Functions and variables which can be optionally exported
	our @EXPORT_OK = qw(
	
		);
}



#
#                End of Exported Stuff section. 
#
#
################################################################



##################################################
#
#	Needed imports from other modules
#


use ApolliaLib::Common::Misc qw ( 
	true 
	false
	Print_Array_Indentedly
	Escape_Single_Quotes_for_Bash
	necho
	);

use ApolliaLib::Common::Script_Details qw ( 
	$Launched_Script_Details
	);
	

use ApolliaLib::Common::Settings qw ( 
	$ApolliaLib_Settings
	);


#
#	End of Needed imports from other modules
#
##################################################



################################################################
#
#	Start of stuff unique to this module
#



sub Make_String_of_Escaped_Args_out_of_Array
{	
	my $ref_to_array=$_[0];


	#----
	my @array;
	my $arg;
	my $string_of_escaped_args;
	
	#@@@@
	@array=@$ref_to_array;
	$string_of_escaped_args="";


	
	##############################
	
	foreach $arg (@array)
	{	
		$arg=Escape_Single_Quotes_for_Bash($arg);
		$string_of_escaped_args = $string_of_escaped_args . " '$arg' ";
		
	}


	#!!!!
	return $string_of_escaped_args;
}



sub Determine_Automaticness_Using_Argv
{	
	
	#----
	my $arg;

	
	
	##############################
	
	foreach $arg (@ARGV)
	{
		if ( Is_Recognized_ApolliaLib_Automaticness_Arg($arg) )
		{	
			
			$ApolliaLib_Settings->{'automaticness_setting'} = $arg;
			
			if ($arg eq "Do_Things_Automatically")
			{	$ApolliaLib_Settings->{'should_do_things_automatically'} = true;
				
			}
			else
			{	$ApolliaLib_Settings->{'should_do_things_automatically'} = false;
			}
			
			
			#!!!!
			return true;
		}
	}
	
	
	#!!!!
	return false;
		
}



sub Determine_Script_Ending_Type_Using_Argv
{	
	
	#----
	my $arg;


	
	##############################
	
	foreach $arg (@ARGV)
	{
		
		if ( Is_Recognized_ApolliaLib_Script_Endingtype_Arg($arg) )
		{	
			$ApolliaLib_Settings->{'script_ending_type'} = $arg;
			
			
			#!!!!
			return true;
		}
	}
	
	
	#!!!!
	return false;
		
}



sub Is_Recognized_ApolliaLib_Script_Endingtype_Arg
{	# Used in if statements and ApolliaLib_Recognized_Arg
	
	my $arg=$_[0];
	

	#----
	my @List_of_Recognized_Script_Endingtype_Args;
	
	
	#@@@@
	if (!defined $arg)
	{
	
		#!!!!
		return false;
	}
	
	@List_of_Recognized_Script_Endingtype_Args=
		@{$ApolliaLib_Settings->{'List_of_Recognized_Script_Endingtype_Args'} };
	


	##############################
	
# http://stackoverflow.com/questions/2925604/how-do-i-search-a-perl-array-for-a-matching-string
	
	if (  /^$arg$/ ~~ @List_of_Recognized_Script_Endingtype_Args )
	{	
		
		#!!!!
		return true;
	}
	
	
	#!!!!
	return false;
	
}



sub Is_Recognized_ApolliaLib_Automaticness_Arg
{	# Used in if statements and ApolliaLib_Recognized_Arg

	my $arg=$_[0];
	
	
	#----	
	my @List_of_Recognized_Automaticness_Args;
	
	#@@@@	
	@List_of_Recognized_Automaticness_Args=
		@{$ApolliaLib_Settings->{'List_of_Recognized_Automaticness_Args'} };
	

	
	##############################
		
# http://stackoverflow.com/questions/2925604/how-do-i-search-a-perl-array-for-a-matching-string
	
	if ( /^$arg$/ ~~ @List_of_Recognized_Automaticness_Args )
	{	
		
		#!!!!
		return true;
	}
	
	
	#!!!!
	return false;
}



sub Is_Recognized_ApolliaLib_Window_Managing_Arg
{	# Used in if statements and ApolliaLib_Recognized_Arg
	
	my $arg=$_[0];
	
	#----	
	my @List_of_Recognized_Window_Managing_Args;
	
	#@@@@
	@List_of_Recognized_Window_Managing_Args=
		@{$ApolliaLib_Settings->{'List_of_Recognized_Window_Managing_Args'} };
	


	##############################
	
# http://stackoverflow.com/questions/2925604/how-do-i-search-a-perl-array-for-a-matching-string
	
	if ( /^$arg$/ ~~ @List_of_Recognized_Window_Managing_Args )
	{	
		
		#!!!!
		return true;
	}
	
	
	#!!!!
	return false;
	
	 
}



sub Is_Recognized_ApolliaLib_Arg
{	# Used in if statements.
	my $arg=$_[0];
	
	
	#necho ("recog aplib arg: $arg ");

	##############################
	
	if	(	Is_Recognized_ApolliaLib_Window_Managing_Arg($arg) 
			|| 
			Is_Recognized_ApolliaLib_Script_Endingtype_Arg($arg) 
			||
			Is_Recognized_ApolliaLib_Automaticness_Arg($arg)
		)
	{	
		
		#!!!!
		return true;
	}
	
	
	#!!!!
	return false;	
}



sub Is_Recognized_Ignored_Non_ApolliaLib_Arg
{	
	my $arg=$_[0];
	
	
	#----
	my @List_of_Non_ApolliaLib_Args_to_Ignore;
	
	
	
	##############################
	
	if (defined @{$ApolliaLib_Settings->{'List_of_Non_ApolliaLib_Args_to_Ignore'} } )
	{
		@List_of_Non_ApolliaLib_Args_to_Ignore=
			@{$ApolliaLib_Settings->{'List_of_Non_ApolliaLib_Args_to_Ignore'} };
	
	
		if ( /^$arg$/ ~~ @List_of_Non_ApolliaLib_Args_to_Ignore )
		{	
			
			#!!!!
			return true;
		}
	}
	
	
	#!!!!
	return false;
	
}



sub Contains_Terminal_Window_Launch_Command
{
	my $arg=$_[0];
	
	
	#----	
	my $position_of_terminal_window_launch_command;



	##############################

	$position_of_terminal_window_launch_command=index($arg, $ApolliaLib_Settings->Start_of_Command_to_Launch_Terminal_Window);
	
	if ( $position_of_terminal_window_launch_command == -1)
	{	
		
		#!!!!
		return false;
	}
	
	
	#!!!!
	return true;	
}



sub Get_Nonignored_Args_from_ARGV
{
	
	#----
	my $arg;
	
	my @ARGV_copy;
	my @results;
	
	#@@@@
	@ARGV_copy=@ARGV;
		# This is needed because manipulating the real ARGV while looping through it
		# confuses foreach and makes it skip elements.
	


	##############################
	
	foreach $arg (@ARGV_copy) 
	{	if ( Is_Recognized_ApolliaLib_Arg($arg) || Is_Recognized_Ignored_Non_ApolliaLib_Arg($arg) || Contains_Terminal_Window_Launch_Command($arg) )
		{	next;
		}
		push (@results, $arg);
	}
	
	
	#!!!!
	return \@results;	
}



sub Remove_Now_Unneeded_Clutter_from_ARGV
{
	
	
	#----
	my $arg;	
	my @ARGV_copy;


	#@@@@
	@ARGV_copy=@ARGV;
		# This is needed because manipulating the real ARGV while looping through it
		# confuses foreach and makes it skip elements.
	


	##############################
	
	foreach $arg (@ARGV_copy)
	{
		if ( Is_Recognized_ApolliaLib_Arg($arg) )
		{	# Then we take it out of the real ARGV array.

			@ARGV = grep(!/$arg/, @ARGV);

		}
		
		if ( Is_Recognized_Ignored_Non_ApolliaLib_Arg($arg) )
		{	@ARGV = grep(!/$arg/, @ARGV);
		}
	}
}



1;