# ApolliaLib :: Common :: Branching.pm
#
# Related to launching other scripts.


use strict;
use warnings;


package ApolliaLib::Common::Branching;



################################################################
#
#                         Exported Stuff
#
# This bit specifies which parts of this module are made available to
# scripts that use the following line:
#
# 	use ApolliaLib::Common::Branching;
#
#
# I pasted much of the below from:
#
# 	http://perldoc.perl.org/perlmod.html


BEGIN 
{
	require Exporter;

	#set the version for version checking
	our $VERSION = 1.00;

	#Inherit from Exporter to export functions and variables
	our @ISA = qw(Exporter);

	#Functions and variables which are exported by default
	our @EXPORT = qw(  
		Launch_Other_Script
		Launch_Many_Relativepath_Scripts
		);

	#Functions and variables which can be optionally exported
	our @EXPORT_OK = qw(
	
		);
}



#
#                End of Exported Stuff section. 
#
#
################################################################



##################################################
#
#	Needed imports from other modules
#


use ApolliaLib::Common::Misc qw ( 
	true 
	false
	Print_Array_Indentedly
	Escape_Single_Quotes_for_Bash
	Display_and_Do_Command_Line_and_Result
	Display_and_Do_Command_Line
	Prefix_Array_Values_With
	String_Consists_of_Nothing_but_Single_Quotes_or_Spaces
	
	
	);

use ApolliaLib::Common::Path_String_Handling qw (
	Get_Filename_from_Path
	);
	

use ApolliaLib::Common::Arguments (
	"Is_Recognized_ApolliaLib_Window_Managing_Arg",
	"Is_Recognized_ApolliaLib_Script_Endingtype_Arg",
	"Is_Recognized_ApolliaLib_Automaticness_Arg",
	"Make_String_of_Escaped_Args_out_of_Array",
	);
	

use ApolliaLib::Common::End_Script (
	"End_Script_For_Problem_Using_Whatever_Ending_Type_The_Settings_Specify"
	);

	
use ApolliaLib::Common::Settings qw ( 
	$ApolliaLib_Settings
	$ApolliaLib_Settings_say_we
	);
	
use ApolliaLib::Common::Separate_Process (
	"Display_Command_Line_and_Issue_It_In_A_Separate_Process"
	);

use ApolliaLib::Common::User_Prompted_Actions (
	"Ask_To_Do_Action"
	);


#
#	End of Needed imports from other modules
#
##################################################




################################################################
#
#	Start of stuff unique to this module
#


# 10:57 05/14/2014.  These functions are largely modeled after:
#
# 	ApolliaLib :: File_and_Folder_Handling :: Many_Relativepath_Items ::
#		Copy_Relativepath_Items_to_Their_Dest_Parentdirs(),
#
#	ApolliaLib :: File_and_Folder_Handling :: Many_Items ::
#		Copy_Many_Items_to_Their_Dest_Parentdirs(),
#
#	and
#
#	ApolliaLib :: File_and_Folder_Handling :: Copy_Item ::
#		Copy_Item()
#
#
# What these sets of functions both have in common is, they both involve making full paths
# out of relative paths and a prefix, then sending those paths to a
# single function which handles the actual execution.



sub Launch_Many_Relativepath_Scripts
{
	my $shell_command_to_launch_other_script=$_[0];

	my $prefix_for_all_array_values=$_[1];
			
	my $ref_to_array_of_relativepaths_to_scripts=$_[2];
	
	my $do_or_dont_open_terminal_window=$_[3];
	my $script_ending_type=$_[4];
	my $automatically_or_not=$_[5];
	
	my $ref_to_array_of_all_other_args=$_[6];
	my $should_launch_in_separate_process=$_[7];
	
	
	#----	
	my @all_other_args;
	my @relativepaths_to_scripts;
	my $ref_to_array_of_fullpaths_to_scripts;
	my @fullpaths_to_scripts;
	
	#@@@@
	if (!defined $do_or_dont_open_terminal_window)
	{	
		print "\n\n	WARNING: Had to use default value for do_or_dont_open_terminal_window in Launch_Many_Relativepath_Scripts()!";
		
		$do_or_dont_open_terminal_window="Dont_Open_Window";
	}
	if (!defined $script_ending_type)
	{	print "\n\n	WARNING: Had to use default value for script_ending_type in Launch_Many_Relativepath_Scripts()!";
	
		$script_ending_type="Wait_For_Return_At_Scripts_End";
	}
	if (!defined $automatically_or_not)
	{	print "\n\n	WARNING: Had to use default value for automatically_or_not in Launch_Many_Relativepath_Scripts()!";
	
		$automatically_or_not="Dont_Do_Things_Automatically";
	}
	
	if (!defined $ref_to_array_of_all_other_args)
	{	
		@all_other_args=("");
	}
	else
	{	@all_other_args=@$ref_to_array_of_all_other_args;
	}
	
	
	@relativepaths_to_scripts=@$ref_to_array_of_relativepaths_to_scripts;
	
	

	##############################
	
	$ref_to_array_of_fullpaths_to_scripts=Prefix_Array_Values_With($prefix_for_all_array_values, \@relativepaths_to_scripts);
	
	@fullpaths_to_scripts=@$ref_to_array_of_fullpaths_to_scripts;
	
	
	Launch_Many_Other_Scripts($shell_command_to_launch_other_script, \@fullpaths_to_scripts, $do_or_dont_open_terminal_window, $script_ending_type, $automatically_or_not, \@all_other_args, $should_launch_in_separate_process);
}



sub Launch_Many_Other_Scripts
{	
	my $shell_command_to_launch_other_script=$_[0];
	
	my $ref_to_array_of_fullpaths_to_scripts=$_[1];
		
	my $do_or_dont_open_terminal_window=$_[2];
	my $script_ending_type=$_[3];
	my $automatically_or_not=$_[4];
	
	my $ref_to_array_of_all_other_args=$_[5];
	
	my $should_launch_in_separate_process=$_[6];
	

	#----	
	my @fullpaths_to_scripts;
	my $fullpath_to_script;
	
	my @all_other_args;
	
	#@@@@
	if (!defined $do_or_dont_open_terminal_window)
	{	
		print "\n\n	WARNING: Had to use default value for do_or_dont_open_terminal_window in Launch_Many_Other_Scripts()!";
		
		$do_or_dont_open_terminal_window="Dont_Open_Window";
	}
	if (!defined $script_ending_type)
	{	print "\n\n	WARNING: Had to use default value for script_ending_type in Launch_Many_Other_Scripts()!";
	
		$script_ending_type="Wait_For_Return_At_Scripts_End";
	}
	if (!defined $automatically_or_not)
	{	print "\n\n	WARNING: Had to use default value for automatically_or_not in Launch_Many_Other_Scripts()!";
	
		$automatically_or_not="Dont_Do_Things_Automatically";
	}
	
	if (!defined $ref_to_array_of_all_other_args)
	{	
		@all_other_args=("");
	}
	else
	{	@all_other_args=@$ref_to_array_of_all_other_args;
	}
	
	@fullpaths_to_scripts=@$ref_to_array_of_fullpaths_to_scripts;
	
	

	##############################

	foreach $fullpath_to_script (@fullpaths_to_scripts)
	{	
		
		Launch_Other_Script($shell_command_to_launch_other_script, $fullpath_to_script, $do_or_dont_open_terminal_window, $script_ending_type, $automatically_or_not, $ref_to_array_of_all_other_args, $should_launch_in_separate_process );
		
	}
}



sub Launch_Other_Script
{	my $shell_command_to_launch_other_script=$_[0];
	my $full_path_to_other_script=$_[1];
	
	my $do_or_dont_open_terminal_window=$_[2];
	my $script_ending_type=$_[3];
	my $automatically_or_not=$_[4];
	
	my $ref_to_array_of_all_other_args=$_[5];
	
	my $should_launch_in_separate_process=$_[6];
	
	
	#----	
	my @all_other_args;
	my $escaped_full_path_to_other_script;
	my $string_of_escaped_args;
	
	my $command_line;
	my $result;
	
	my $should_proceed;
	my $filename_of_other_script;
	my $parentdirs_of_other_script;
	
	#@@@@
	$should_proceed=false;
	
	if (!defined $do_or_dont_open_terminal_window)
	{	
		print "\n\n	WARNING: Had to use default value for do_or_dont_open_terminal_window in Launch_Other_Script()!";
		
		$do_or_dont_open_terminal_window="Dont_Open_Window";
	}
	if (!defined $script_ending_type)
	{	print "\n\n	WARNING: Had to use default value for script_ending_type in Launch_Other_Script()!";
	
		$script_ending_type="Wait_For_Return_At_Scripts_End";
	}
	if (!defined $automatically_or_not)
	{	print "\n\n	WARNING: Had to use default value for automatically_or_not in Launch_Other_Script()!";
	
		$automatically_or_not="Dont_Do_Things_Automatically";
	}
	
	
	
	##############################
	
	
	if (!defined $shell_command_to_launch_other_script)
	{	
		
		
		#!!!!
		End_Script_For_Problem_Using_Whatever_Ending_Type_The_Settings_Specify("\n\n\n\nAborting script, because in the function Launch_Other_Script(), the shell command to launch the other script with wasn't specified!");
	}
	
	
	if ($shell_command_to_launch_other_script ne "perl" && $shell_command_to_launch_other_script ne "sh" )
	{	
		
		#!!!!
		End_Script_For_Problem_Using_Whatever_Ending_Type_The_Settings_Specify("\n\n\n\nAborting script, because you can't use a shell command other than 'perl' or 'sh' with the function Launch_Other_Script()!  And the command '$shell_command_to_launch_other_script' was used.");
		
	}
	
	if ( !Is_Recognized_ApolliaLib_Window_Managing_Arg($do_or_dont_open_terminal_window) )
	{
		
		#!!!!
		End_Script_For_Problem_Using_Whatever_Ending_Type_The_Settings_Specify("\n\n\n\nAborting script, because with the function Launch_Other_Script(), you must use a recognized ApolliaLib window managing argument for the do_or_dont_open_terminal_window argument, and instead,  '$do_or_dont_open_terminal_window' was used.");
	}
	
	if ( !Is_Recognized_ApolliaLib_Script_Endingtype_Arg($script_ending_type) )
	{
		
		#!!!!
		End_Script_For_Problem_Using_Whatever_Ending_Type_The_Settings_Specify("\n\n\n\nAborting script, because with the function Launch_Other_Script(), you must use a recognized ApolliaLib script ending type argument for the script_ending_type argument, and instead, '$script_ending_type' was used.");
	}
	if ( !Is_Recognized_ApolliaLib_Automaticness_Arg($automatically_or_not) )
	{
		
		#!!!!
		End_Script_For_Problem_Using_Whatever_Ending_Type_The_Settings_Specify("\n\n\n\nAborting script, because with the function Launch_Other_Script(), you must use a recognized ApolliaLib script automaticness argument for the automatically_or_not argument, and instead, '$automatically_or_not' was used.");
	}
	
	
		#@@@@
	# No error checking for the miscellaneous $all_other_args.
	if (defined $ref_to_array_of_all_other_args)
	{	
		@all_other_args=@$ref_to_array_of_all_other_args;
		$string_of_escaped_args=Make_String_of_Escaped_Args_out_of_Array(\@all_other_args);
	}
	else
	{	$string_of_escaped_args="";
	}
	
	$escaped_full_path_to_other_script=Escape_Single_Quotes_for_Bash($full_path_to_other_script);
		#@@@@
	
	
	
	#2#############################
	
	$command_line="$shell_command_to_launch_other_script '$escaped_full_path_to_other_script' '$do_or_dont_open_terminal_window' '$script_ending_type' '$automatically_or_not'";
	
	if ( !String_Consists_of_Nothing_but_Single_Quotes_or_Spaces($string_of_escaped_args) )
	{	$command_line = $command_line . " $string_of_escaped_args";
	}
	
	if ($ApolliaLib_Settings_say_we->should_ask__when_branching__whether_to_run_script)
	{	
		$filename_of_other_script=Get_Filename_from_Path($full_path_to_other_script);	
	
		$should_proceed = Ask_To_Do_Action
		(		
			"run the script $filename_of_other_script"
			
		);
		
		if ($should_proceed==false)
		{	
			
			#!!!!
			return false;
		}
	}
	
	
	
	if ( $should_launch_in_separate_process )
	{
		$command_line=$command_line . " &";
	
		Display_Command_Line_and_Issue_It_In_A_Separate_Process($command_line);
		
	}
	else
	{	Display_and_Do_Command_Line($command_line);
	}

	
	#!!!!
	return true;
	
}



1;