# ApolliaLib :: Common :: Completed_Script.pm
#
# Stuff related to creating and reading notes in /root/zzzzz-tmp/ which state that a script has been
# successfully completed.


use strict;
use warnings;


package ApolliaLib::Common::Completed_Script;



################################################################
#
#                         Exported Stuff
#
# This bit specifies which parts of this module are made available to
# scripts that use the following line:
#
# 	use ApolliaLib::Common::Completed_Script;
#
#
# I pasted much of the below from:
#
# 	http://perldoc.perl.org/perlmod.html


BEGIN 
{
	require Exporter;

	#set the version for version checking
	our $VERSION = 1.00;

	#Inherit from Exporter to export functions and variables
	our @ISA = qw(Exporter);

	#Functions and variables which are exported by default
	our @EXPORT = qw(  
		Note_of_Successful_Completion_Exists_or_Not
		Note_Successfully_Completed_Script
		Get_Path_of_Note_of_Successful_Completion
		);

	#Functions and variables which can be optionally exported
	our @EXPORT_OK = qw(
	
		);
}



#
#                End of Exported Stuff section. 
#
#
################################################################



##################################################
#
#	Needed imports from other modules
#


use ApolliaLib::Common::Misc qw ( 
	true
	false
	Escape_Single_Quotes_for_Bash
	necho
	);
	

use ApolliaLib::Common::Path_String_Handling qw (
	Get_Parentdirs_of_Path
	);
	
use ApolliaLib::Common::Script_Details qw ( 
	$Launched_Script
	);
	
use ApolliaLib::Common::Settings qw ( 
	$ApolliaLib_Settings
	$ApolliaLib_Settings_say_we
	);

use File::Path ("mkpath");


#
#	End of Needed imports from other modules
#
##################################################




################################################################
#
#	Start of stuff unique to this module
#



sub Note_of_Successful_Completion_Exists_or_Not
{	
	# For use in if statements.
	
	
	#----
	my $note_path;
	
	
	
	##############################
	
	$note_path=Get_Path_of_Note_of_Successful_Completion();
	
	if (-e $note_path)
	{	#necho "Yes, note of successful completion exists at $note_path!";
	
		
		#!!!!
		return true;
	}
	#necho "No, note of successful completion doesn't exist at $note_path!";
	
	
	#!!!!
	return false;	
}



sub Get_Path_of_Note_of_Successful_Completion
{
	
	
	#----
	my $note_path;


	
	##############################
		
	$note_path=$ApolliaLib_Settings->full_parentdirs_of_notes_of_successfully_completed_scripts . $Launched_Script->full_unescaped_path;


	#!!!!
	return $note_path;
}



sub Note_Successfully_Completed_Script
{	
	
	
	#----
	my $note_path;
	my $escaped_note_path;
	my $parentdirs_of_note_path;
	my $printable_datime;
	my $printable_script_path;
	


	##############################
	
	if ( $ApolliaLib_Settings_say_we->should_note_successfully_completed_script )
	{	

		if ( $Launched_Script->was_successfully_completed_on_this_run )
		{	
			$note_path=Get_Path_of_Note_of_Successful_Completion();
			
			$parentdirs_of_note_path=Get_Parentdirs_of_Path($note_path);
			
			mkpath $parentdirs_of_note_path;
			
			
			open(my $NOTE_FILE, '>', $note_path) or print "Couldn't open $note_path : $!";
   
		
			$printable_script_path=$Launched_Script->full_unescaped_path;
			$printable_datime=localtime( $Launched_Script->datime_of_this_run );
			
			print $NOTE_FILE "Completed running\n\n	$printable_script_path\n\non:\n\n";
			print $NOTE_FILE $printable_datime;
			close $NOTE_FILE;
		}
	}
}	



1;