# ApolliaLib :: Common :: End_Script.pm
#
# Stuff related to ending a script.


use strict;
use warnings;


package ApolliaLib::Common::End_Script;



################################################################
#
#                         Exported Stuff
#
# This bit specifies which parts of this module are made available to
# scripts that use the following line:
#
# 	use ApolliaLib::Common::End_Script;
#
#
# I pasted much of the below from:
#
# 	http://perldoc.perl.org/perlmod.html


BEGIN 
{
	require Exporter;

	#set the version for version checking
	our $VERSION = 1.00;

	#Inherit from Exporter to export functions and variables
	our @ISA = qw(Exporter);

	#Functions and variables which are exported by default
	our @EXPORT = qw(  

		Get_Script_Ending_Type_from_Argv
		End_Script  
		End_Script_Immediately
		End_Script_with_Autoclose
		End_Script_and_Wait_for_Return
		End_Script_For_Problem_Using_Whatever_Ending_Type_The_Settings_Specify
		Immediately_Abort_if_Variable_is_Undefined
		);

	#Functions and variables which can be optionally exported
	our @EXPORT_OK = qw(
	
		);
}



#
#                End of Exported Stuff section. 
#
#
################################################################



##################################################
#
#	Needed imports from other modules
#


use ApolliaLib::Common::Misc qw ( 
	
	Escape_Single_Quotes_for_Bash
	necho
	);

use ApolliaLib::Common::User_Prompted_Actions qw (
	More_Safely_Get_STDIN
	);

use ApolliaLib::Common::Script_Details qw ( 
	
	$Launched_Script
	
	);
	
use ApolliaLib::Common::Settings qw ( 
	
	$ApolliaLib_Settings
	$ApolliaLib_Settings_say_we
	);

use ApolliaLib::Common::Arguments qw (

	Is_Recognized_ApolliaLib_Script_Endingtype_Arg

	);
	

use ApolliaLib::Common::Completed_Script (
	"Note_Successfully_Completed_Script"
	);
use ApolliaLib::Common::X_Dialog (
	"Display_Xdialog",
	"Display_Xmessage"
	);
	

#
#	End of Needed imports from other modules
#
##################################################




################################################################
#
#	Start of stuff unique to this module
#



sub Immediately_Abort_if_Variable_is_Undefined
{	
	my $var=$_[0];
	
	if (!defined $var)
	{
		print "Found undefined var!  Must abort immediately!";
		
		End_Script_Immediately();
	}
}



sub End_Script_For_Problem_Using_Whatever_Ending_Type_The_Settings_Specify
{	
	my $last_words=$_[0];
	
	
	#----

	my $xdialog_title;
	my $color_for_last_words_xmessage;

	
	
	##############################
	
	print $last_words;
	
	
	
	if ($ApolliaLib_Settings_say_we->should_display_last_words_in_popup_xmessage_when_aborting)
	{	
			#@@@@
		$color_for_last_words_xmessage=$ApolliaLib_Settings->color_for_last_words_xmessage;
			#@@@@
		
		Display_Xmessage($last_words, $color_for_last_words_xmessage);
	}
	
	if ($ApolliaLib_Settings_say_we->should_display_last_words_in_popup_xdialog_when_aborting)
	{
			#@@@@
		$xdialog_title=$Launched_Script->Title;
		$xdialog_title="Aborted $xdialog_title !";
			#@@@@
		
		
		Display_Xdialog($xdialog_title, $last_words);	
		
		
	}
 
 
 	#!!!!
	End_Script($ApolliaLib_Settings->script_ending_type_when_a_problem_is_found, $ApolliaLib_Settings->num_of_seconds_before_autoclose_when_a_problem_is_making_the_script_close_prematurely);	
}



sub End_Script_and_Wait_for_Return
{
	
	
	#----
	my $wait_for_return;


	
	##############################
	
	Routine_Script_Ending_Stuff();
	
	print "\n\n\nYou can press Return (or Enter) to end this script.\n";
	
	$wait_for_return = More_Safely_Get_STDIN();
	
	
	#!!!!
	die;	
}



sub Routine_Script_Ending_Stuff
{
	
	
	#----
	my $datime_this_script_ended;
	my $printable_datime;
	my $divider;



	##############################
	
	$divider=$ApolliaLib_Settings->script_ending_divider_text;
	
	print $divider;
	
	
	$datime_this_script_ended=time;
										$Launched_Script->{'datime_this_script_ended'}=$datime_this_script_ended;
										
	$printable_datime=localtime( $datime_this_script_ended );
	
	
	print "\n\n\nScript finished at $printable_datime!";
	
	
	$Launched_Script->Print_Script_Title_and_Path();

}



sub End_Script_Immediately
{

	
	
	##############################
	
	Routine_Script_Ending_Stuff();
	

	print "\n\nImmediate end of script!";	
	
	print "\n\n\n\n*(*)*\n\n\n\n\n";


	#!!!!
	die;
}


sub End_Script_with_Autoclose
{
	my $secs=$_[0];

	
	
	##############################
	
	if(!$secs) # Secs sent as an argument take precedence over config settings.
	{	$secs = $ApolliaLib_Settings->num_of_seconds_before_autoclose;
		if (!$secs)
		{	print "\n\nApolliaLib_Settings num_of_seconds_before_autoclose was unset!  Default: 5 secs";
			$secs=5;
		}
	}
	
	Routine_Script_Ending_Stuff();
	
	print "\n\nThis script will automatically end in $secs seconds.\n\n";
		
	sleep $secs;


	#!!!!
	die;
}



sub End_Script
{
	my $script_ending_type=$_[0];
	my $secs=$_[1];
	
	

	##############################
	
	
	if (defined $script_ending_type)
	{
		if ( ! Is_Recognized_ApolliaLib_Script_Endingtype_Arg($script_ending_type) )
		{	
			print "\n\n\n\nWarning: No recognized ApolliaLib script ending type provided!";
			
			
			#!!!!
			End_Script_and_Wait_for_Return();
			die;
			
		}
	}
	else
	{	$script_ending_type=$ApolliaLib_Settings->script_ending_type;
	}
	
	if (!$secs) # Secs sent as an argument take precedence over config settings.
	{	
		$secs = $ApolliaLib_Settings->num_of_seconds_before_autoclose;
	}


	Note_Successfully_Completed_Script();
	
	
	if (!$script_ending_type || $script_ending_type eq "Wait_For_Return_At_Scripts_End")
	{
		# Then don't quit until user presses Return.
		
		
		#!!!!
		End_Script_and_Wait_for_Return();
		die;
	}
	elsif ($script_ending_type eq "Autoclose_After_X_Seconds")
	{	
		# Wait x seconds, then close automatically.
		
		
		#!!!!
		End_Script_with_Autoclose($secs);
		die;
	}
	elsif ($script_ending_type eq "Dont_Wait_For_Return_At_Scripts_End")
	{	
		# End script immediately.
		
		
		#!!!!
		End_Script_Immediately();
		die;
	}
	else
	{
		print "\n\nEnding type invalid! $script_ending_type  This error should never be reached!";
	
	
		#!!!!
		End_Script_and_Wait_for_Return();
		die;
	}
}



1;