# ApolliaLib :: Common :: Loader.pm
#
# This is the ApolliaLib that loads the other ApolliaLib :: Common libraries.
#
# This module is a Loader module, which doesn't itself create any new
# functions, etc., but instead only loads other modules and makes things 
# from those modules available to any script which uses the following line:
#
# 	use ApolliaLib::Common::Loader;


use strict;
use warnings;


package ApolliaLib::Common::Loader;



################################################################
#
#                         Exported Stuff
#
# This bit specifies which parts of this module are made available to
# scripts that use the following line:
#
# 	use ApolliaLib::Common::Loader;
#
#
# I pasted much of the below from:
#
# 	http://perldoc.perl.org/perlmod.html


BEGIN 
{
	require Exporter;

	#set the version for version checking
	our $VERSION = 1.00;

	#Inherit from Exporter to export functions and variables
	our @ISA = qw(Exporter);

	#Functions and variables which are exported by default
	our @EXPORT = qw(
		$ApolliaLib_Settings 
		$ApolliaLib_Settings_say_we
		$ApolliaLib
		
		true  
		false  
		$dollar_sign  
		Escape_Quotes  
		Escape_Single_Quotes_for_Bash  
		Print_Array_Indentedly
		String_Contains_Space
		Pause_Until_User_Presses_Enter
		Get_Script_Ending_Type_from_Argv
		End_Script  
		
		$Launched_Script_Details  
		$Launched_Script
		
		Typical_Script_Launch
		Possibly_Open_New_Terminal_Window_and_Relaunch_Self  
		
		Get_Either_Dragdrop_Args_or_Config_Args
		Remove_Now_Unneeded_Clutter_from_ARGV
		
		Note_Successfully_Completed_Script
		Note_of_Successful_Completion_Exists
		
		Ask_or_Dont_Ask_Whether_To_Do_Action
		Ask_User_to_Choose_from_Numbered_Menu
		Ask_or_Dont_Ask_User_to_Choose_from_Numbered_Menu
		Ask_To_Do_Action
		Launch_Other_Script
		Launch_Many_Relativepath_Scripts
		String_Consists_of_Nothing_but_Single_Quotes_or_Spaces
		necho
		Print_Octal_Value_Properly
		Get_Random_Item_from_Array_
		Add_Slash_If_Needed_To_
		Get_Filename_from_Path
		Get_Dragdrop_Args
		
		Get_File_Suffix_from_Path
		Get_First_Named_Dir_in_Path
		Get_Second_Named_Dir_in_Path
		Print_Hash_Indentedly

		Display_and_Do_Command_Line
		Display_and_Do_Command_Line_and_Result
		Get_Parentdirs_of_Path
		End_Script_Immediately

		);

	#Functions and variables which can be optionally exported
	our @EXPORT_OK = qw(
	
		);
}



#
#                End of Exported Stuff section. 
#
#
################################################################




################################################################
#
#	This module is a Loader module, which only loads other modules, 
#	and doesn't itself create any new functions, etc.
#


use ApolliaLib::Common::Settings;
	# Creates the $ApolliaLib_Settings object, which gives you some
	# custom control over what the ApolliaLib::Common modules do.
	# Also has related functions.


use ApolliaLib::Common::Arguments;
	# Deals with arguments sent to a script via the command line or drag & drop.
	# (See also: Dragdrop.pm)


use ApolliaLib::Common::Misc;
	# Things that I (Apollia) commonly use in my Perl scripts.
	#
	# Miscellaneous basics like true/false constants, a dollar sign variable, 
	# and various common functions like string escapers.  And other things 
	# which perhaps it might make sense to put elsewhere.
	

use ApolliaLib::Common::Path_String_Handling;
	# Stuff which deals with filepath and folderpath strings.
	
	
use ApolliaLib::Common::Script_Details;	
	# Creates the $Launched_Script_Details object, which contains
	# details about the script that was launched.  Also has related
	# functions.
	
	
use ApolliaLib::Common::Relaunch_in_Terminal_Window;
	# Stuff related to relaunching scripts in a terminal window so you can see output.


use ApolliaLib::Common::Dragdrop;
	# Not every script uses drag & drop, but it's good to be able 
	# to easily make any script capable of ignoring drag & drop,
	# to make it easier to avoid scripts being accidentally run.
	#
	# So, that's why I made drag & drop-related stuff part of the 
	# ApolliaLib :: Common modules.
	#
	# There's actually no certain way for a script to distinguish between 
	# drag & drop arguments and arguments typed on a command line.
	#
	# (See also: Arguments.pm)


use ApolliaLib::Common::End_Script;
	# Stuff related to ending a script.
	
	
use ApolliaLib::Common::Completed_Script;
	# Stuff related to creating and reading notes in /tmp/ which state that a script 
	# has been successfully completed.
	
	
use ApolliaLib::Common::User_Prompted_Actions;
	# Stuff related to asking the user what to do.
	
use ApolliaLib::Common::X_Dialog;
	# 19:57 05/12/2014.  For now, just two functions which display either
	# an Xmessage dialog or an Xdialog dialog.

	
use ApolliaLib::Common::Branching;
	# Stuff related to launching other scripts.



1;