# ApolliaLib :: Common :: Path_String_Handling.pm
#
# Stuff which deals with filepath and folderpath strings.


use strict;
use warnings;


package ApolliaLib::Common::Path_String_Handling;



#############################################################################
#
#                         Exported Stuff
#
# This bit specifies which parts of this module are made available to
# scripts that use the following line:
#
# 	use ApolliaLib::Common::Path_String_Handling;
#
#
# I pasted much of the below from:
#
# 	http://perldoc.perl.org/perlmod.html


BEGIN 
{
	require Exporter;

	#set the version for version checking
	our $VERSION = 1.00;

	#Inherit from Exporter to export functions and variables
	our @ISA = qw(Exporter);

	#Functions and variables which are exported by default
	our @EXPORT = qw(
	
		Convert_Questionable_Chars_to_Dash
		Convert_Questionable_Chars_Except_Slash_to_Dash
		
		Get_First_Named_Dir_in_Path
		Get_Second_Named_Dir_in_Path
		Get_Parentdirs_of_Path
		Get_Filename_from_Path
		Get_File_Suffix_from_Path
		
		
		Get_Filename_and_Parentdirs
		Get_Filename__Parentdirs__and_File_Suffix
		
		
		);

	#Functions and variables which can be optionally exported
	our @EXPORT_OK = qw(
	
		);
}


#
#
#                End of Exported Stuff section. 
#
#
################################################################



##################################################
#
#	Needed imports from other modules
#
#

use ApolliaLib::Common::Misc ( 
	"true",
	"false",
	"Escape_Single_Quotes_for_Bash",
	"Print_Array_Indentedly"
	);

use File::Basename qw( fileparse );

#
#
#	End of Needed imports from other modules
#
##################################################




################################################################
#
#	Start of stuff unique to this module
#
#



sub Convert_Questionable_Chars_to_Dash
{
	my $string=$_[0];
	
	#----
	my $result;
	
	
	
	##############################
	
	
	if (defined $string)
	{
	
		$string=~s/[^A-Za-z0-9.,!'~ _-]/-/g;
	}
	
	$result=$string;
	
	
	#!!!!
	return $result;
}



sub Convert_Questionable_Chars_Except_Slash_to_Dash
{
	my $string=$_[0];
	
	#----
	my $result;
	
	
	
	##############################
	
	
	
	$string=~s{[^A-Za-z0-9.,!'~ _-]^/}/-/g;
	
	$result=$string;
	
	
	#!!!!
	return $result;
}



sub Get_First_Named_Dir_in_Path
{
	my $path_string=$_[0];
	
	
	#----
	my @split_pathstring;
	
	#@@@@
	
	
	
	##############################
	
	@split_pathstring=split("/", $path_string);
	
	return $split_pathstring[1];	
		
}



sub Get_Second_Named_Dir_in_Path
{
	my $path_string=$_[0];
	
	
	#----
	my @split_pathstring;
	
	#@@@@
	
	
	
	##############################
	
	@split_pathstring=split("/", $path_string);
	
	return $split_pathstring[2];	
		
}



sub Get_Parentdirs_of_Path
{	
	my $full_path=$_[0];


	#----
	my $filename;
	my $parentdirs;



	##############################
	
	($filename, $parentdirs) = fileparse($full_path );
	
	
	#!!!!
	return $parentdirs;
}



sub Get_Filename_from_Path
{	# Provides the full filename with the suffix/extension after the dot (if there is one).

	my $full_path=$_[0];
	
	
	#----
	my $filename;
	my $parentdirs;


	
	##############################
	
	($filename, $parentdirs) = fileparse($full_path );
	

	#!!!!
	return $filename;
}



sub Get_File_Suffix_from_Path
{	
	my $full_path=$_[0];


	#----
	my $filename;
	my $parentdirs;
	my $suffix;



	##############################
	
	($filename, $parentdirs, $suffix) = fileparse($full_path, qr{\..*} );
	
	
	#!!!!
	return $suffix;
}



sub Get_Filename_and_Parentdirs
{	# 01:33 09/20/2014.  Filename here has both the filename and suffix.

	my $full_path=$_[0];

	
	#----
	my $filename;
	my $parentdirs;



	##############################
	
	($filename, $parentdirs) = fileparse($full_path );
	
	
	#!!!!
	return ($filename, $parentdirs);
}



sub Get_Filename__Parentdirs__and_File_Suffix
{	# 01:33 09/20/2014.  Filename here is filename without suffix.

	my $full_path=$_[0];


	#----
	my $filename;
	my $parentdirs;
	my $suffix;
	


	##############################
	
	($filename, $parentdirs, $suffix) = fileparse($full_path, qr{\..*} );
	
	
	#!!!!
	return ($filename, $parentdirs, $suffix);
	
}



1;