# ApolliaLib :: Common :: Settings.pm
#
# Creates the $ApolliaLib_Settings object, which gives you some
# custom control over what the ApolliaLib::Common modules do.
# Also has related functions.


use strict;
use warnings;


package ApolliaLib::Common::Settings;



################################################################
#
#                         Exported Stuff
#
# This bit specifies which parts of this module are made available to
# scripts that use the following line:
#
# 	use ApolliaLib::Common::Settings;
#
#
# I pasted much of the below from:
#
# 	http://perldoc.perl.org/perlmod.html


BEGIN 
{
	require Exporter;

	#set the version for version checking
	our $VERSION = 1.00;

	#Inherit from Exporter to export functions and variables
	our @ISA = qw(Exporter);

	#Functions and variables which are exported by default
	our @EXPORT = qw(  
		$ApolliaLib_Settings 
		$ApolliaLib_Settings_say_we
		$ApolliaLib
		);

	#Functions and variables which can be optionally exported
	our @EXPORT_OK = qw(
	
		);
}



#
#                End of Exported Stuff section. 
#
#
################################################################



##################################################
#
#	Needed imports from other modules
#


use ApolliaLib::Common::Misc ( 
	"true",
	"false",
	"Escape_Single_Quotes_for_Bash",
	"Print_Array_Indentedly",
	"necho"
	);

use ApolliaLib::Common::Script_Details ( 
	'$Launched_Script_Details'
	);
	
	
use ApolliaLib::Common::End_Script (
	"Get_Script_Ending_Type_from_Argv"
	);
	
use Object::Tiny (
	"terminal_window_title",
	"Current_ApolliaLib_Version",
	
	"should_launch_new_terminal_window",
	"should_print_arguments_given_to_script",
	
	"List_of_Recognized_Script_Endingtype_Args",
	"List_of_Recognized_Window_Managing_Args",
	"List_of_Recognized_Automaticness_Args",
	
	"should_be_capable_of_running_without_dragdrop_args",
	"should_accept_dragdrop_args",
	"num_of_seconds_before_autoclose",

	"script_ending_type",
	"should_use_separate_config_file",
	"name_of_separate_config_file",
	"escaped_name_of_separate_config_file",
	
	"path_to_separate_config_file",
	"escaped_path_to_separate_config_file",
	
	"should_abort_if_run_via_click",
	
	"should_use_script_title_in_terminal_window_title",
	"should_use_script_version_in_terminal_window_title",
	"should_use_script_version_in_announcement_of_script_running",
	
	"should_abort_if_private_script_is_run_from_a_public_location",
	"should_warn_if_private_script_is_run_from_a_public_location",
	
	"should_abort_if_ApolliaLib_version_not_specified",
	"should_warn_if_ApolliaLib_version_not_specified",
	
	"should_print_relaunch_command_in_launch_phase_2",
	
	"min_number_of_args_other_than_ignored_args",
	"max_number_of_args_other_than_ignored_args",
	"List_of_Non_ApolliaLib_Args_to_Ignore",
	
	"should_not_run_at_all_no_matter_what",
	"Start_of_Command_to_Launch_Terminal_Window",
	"script_ending_type_when_a_problem_is_found",
	"num_of_seconds_before_autoclose_when_a_problem_is_making_the_script_close_prematurely",
	"should_arbitrarily_abort_script_before_terminal_window_launch",

	"should_abort_script_if_run_with_no_arguments",
	"should_windowlessly_abort_if_wrong_quantity_of_nonignored_arguments_is_found",
	"should_windowlessly_abort_if_script_is_run_with_no_arguments",
	"should_windowedly_abort_if_script_is_run_with_no_arguments",
	"should_display_last_words_in_popup_xmessage_when_aborting",
	"should_display_last_words_in_popup_xdialog_when_aborting",
	"color_for_last_words_xmessage",
	"should_display_success_messages_in_popup_xdialog",
	"should_display_success_messages_in_popup_xmessage",
	
	"should_note_successfully_completed_script",
	"full_parentdirs_of_notes_of_successfully_completed_scripts",
	"should_abort_script_if_it_was_successfully_completed_before_this_run",
	"script_ending_divider_text",

	"full_parentdirs_for_new_user_home_directory",
	"should__if_script_was_successfully_completed_before_this_run__warn_and_ask_if_user_wants_to_continue",

	
	"should_do_things_automatically",
	"automaticness_setting",
	"working_dir_for_proc_daemon_perl_module",
	
	"fullpath_to_ApolliaLib_Temp_Workspace_for_Downloads",
	"fullpath_to_ApolliaLib_Temp_Workspace_for_Tarballs",
	
	
	"should_ask__when_branching__whether_to_run_script",
	"ApolliaLib_Version",
	
	"mountpoint_parentdir_of_truecrypt_volumes",
	"fullpath_to_landmark_template",
	"fullpath_to_diff_note_1",
	"fullpath_to_diff_note_2",
	"should_tell_wget_NOT_to_check_certificate",
	
	);
	
#use Data::Dumper;									


#
#	End of Needed imports from other modules
#
##################################################




################################################################
#
#	Start of stuff unique to this module
#



sub new
{
	my $class = shift;
	
	
	#----
	my $self;

	#@@@@
	$self = {};
	
	
	
	##############################
			
	bless $self, $class;
	
	
	#!!!!
	return $self;
}



sub Initialize
{	
	my $self = $_[0];


	#########################################################################
	#
	#   Default settings you can adjust if desired.  Just bear in mind it
	#	will affect every script that uses ApolliaLib!
	#

	my $ApolliaLib_Version="1";
										$self->{'ApolliaLib_Version'} = $ApolliaLib_Version;
		# Example: 1						
										
	my $Start_of_Command_to_Launch_Terminal_Window="rxvt -g 80x24 -rv -sb -sr -sl 99999 -title";
										$self->{'Start_of_Command_to_Launch_Terminal_Window'} = $Start_of_Command_to_Launch_Terminal_Window;
		# Default:
		#	rxvt -g 80x44 -rv -sb -sr -sl 99999 -title
		
	
	my $fullpath_to_landmark_template="/root/zzzzz-tmp/ApolliaLib templates/Landmark Template.sh";
										$self->{'fullpath_to_landmark_template'} = $fullpath_to_landmark_template;
	
	
	my $fullpath_to_diff_note_1="/root/zzzzz-tmp/ApolliaLib tmp workspace/diff_path_1.txt";
										$self->{'fullpath_to_diff_note_1'} = $fullpath_to_diff_note_1;
	
	my $fullpath_to_diff_note_2="/root/zzzzz-tmp/ApolliaLib tmp workspace/diff_path_2.txt";
										$self->{'fullpath_to_diff_note_2'} = $fullpath_to_diff_note_2;
	
	
	my $working_dir_for_proc_daemon_perl_module="/root/zzzzz-tmp/ApolliaLib tmp workspace/";
										$self->{'working_dir_for_proc_daemon_perl_module'} = $working_dir_for_proc_daemon_perl_module;
		# Default: 
		#	/root/zzzzz-tmp/ApolliaLib tmp workspace/
	
	
	my $fullpath_to_ApolliaLib_Temp_Workspace_for_Downloads="/root/zzzzz-tmp/ApolliaLib tmp download workspace/";
										$self->{'fullpath_to_ApolliaLib_Temp_Workspace_for_Downloads'} = $fullpath_to_ApolliaLib_Temp_Workspace_for_Downloads;
										
	my $fullpath_to_ApolliaLib_Temp_Workspace_for_Tarballs="/root/zzzzz-tmp/ApolliaLib tmp tarball workspace/";
										$self->{'fullpath_to_ApolliaLib_Temp_Workspace_for_Tarballs'} = $fullpath_to_ApolliaLib_Temp_Workspace_for_Tarballs;						
	my $should_tell_wget_NOT_to_check_certificate=false;
										$self->{'should_tell_wget_NOT_to_check_certificate'} = $should_tell_wget_NOT_to_check_certificate;
										
										
										
	my $should_ask__when_branching__whether_to_run_script=false;
										$self->{'should_ask__when_branching__whether_to_run_script'} = $should_ask__when_branching__whether_to_run_script;
		# true|false						
										
	my $terminal_window_title="Not Yet Specified";
										$self->{'terminal_window_title'} = $terminal_window_title;
		# This gets set later.
		
	
	my $should_abort_script_if_it_was_successfully_completed_before_this_run=false;
										$self->{'should_abort_script_if_it_was_successfully_completed_before_this_run'} = $should_abort_script_if_it_was_successfully_completed_before_this_run;
		# true|false	
		
				
	my $should_display_last_words_in_popup_xmessage_when_aborting=false;
										$self->{'should_display_last_words_in_popup_xmessage_when_aborting'} = $should_display_last_words_in_popup_xmessage_when_aborting;
		# true|false	
		
						
	my $should_display_last_words_in_popup_xdialog_when_aborting=false;
										$self->{'should_display_last_words_in_popup_xdialog_when_aborting'} = $should_display_last_words_in_popup_xdialog_when_aborting;
		# true|false	
		
	
	#my $script_ending_divider_text="\n\n\n\n**********";
	my $script_ending_divider_text="\n\n\n\n------------------------------------------------------------";
	#my $script_ending_divider_text="\n\n\n\n---";
										$self->{'script_ending_divider_text'} = $script_ending_divider_text;
		# Default: 
		#	\n\n\n\n---
							

	#my $should_display_success_messages_in_popup_xdialog=true;
									#	$self->{'should_display_success_messages_in_popup_xdialog'} = $should_display_success_messages_in_popup_xdialog;
	
	
	#my $should_display_success_messages_in_popup_xmessage=false;
									#	$self->{'should_display_success_messages_in_popup_xmessage'} = $should_display_success_messages_in_popup_xmessage;	

										
										
	my $should_launch_new_terminal_window=true;
										$self->{'should_launch_new_terminal_window'} = $should_launch_new_terminal_window;
		# true|false
		

	my $should_windowlessly_abort_if_wrong_quantity_of_nonignored_arguments_is_found=false;
										$self->{'should_windowlessly_abort_if_wrong_quantity_of_nonignored_arguments_is_found'} = $should_windowlessly_abort_if_wrong_quantity_of_nonignored_arguments_is_found;
		# true|false
	
	
										
	my $min_number_of_args_other_than_ignored_args=0;
										$self->{'min_number_of_args_other_than_ignored_args'} = $min_number_of_args_other_than_ignored_args;
		# Default: 0
		# No minimum number of args.

		
	my $max_number_of_args_other_than_ignored_args=0;
										$self->{'max_number_of_args_other_than_ignored_args'} = $max_number_of_args_other_than_ignored_args;
		# Default: 0
		# Args beyond 0 are forbidden.

		
	my $should_arbitrarily_abort_script_before_terminal_window_launch=false;
										$self->{'should_arbitrarily_abort_script_before_terminal_window_launch'} = $should_arbitrarily_abort_script_before_terminal_window_launch;
		# true|false

	
										
	my $script_ending_type="Wait_For_Return_At_Scripts_End";
	
										$self->{'script_ending_type'} = $script_ending_type;
		# Possible values: 
		#
		# Wait_For_Return_At_Scripts_End
		# Autoclose_After_X_Seconds
		# Dont_Wait_For_Return_At_Scripts_End



	my $num_of_seconds_before_autoclose=5;
										$self->{'num_of_seconds_before_autoclose'} = $num_of_seconds_before_autoclose;
										
		
	my $script_ending_type_when_a_problem_is_found="Wait_For_Return_At_Scripts_End";
										$self->{'script_ending_type_when_a_problem_is_found'} = $script_ending_type_when_a_problem_is_found;
		# Possible values: 
		#
		# Wait_For_Return_At_Scripts_End
		# Autoclose_After_X_Seconds
		# Dont_Wait_For_Return_At_Scripts_End
		
				
	my $num_of_seconds_before_autoclose_when_a_problem_is_making_the_script_close_prematurely=3;
										$self->{'num_of_seconds_before_autoclose_when_a_problem_is_making_the_script_close_prematurely'} = $num_of_seconds_before_autoclose_when_a_problem_is_making_the_script_close_prematurely;
		# Default: 3 
		# (seconds)
	
	
	my $should_do_things_automatically=false;
										$self->{'should_do_things_automatically'} = $should_do_things_automatically;
		# true|false
	
	
	my $should_print_relaunch_command_in_launch_phase_2=true;
										$self->{'should_print_relaunch_command_in_launch_phase_2'} = $should_print_relaunch_command_in_launch_phase_2;
		# true|false

		
	# 11:19:38 8/15/2014.  I don't think I have separate config file stuff working yet.
	my $should_use_separate_config_file=false;
	
										$self->{'should_use_separate_config_file'} = $should_use_separate_config_file;
		# true|false
	

	my $should_use_script_title_in_terminal_window_title=true;
										$self->{'should_use_script_title_in_terminal_window_title'} = $should_use_script_title_in_terminal_window_title;
		# true|false
		
					
	my $should_use_script_version_in_terminal_window_title=false;
										$self->{'should_use_script_version_in_terminal_window_title'} = $should_use_script_version_in_terminal_window_title;
		# true|false
		
	
	my $should_use_script_version_in_announcement_of_script_running=false;
						$self->{'should_use_script_version_in_announcement_of_script_running'} = $should_use_script_version_in_announcement_of_script_running;
	
		# true|false
										
	my $should_print_arguments_given_to_script=true;
									$self->{'should_print_arguments_given_to_script'} = $should_print_arguments_given_to_script;

		# true|false
		

	#---------------------------------------------------------------------------
	#
	# 	Settings related to privacy settings
	#
	#						
	
	my $should_abort_if_private_script_is_run_from_a_public_location=true;
									$self->{'should_abort_if_private_script_is_run_from_a_public_location'} = $should_abort_if_private_script_is_run_from_a_public_location;
		# true|false
		#
		# Just silently aborts, unless it's supposed to warn, 
		# as stated in $should_warn_if_private_script_is_run_from_a_public_location.
		
	
	my $should_warn_if_private_script_is_run_from_a_public_location=true;
									$self->{'should_warn_if_private_script_is_run_from_a_public_location'} = $should_warn_if_private_script_is_run_from_a_public_location;
		
		# true|false
		#
		# Warns but doesn't abort, unless
		# $should_abort_if_private_script_is_run_from_a_public_location is true.
	
	
	
	my $should_abort_if_ApolliaLib_version_not_specified=false;
									$self->{'should_abort_if_ApolliaLib_version_not_specified'} = $should_abort_if_ApolliaLib_version_not_specified;

	my $should_warn_if_ApolliaLib_version_not_specified=true;
									$self->{'should_warn_if_ApolliaLib_version_not_specified'} = $should_warn_if_ApolliaLib_version_not_specified;

	
	
	
	#---------------------------------------------------------------------------
	#
	# 	Settings related to Notes of Successfully Completed Scripts
	#
	#

	my $full_parentdirs_of_notes_of_successfully_completed_scripts
		= "/root/zzzzz-tmp/ApolliaLib Notes of Successfully Completed Scripts";
		
										$self->{'full_parentdirs_of_notes_of_successfully_completed_scripts'} = $full_parentdirs_of_notes_of_successfully_completed_scripts;
	
		# Default:
		#	/root/zzzzz-tmp/ApolliaLib Notes of Successfully Completed Scripts
		#
		# Don't put a slash at the end.

										
	my $should__if_script_was_successfully_completed_before_this_run__warn_and_ask_if_user_wants_to_continue
		= false;
										$self->{'should__if_script_was_successfully_completed_before_this_run__warn_and_ask_if_user_wants_to_continue'} = $should__if_script_was_successfully_completed_before_this_run__warn_and_ask_if_user_wants_to_continue;
	
		#true|false
	


	my $should_note_successfully_completed_script=false;
										$self->{'should_note_successfully_completed_script'} = $should_note_successfully_completed_script;
		
		#true|false



my $full_parentdirs_for_new_user_home_directory = 
		"/root/aphome/";
										$self->{'full_parentdirs_for_new_user_home_directory'} = $full_parentdirs_for_new_user_home_directory;
		# Default: /root/aphome/
		#
		# Should have a slash at the end.
		
		
#
#   End of Default settings you can adjust if desired
#
#########################################################################
	



	
#########################################################################
#
#   Settings you shouldn't mess with, which are calculated or observed, 
#	or which are set later inside individual scripts.
#


	#---------------------------------------------------------------------------
	#
	# 	Settings you shouldn't mess with:
	#
	#	Related to automaticness
	#
	#

	my $automaticness_setting;
	
	if ($should_do_things_automatically)
	{	$automaticness_setting="Do_Things_Automatically";
	}
	else
	{	$automaticness_setting="Dont_Do_Things_Automatically";
	}
	
	$self->{'automaticness_setting'}=$automaticness_setting;



	#---------------------------------------------------------------------------
	#
	# 	Settings you shouldn't mess with:
	#
	#	Arrays of recognized arg names
	#
	#
		
	my @List_of_Non_ApolliaLib_Args_to_Ignore;
		# This is so the Puppy Setup Kit can make the ApolliaLib not count the
		# "Install_Without_Asking" arg, etc.
	
	
	my @List_of_Recognized_Window_Managing_Args=(
		"Terminal_Window_Was_Opened_Automatically_by_ApolliaLib", 
		"Dont_Open_Window", 
		"Do_Open_Window"
	);
										@{$self->{'List_of_Recognized_Window_Managing_Args'} } = @List_of_Recognized_Window_Managing_Args;


	my @List_of_Recognized_Script_Endingtype_Args=(
		"Wait_For_Return_At_Scripts_End", 
		"Autoclose_After_X_Seconds", 
		"Dont_Wait_For_Return_At_Scripts_End"
	);
										@{$self->{'List_of_Recognized_Script_Endingtype_Args'} } = @List_of_Recognized_Script_Endingtype_Args;
										
	
	my @List_of_Recognized_Automaticness_Args=(
		"Do_Things_Automatically", 
		"Dont_Do_Things_Automatically"
	);
										@{$self->{'List_of_Recognized_Automaticness_Args'} } = @List_of_Recognized_Automaticness_Args;
		
	
	return $self;
}



#
#	End of Initialize() method
#	
#	of 
#	
#	$ApolliaLib_Settings
#
#
####################################################






############################################################################
#
#
# 	Boring setter functions
#
#



sub Should_Ask__When_Branching__Whether_to_Run_Script
{
	my $self = $_[0];
	
	
	
	##############################
	
	$self->{'should_ask__when_branching__whether_to_run_script'}=true;
}



sub Shouldnt_Launch_New_Terminal_Window
{	
	my $self = $_[0];



	##############################
	
	$self->{'should_launch_new_terminal_window'}=false;
	
}



#-------------------------------------------------
#
#
# 	Boring setter functions related to separate config file
#
#



sub Set_Name_of_Separate_Config_File
{	my $self = $_[0];
	my $string = $_[1];
	
	
	
	##############################

	$self->{'name_of_separate_config_file'}=$string;
	$self->{'escaped_name_of_separate_config_file'}=Escape_Single_Quotes_for_Bash($string);
	
	#print $self->name_of_separate_config_file;
	#print $self-> escaped_name_of_separate_config_file;
}



sub Set_Path_to_Separate_Config_File
{	my $self = $_[0];
	my $string = $_[1];

	
	
	##############################
		
	$self->{'path_to_separate_config_file'}=$string;
	$self->{'escaped_path_to_separate_config_file'}=Escape_Single_Quotes_for_Bash($string);
	
	#print $self->path_to_separate_config_file;
	#print $self-> escaped_path_to_separate_config_file;
	
}


#
#
# 	End of Boring setter functions related to separate config file
#
#
#-------------------------------------------------



#-------------------------------------------------
#
#
# 	Boring setter functions related to script completion
#
#


sub If_Script_Has_Been_Successfully_Completed_Before__Warn_and_Ask_if_User_Wants_to_Continue
{
	my $self = $_[0];
	
	
	
	##############################
	
	$self->{'should__if_script_was_successfully_completed_before_this_run__warn_and_ask_if_user_wants_to_continue'}=true;
}



sub Dont_Rerun_If_Script_Has_Been_Successfully_Completed_Before
{	my $self = $_[0];



	##############################
	
	$self->{'should_abort_script_if_it_was_successfully_completed_before_this_run'}=true;
}



sub Should_Note_Successful_Script_Completion
{
	my $self = $_[0];

	
	
	##############################
	
	$self->{'should_note_successfully_completed_script'}=true;
}


#
#
# 	End of Boring setter functions related to script completion
#
#
#-------------------------------------------------



#
#
# 	End of Boring setter functions
#
#
############################################################################




############################################################################
#
#
# 	Function related to custom args to ignore
#
#


sub Add_Non_ApolliaLib_Args_to_Ignore
{	my $self = $_[0];
	my $ref_to_list_of_new_args_to_ignore=$_[1];
	
	
	#----
	my @List_of_New_Args_to_Ignore;
	
	#@@@@
	@List_of_New_Args_to_Ignore=@$ref_to_list_of_new_args_to_ignore;
	

	
	##############################
	
	push ( @{$self->{'List_of_Non_ApolliaLib_Args_to_Ignore'} }, 
	@List_of_New_Args_to_Ignore
	);
	
	#print "\n\nThe current List_of_Non_ApolliaLib_Args_to_Ignore - ";
	#Print_Array_Indentedly( @{$self->{'List_of_Non_ApolliaLib_Args_to_Ignore'} } );
}


#
#
# 	End of Function related to custom args to ignore
#
#
############################################################################



sub Make_Terminal_Window_Title
{

	my $self = $_[0];
	
	
	#----
	my $terminal_window_title;
	
	#@@@@
	$terminal_window_title="";



	##############################
	
	if ($self->should_use_script_title_in_terminal_window_title)
	{	$terminal_window_title=$Launched_Script_Details->script_title;
	}
	else
	{	$terminal_window_title="ApolliaLib-Based Perl Script";
	}
	

	if ($self->should_use_script_version_in_terminal_window_title)
	{	

		print $self->should_use_script_version_in_terminal_window_title;
		
		$terminal_window_title = $terminal_window_title . " - Version " . $Launched_Script_Details->script_version;
	}



	# No need to escape it now - the launcher function handles that later.
	
	$self->{'terminal_window_title'}=$terminal_window_title;
	
	return true;
}



#---------------------------------------------------------------------------
#
#	Now the $ApolliaLib_Settings object actually gets created, 
#	but no settings are set yet.
#
#

our $ApolliaLib_Settings;

$ApolliaLib_Settings = ApolliaLib::Common::Settings->new();



#---------------------------------------------------------------------------
#
# 	Aliases, equivalent to $ApolliaLib_Settings
#
#

our $ApolliaLib;
our $ApolliaLib_Settings_say_we; 	# Makes if statements very readable.

$ApolliaLib = $ApolliaLib_Settings;
$ApolliaLib_Settings_say_we = $ApolliaLib_Settings;



#---------------------------------------------------------------------------
#
#	Now, the $ApolliaLib_Settings get set to their defaults.
#
#	The defaults don't suit every script, but that's OK, because 
#	individual scripts can (and often should) adjust the defaults.


$ApolliaLib_Settings->Initialize();



1;