# ApolliaLib :: Common :: User_Prompted_Actions.pm
#
# Stuff related to asking the user what to do.


use strict;
use warnings;


package ApolliaLib::Common::User_Prompted_Actions;



################################################################
#
#                         Exported Stuff
#
# This bit specifies which parts of this module are made available to
# scripts that use the following line:
#
# 	use ApolliaLib::Common::User_Prompted_Actions;
#
#
# I pasted much of the below from:
#
# 	http://perldoc.perl.org/perlmod.html


BEGIN 
{
	require Exporter;

	#set the version for version checking
	our $VERSION = 1.00;

	#Inherit from Exporter to export functions and variables
	our @ISA = qw(Exporter);

	#Functions and variables which are exported by default
	our @EXPORT = qw(  

		Ask_or_Dont_Ask_Whether_To_Do_Action
		Ask_User_to_Choose_from_Numbered_Menu
		Ask_or_Dont_Ask_User_to_Choose_from_Numbered_Menu
		Ask_To_Do_Action
		Do_Action_Or_Not
		More_Safely_Get_STDIN
		
		Pause_Until_User_Presses_Enter	
		);

	#Functions and variables which can be optionally exported
	our @EXPORT_OK = qw(

		
		);
}



#
#                End of Exported Stuff section. 
#
#
################################################################



##################################################
#
#	Needed imports from other modules
#


use ApolliaLib::Common::Misc qw ( 
	true 
	false 
	Escape_Single_Quotes_for_Bash	
	is_integer
	Get_Arrin_of_String_in_Array
	);


use ApolliaLib::Common::End_Script qw ( 
	
	End_Script_and_Wait_for_Return
	End_Script_For_Problem_Using_Whatever_Ending_Type_The_Settings_Specify
	End_Script_Immediately
	Immediately_Abort_if_Variable_is_Undefined
	);

use ApolliaLib::Common::X_Dialog (
	"Display_Xdialog"
);


#
#	End of Needed imports from other modules
#
##################################################




################################################################
#
#	Start of stuff unique to this module
#



sub Pause_Until_User_Presses_Enter
{	
	my $wait_for_return;


	
	##############################
	
	print "\n\n\n\nPlease press Return (or Enter) to contine running this script.\n";
	
	$wait_for_return = More_Safely_Get_STDIN();

}



sub More_Safely_Get_STDIN
{
	# 13:45 08/16/2014.  An important safeguard to protect against a huge 
	# memory leak if the script is run without a terminal window.
	#
	# This function makes it so the script immediately aborts if the value 
	# from STDIN is found to be undefined.
	#
	# If this isn't done, then, loops that are supposed to read user
	# input are unable to finish when there's no way for the user to type
	# input (such as when the script is run without a terminal window).
	# That can result in an infinite loop which can drain your RAM 
	# all the way down to 0.
	#
	# Surprisingly, my Lucid Puppy 5.2.8 version 004 kept running when I ran
	# into this glitch a few times before figuring out what was happening and
	# how to fix it.  However, one of my VirtualBoxes crashed for lack of enough memory.  
	# (Luckily, even virtual Windows XP is quite resilient.)  
	#
	# And restarting the X server didn't  bring back my Puppy system's memory, 
	# and changed my window manager to the default,  and taskbar widgets and 
	# the clock wouldn't even appear.  Rebooting was the only way I knew to fix it.
	#
	#
	# However, if you stop a leaky Perl script fast enough by killing its
	# process, you might have enough memory remaining that you won't have
	# to reboot your entire computer.  If you have enough memory remaining, 
	# just restarting the X server should bring back the leaked memory.
	#
	# In Lucid Puppy 5.2.8 version 004, you can kill processes using Pprocess.
	# 
	# 	* Type "pprocess" (without quotes) in a terminal window.
	#
	#	* Then type "perl" into the search.
	#
	#	* Highlight your leaky Perl script.
	#
	# 	* Then click the big "End process" button.
	
	
	
	#----
	my $user_command;
	
	
	
	##############################
	
	$user_command = <STDIN>;

	Immediately_Abort_if_Variable_is_Undefined($user_command);
	
	
	#!!!!
	return $user_command;
}



sub Do_Action_Or_Not
{

	my $do_action=$_[0];
	my $action_announcement=$_[1];
	my $action=$_[2];



	##############################

	if ($do_action == false)
	{
		print "\nNot $action_announcement\n\n";
		
		
		#!!!!
		return false;
	}
	if ($do_action == true)
	{
		print "\n$action_announcement";
		print "\n\nCommand line being executed: 

# $action\n\n";
		
		system("$action");
		
		
		#!!!!
		return true;
	}
}



sub Ask_or_Dont_Ask_User_to_Choose_from_Numbered_Menu
{	
	my $menu_header=$_[0];
	my $automatically_or_not=$_[1];
	my $ref_to_numbered_menu_array=$_[2];
	my $default_item_text=$_[3];
	my $should_equate_enter_to_default_item=$_[4];
	
	
	#----
	my $number_chosen;
	my $number_chosen_for_display;
	my @numbered_menu;
	
	#@@@@
	@numbered_menu = @$ref_to_numbered_menu_array;
	
	if ( !defined $should_equate_enter_to_default_item )
	{	$should_equate_enter_to_default_item=false;
	}
	
	
	
	##############################
	
	if ($automatically_or_not eq false)
	{
		$number_chosen=Ask_User_to_Choose_from_Numbered_Menu( $menu_header, \@numbered_menu, $default_item_text, $should_equate_enter_to_default_item );
	}
	else
	{
		$number_chosen=Get_Arrin_of_String_in_Array($default_item_text, \@numbered_menu);
		$number_chosen_for_display=$number_chosen+1;
		
		print "Automatically choosing from this menu...";
		
		Print_Numbered_Menu(\@numbered_menu);
		
		print "\n\n\nAutomatically choosing default:";
		print "\n\n[$number_chosen_for_display] - $default_item_text";
	}
	
	
	#!!!!
	return $number_chosen;	
}



sub Print_Numbered_Menu
{	
	my $ref_to_numbered_menu_array=$_[0];
	
	
	#----
	my @numbered_menu;
	my $current_number;
	
	#@@@@
	@numbered_menu = @$ref_to_numbered_menu_array;
	$current_number=1;
	
	
	
	##############################
		
	for (@numbered_menu)
	{	
		print "\n\n[$current_number] - $_";
		
		$current_number++;
	}
}



sub Ask_User_to_Choose_from_Numbered_Menu
{	my $menu_header=$_[0];
	my $ref_to_numbered_menu_array=$_[1];
	my $default_item_text=$_[2];
	my $should_equate_enter_to_default_number=$_[3];
	
	my $should_not_print_skip_this_step_text=$_[4];
	
	
	#----
	my $count_of_numbered_menu_items;
	
	my $default_number;
	my $default_number_for_display;
	
	my $user_command;
	my $command_not_understood;
	
	my $number_chosen;
	
	my $selected_menu_item;
	
	my @numbered_menu;
	
	#@@@@
	@numbered_menu = @$ref_to_numbered_menu_array;
		
	$count_of_numbered_menu_items=@numbered_menu;
	
	if ( !defined $should_equate_enter_to_default_number )
	{	$should_equate_enter_to_default_number=false;
	}
	


	##############################	
	
	$default_number=Get_Arrin_of_String_in_Array($default_item_text, \@numbered_menu);
	

	$default_number_for_display=$default_number+1;
	
	$command_not_understood=true;
	
	print "\n\nMenu Selection: $menu_header\n";
	
	while ($command_not_understood)
	{
		print "\n\nPlease type one of the below numbers, and press Return.";
		
		if ($should_equate_enter_to_default_number)
		{	
			
			print "\n\nOr, press only Return to use the default: [$default_number_for_display] $default_item_text";
		}
		
		print "\n\nOr, type 'q' to quit";
		
		if (!$should_not_print_skip_this_step_text)
		{	print ", or 's' to skip this step.";
		}
		else
		{	print ".";
		}
		Print_Numbered_Menu(\@numbered_menu);
		
		print "\n\n> ";
				
		$user_command = More_Safely_Get_STDIN();

		chomp($user_command);
		
		if ( is_integer($user_command) )
		{	
			
			if ( $user_command > $count_of_numbered_menu_items || $user_command < 0 )
			{	print "\n\nPlease choose a number from the menu.";
				next;
			}
			else
			{	
				$number_chosen=$user_command-1;
				
				$selected_menu_item=$numbered_menu[$number_chosen];
				
				print "\n\nYou chose: [$user_command] " . $selected_menu_item;
				
				
				#!!!!
				return $selected_menu_item;
				
			}
		}
		elsif ($user_command eq "")
		{	if ($should_equate_enter_to_default_number)
			{	$number_chosen=$default_number;
				$selected_menu_item=$numbered_menu[$number_chosen];
				
				print "\n\nChose default: [$default_number_for_display] " . $selected_menu_item;
				
				
				#!!!!
				return $selected_menu_item;
				
			}
			else
			{	print "\n\nPlease choose a number from the menu.";
				next;
			}
		}
		elsif ($user_command eq 'q' || $user_command eq 'quit' || $user_command eq 'exit')
		{	
		
			#!!!!
			End_Script_Immediately();
		}
		elsif ($user_command eq 's' || $user_command eq 'skip')
		{
			if ($should_not_print_skip_this_step_text)
			{	print "\n\nCan't skip this selection!";
			}
			else
			{
				print "\n\nSkipping the above...\n\n";
				
				
				#!!!!
				return "Selection Skipped";
				
			}
		}
		elsif ($user_command eq 'help')
		{	print "\nPlease type a command!";
		}
		else
		{	print "\nCommand not understood!\n";
		}
	}
}



sub Ask_or_Dont_Ask_Whether_To_Do_Action
{
	my $automatically_or_not=$_[0];
	my $action_phrase=$_[1];
	
	
	#----
	my $should_do_action_or_not;
	
	#@@@@
	$should_do_action_or_not=false;
	
	
	
	##############################
	
	if ($automatically_or_not eq false)
	{	
		$should_do_action_or_not=Ask_To_Do_Action($action_phrase);
	}
	elsif ($automatically_or_not eq true)
	{	
		$should_do_action_or_not=true;
	}
	
	
	#!!!!
	return $should_do_action_or_not;
}



sub Ask_To_Do_Action
{	my $action_phrase=$_[0];
	my $dont_display_the_skip_this_action_text=$_[1];
	
	
	#----
	my $user_command;
	my $command_not_understood;	
	my $unchomped_user_command;
	
	#@@@@
	$command_not_understood=true;
	
	
	
	##############################
	
	while ($command_not_understood)
	{
		print "\n\nPress Return to $action_phrase.  Or type 'q' (without quotes) to quit";
		
		if ( !defined $dont_display_the_skip_this_action_text)
		{	print ", or 's' to skip this action.";
		}
		else
		{	print ".";
		}
	
		
		print "\n\n> ";
	
		$user_command = More_Safely_Get_STDIN();
		
		$unchomped_user_command = $user_command;
		
		chomp($user_command);
	
		if ($user_command eq 'q' || $user_command eq 'quit' || $user_command eq 'exit')
		{	
			
			#!!!!
			End_Script_Immediately();
		}
		elsif ($user_command eq 's' || $user_command eq 'skip')
		{	
			if ($dont_display_the_skip_this_action_text)
			{	print "\n\nCan't skip this question!";
			}
			else 
			{
				print "\n\nSkipping the above...\n\n";
				
				
				#!!!!
				return false;
			}
						
		}
		elsif ( $user_command eq "")
		{	if ($unchomped_user_command eq "\n")
			{	
				
				#!!!!
				return true;
			}
			
			
			#!!!!
			return false;
			
		}
		elsif ( $user_command eq "help")
		{	print "\nPlease type a command!";
		}
		else
		{	print "\nCommand not understood!\n";
		}
	}
	
	
	#!!!!
	return false;
}



1;