# ApolliaLib :: Disk_Navigation :: Signposts.pm
#
# Stuff related to creating "signposts" on disks like "Welcome to [diskname!"
# and which state the disk classification (like "rumino").


use strict;
use warnings;


package ApolliaLib::Disk_Navigation::Signposts;



################################################################
#
#                         Exported Stuff
#
# This bit specifies which parts of this module are made available to
# scripts that use the following line:
#
# 	use ApolliaLib::Disk_Navigation::Signposts;
#
#
# I pasted much of the below from:
#
# 	http://perldoc.perl.org/perlmod.html


BEGIN 
{
	require Exporter;

	#set the version for version checking
	our $VERSION = 1.00;

	#Inherit from Exporter to export functions and variables
	our @ISA = qw(Exporter);

	#Functions and variables which are exported by default
	our @EXPORT = qw(  
		Make_Welcome_to_Disk_Signpost
		Make_Disk_Classification_Signpost
		Copy_Landmark_Template_to_Folder
		Make_Disk_Contents_Signpost
		);

	#Functions and variables which can be optionally exported
	our @EXPORT_OK = qw(
	
		);
}



#
#                End of Exported Stuff section. 
#
#
################################################################



##################################################
#
#	Needed imports from other modules
#

use ApolliaLib::Common::Misc qw (
	true
	false
	necho
	
	);

use ApolliaLib::File_Handling::Text_File_Creation (
	"Create_Text_File_Whose_Filename_and_Contents_are_the_Same"
	);

use ApolliaLib::File_or_Folder_Handling::Single_Item (
	"Copy_Item"
	);

use ApolliaLib::Common::Settings qw ( 
	$ApolliaLib_Settings
	$ApolliaLib_Settings_say_we
	);
	
#
#	End of Needed imports from other modules
#
##################################################




################################################################
#
#	Start of stuff unique to this module
#



sub Make_Disk_Contents_Signpost
{
	my $signpost_text=$_[0];
	my $full_parentdirs_of_signpost=$_[1];
	
	
	#----
	my $filename;
	
	#@@@@
	$filename="~Contents - " . $signpost_text;
	
	
	
	##############################
	
	Create_Text_File_Whose_Filename_and_Contents_are_the_Same($full_parentdirs_of_signpost, $filename);
	
}



sub Copy_Landmark_Template_to_Folder
{
	my $full_parentdirs_for_landmark_template=$_[0];
	
	
	#----
	my $fullpath_to_landmark_template;
	
	#@@@@
	$fullpath_to_landmark_template=$ApolliaLib_Settings->fullpath_to_landmark_template;
	
	
	
	##############################
	
	Copy_Item($fullpath_to_landmark_template, "and put it inside", $full_parentdirs_for_landmark_template);
	
}



sub Make_Disk_Classification_Signpost
{
	my $filename=$_[0];
	my $full_parentdirs_of_signpost=$_[1];
	
	
	##############################
	
	Create_Text_File_Whose_Filename_and_Contents_are_the_Same($full_parentdirs_of_signpost, $filename);
	
}



sub Make_Welcome_to_Disk_Signpost
{
	my $diskname=$_[0];
	my $full_parentdirs_of_signpost=$_[1];
	my $should_exclude_exclamation_point=$_[2];
	
	
	#----
	my $welcome_text;
	
	#@@@@
	$welcome_text = "Welcome to " . $diskname;

	if (! defined $should_exclude_exclamation_point)
	{	$welcome_text = $welcome_text . "!";
	}
	
	
	##############################
	
	Create_Text_File_Whose_Filename_and_Contents_are_the_Same($full_parentdirs_of_signpost, $welcome_text);
	
}



1;