# ApolliaLib :: File_Handling :: Moving :: Single_File.pm
#
# Stuff related to moving single files.


use strict;
use warnings;


package ApolliaLib::File_Handling::Moving::Single_File;



################################################################
#
#                         Exported Stuff
#
# This bit specifies which parts of this module are made available to
# scripts that use the following line:
#
# 	use ApolliaLib::File_Handling::Moving::Single_File;
#
#
# I pasted much of the below from:
#
# 	http://perldoc.perl.org/perlmod.html


BEGIN 
{
	require Exporter;

	#set the version for version checking
	our $VERSION = 1.00;

	#Inherit from Exporter to export functions and variables
	our @ISA = qw(Exporter);

	#Functions and variables which are exported by default
	our @EXPORT = qw(  
	Move_File
	);

	#Functions and variables which can be optionally exported
	our @EXPORT_OK = qw();
}



#
#                End of Exported Stuff section. 
#
#
################################################################



##################################################
#
#	Needed imports from other modules
#

use ApolliaLib::Common::Misc qw (
	true
	false
	Escape_Single_Quotes_for_Bash
	Display_and_Do_Command_Line_and_Result
	);
	
#
#	End of Needed imports from other modules
#
##################################################




################################################################
#
#	Start of stuff unique to this module
#



sub Move_File
{
	
	my $file_to_move=$_[0];
	
	# Arg 1 - not used, just for readability.  I usually put "and put it at"
	
	my $destination_folder=$_[2];
	
	#----
	my $escaped_file_to_move;
	my $escaped_destination_folder;
	my $command_line;
	
	#@@@@
	$escaped_file_to_move=Escape_Single_Quotes_for_Bash($file_to_move);
	$escaped_destination_folder=Escape_Single_Quotes_for_Bash($destination_folder);
	
	
	
	##############################
	
	if ( !-e $file_to_move || -l $file_to_move || -d $file_to_move || !-f $file_to_move || !-d $destination_folder  )
	{
		print "\n\nNot moved:
	$file_to_move";
		
		if (!-e $file_to_move) # Nothing exists at that path.
		{	print "\n\nDidn't exist in the first place.";
		
		}
		
		if (-l $file_to_move)
		{	print "\n\nThis path leads to a symlink, and symlinks can't be moved using the Move_File() function!";
		}
		
		if (-d $file_to_move)
		{	print "\n\nThis path leads to a folder, and folders can't be moved using the Move_File() function!";
		}
		
		if ( !-f $file_to_move && -e $file_to_move )
		{	print "\n\nOnly files can be moved using the Move_File() function.";
		}
		
		if ( !-d $destination_folder )
		{	print "\n\nThe destination folder path provided leads to something other than a folder!
	$destination_folder";
		}
	
		
		#!!!!
		return false;
	}
	
	$command_line = "mv --no-clobber '$escaped_file_to_move' '$escaped_destination_folder'";
	
	Display_and_Do_Command_Line_and_Result($command_line);
	
	
	if (-e $destination_folder)
	{	print "\n\nMoved successfully:
	$file_to_move =>
		$destination_folder";
		return true;
	}
	else
	{	print "\n\nFAILED to move:
	$file_to_move =>
		$destination_folder";
		return false;
	}
	
	
	
	print "\n\nThis section of Move_and_Rename_File() should never be reached!";
	
	
	#!!!!
	return false;
}



1;