# ApolliaLib :: File_Handling :: Moving_with_Rename :: Single_File.pm
#
# Stuff related to moving single files.


use strict;
use warnings;


package ApolliaLib::File_Handling::Moving_with_Rename::Single_File;



################################################################
#
#                         Exported Stuff
#
# This bit specifies which parts of this module are made available to
# scripts that use the following line:
#
# 	use ApolliaLib::File_Handling::Moving_with_Rename::Single_File;
#
#
# I pasted much of the below from:
#
# 	http://perldoc.perl.org/perlmod.html


BEGIN 
{
	require Exporter;

	#set the version for version checking
	our $VERSION = 1.00;

	#Inherit from Exporter to export functions and variables
	our @ISA = qw(Exporter);

	#Functions and variables which are exported by default
	our @EXPORT = qw(  
	Move_and_Rename_File
	);

	#Functions and variables which can be optionally exported
	our @EXPORT_OK = qw();
}



#
#                End of Exported Stuff section. 
#
#
################################################################



##################################################
#
#	Needed imports from other modules
#

use ApolliaLib::Common::Misc qw (
	true
	false
	Escape_Single_Quotes_for_Bash
	Display_and_Do_Command_Line_and_Result
	);
	
#
#	End of Needed imports from other modules
#
##################################################




################################################################
#
#	Start of stuff unique to this module
#



sub Move_and_Rename_File
{
	
	my $file_to_move_n_rename=$_[0];
	my $new_filepath=$_[1];
	
	#----
	my $escaped_file_to_move_n_rename;
	my $escaped_new_filepath;
	my $command_line;
	
	#@@@@
	$escaped_file_to_move_n_rename=Escape_Single_Quotes_for_Bash($file_to_move_n_rename);
	$escaped_new_filepath=Escape_Single_Quotes_for_Bash($new_filepath);
	
	
	
	##############################
	
	if ( !-e $file_to_move_n_rename || -l $file_to_move_n_rename || -d $file_to_move_n_rename || !-f $file_to_move_n_rename || -e $new_filepath || -l $new_filepath )
	{
		print "\n\nNot moved nor renamed:
	$file_to_move_n_rename";
		
		if (!-e $file_to_move_n_rename) # Nothing exists at that path.
		{	print "\n\nDidn't exist in the first place.";
		
		}
		
		if (-l $file_to_move_n_rename)
		{	print "\n\nThis path leads to a symlink, and symlinks can't be moved/renamed using the Move_and_Rename_File() function!";
		}
		
		if (-d $file_to_move_n_rename)
		{	print "\n\nThis path leads to a folder, and folders can't be moved/renamed using the Move_and_Rename_File() function!";
		}
		
		if ( !-f $file_to_move_n_rename && -e $file_to_move_n_rename )
		{	print "\n\nOnly files can be moved/renamed using the Move_and_Rename_File() function.";
		}
		
		if ( -e $new_filepath )
		{	print "\n\nSomething already exists at the intended new location for that file!
	$new_filepath";
		}
		
		if ( -l $new_filepath)
		{	print "\n\nThere is a symlink at the intended new location for that file!
	$new_filepath";
		}
		
		#!!!!
		return false;
	}
	
	$command_line = "mv --no-clobber '$escaped_file_to_move_n_rename' '$escaped_new_filepath'";
	
	Display_and_Do_Command_Line_and_Result($command_line);
	
	
	if (-e $new_filepath)
	{	print "\n\nMoved and renamed successfully:
	$file_to_move_n_rename =>
		$new_filepath";
		return true;
	}
	else
	{	print "\n\nFAILED to move and rename:
	$file_to_move_n_rename =>
		$new_filepath";
		return false;
	}
	
	
	
	print "\n\nThis section of Move_and_Rename_File() should never be reached!";
	
	
	#!!!!
	return false;
}



1;