# ApolliaLib :: File_Handling :: Text_File_Creation.pm
#
# Stuff related to non-destructively creating text files.
# Won't overwrite files that already exist.


use strict;
use warnings;


package ApolliaLib::File_Handling::Text_File_Creation;



################################################################
#
#                         Exported Stuff
#
# This bit specifies which parts of this module are made available to
# scripts that use the following line:
#
# 	use ApolliaLib::File_Handling::Text_File_Creation;
#
#
# I pasted much of the below from:
#
# 	http://perldoc.perl.org/perlmod.html


BEGIN 
{
	require Exporter;

	#set the version for version checking
	our $VERSION = 1.00;

	#Inherit from Exporter to export functions and variables
	our @ISA = qw(Exporter);

	#Functions and variables which are exported by default
	our @EXPORT = qw(  
		Create_Text_File_Whose_Filename_and_Contents_are_the_Same
		);

	#Functions and variables which can be optionally exported
	our @EXPORT_OK = qw(
	
		);
}



#
#                End of Exported Stuff section. 
#
#
################################################################



##################################################
#
#	Needed imports from other modules
#

use ApolliaLib::Common::Misc qw (
	true
	false
	necho
	Add_Slash_If_Needed_To_
	);

use ApolliaLib::Common::Path_String_Handling qw (
	Convert_Questionable_Chars_to_Dash
	);
	
#
#	End of Needed imports from other modules
#
##################################################




################################################################
#
#	Start of stuff unique to this module
#



sub Create_Text_File_Whose_Filename_and_Contents_are_the_Same
{
	my $full_parentdirs_for_text_file=$_[0];
	my $filename=$_[1];
	
	
	#----
	my $fullpath_for_text_file;
	
	#@@@@
	$filename = Convert_Questionable_Chars_to_Dash($filename);
	
	$fullpath_for_text_file=Add_Slash_If_Needed_To_($full_parentdirs_for_text_file);
	$fullpath_for_text_file=$fullpath_for_text_file . $filename;
	
	
	
	##############################
	
	
	my $OUTPUT_FILE;
	
	if (!-e $fullpath_for_text_file)
	{	print "\n\n\n\nNow attempting to create this not-yet-created file:\n\n	$fullpath_for_text_file";
		
		open ($OUTPUT_FILE, ">>$fullpath_for_text_file");
		print $OUTPUT_FILE "$filename";
		close ($OUTPUT_FILE);
		
		
		if (-e $fullpath_for_text_file)
		{	print "\n\n\nSuccessfully created file at: $fullpath_for_text_file";
			
			
			#!!!!
			return true;
		}
		else
		{	print "\n\n\nError: For some reason, couldn't create file at\n\n	$fullpath_for_text_file";
		}
	
	
		#!!!!
		return false;
	}
	else
	{	print "A file already exists at:\n\n	$fullpath_for_text_file\n\nNot remaking.";
	
	
		#!!!!
		return false;
	}
}



1;