# ApolliaLib :: File_or_Folder_Handling :: Many_Items.pm
#
# Functions related to many files or folders.  Chmod; copying to dest parentdirs; 
# chown; change group.


use strict;
use warnings;


package ApolliaLib::File_or_Folder_Handling::Many_Items;



################################################################
#
#                         Exported Stuff
#
# This bit specifies which parts of this module are made available to
# scripts that use the following line:
#
# 	use ApolliaLib::File_or_Folder_Handling::Many_Items;
#
#
# I pasted much of the below from:
#
# 	http://perldoc.perl.org/perlmod.html


BEGIN 
{
	require Exporter;

	#set the version for version checking
	our $VERSION = 1.00;

	#Inherit from Exporter to export functions and variables
	our @ISA = qw(Exporter);

	#Functions and variables which are exported by default
	our @EXPORT = qw(  
		Chmod_Many_Items
		Copy_Many_Items_to_Their_Dest_Parentdirs
		Copy_and_Rename_Many_Items
		Copy_Contents_of_Folder_into_Dest_Parentdir
		Chown_to_Spot_Recursively
		Chown_Item_Recursively
		Change_Group_of_Item_Recursively
		Chown_Many_Items_Recursively
		);

	#Functions and variables which can be optionally exported
	our @EXPORT_OK = qw(
	
		);
}



#
#                End of Exported Stuff section. 
#
#
################################################################



##################################################
#
#	Needed imports from other modules
#

use ApolliaLib::Common::Misc qw (
	true
	false
	Print_Hash_Indentedly
	Print_Array_Indentedly
	necho
	Escape_Single_Quotes_for_Bash
	Display_and_Do_Command_Line_and_Result
	);

use ApolliaLib::File_or_Folder_Handling::Single_Item(
	"Copy_Item",
	"Chmod_Item",
	"Copy_and_Rename_Item",
	);

use ApolliaLib::Folder_Handling::Contents(
	"Make_Array_of_Fullpaths_to_Folder_Contents"
	);
	
#
#	End of Needed imports from other modules
#
##################################################




################################################################
#
#	Start of stuff unique to this module
#



sub Change_Group_of_Item_Recursively
{
	my $path_to_change_group_of_recursively=$_[0];
	
	my $new_group_name=$_[1];
	
	
	#----
	my $escaped_path_to_change_group_of_recursively;
	my $command_line;
	
	my $result;
	
	#@@@@
	$escaped_path_to_change_group_of_recursively=Escape_Single_Quotes_for_Bash($path_to_change_group_of_recursively);
	
	
	
	##############################

	print "\n\n\n\nAttempting to change group of $path_to_change_group_of_recursively and all of its contents...";
	
	$command_line = "chgrp -R --no-dereference $new_group_name '$escaped_path_to_change_group_of_recursively'";
	
	$result = Display_and_Do_Command_Line_and_Result($command_line);
	
	
	#!!!!
	return $result;
}



sub Chown_to_Spot_Recursively
{	
	my $path_to_chown_recursively=$_[0];


	#----
	my $result;
	
	
	
	##############################
	
	$result = Chown_Item_Recursively( $path_to_chown_recursively, "and change its owner to", "spot", "spot" );
	
	
	#!!!!
	return $result;
	
}



sub Chown_Item_Recursively
{
	# Arg 1 - just for readability.  I usually set it to: "and change its owner to"
	
	my $path_to_chown_recursively=$_[0];
	
	my $new_owner_username=$_[2];
	my $new_owner_groupname=$_[3];
	
	
	#----
	my $escaped_path_to_chown_recursively;
	my $command_line;
	my $result;

	#@@@@
	$escaped_path_to_chown_recursively=Escape_Single_Quotes_for_Bash($path_to_chown_recursively);
	
	if (!$new_owner_username  && !$new_owner_groupname )
	{	print "Can't change owner of:\n	$path_to_chown_recursively\n\nNo user name or groupname was provided!";
		
		
		#!!!!
		return false;
	}
	
	
	
	##############################

	print "\n\n\n\nAttempting to change owner of $path_to_chown_recursively and all of its contents...";
	
	$command_line = "chown -R --no-dereference $new_owner_username";
	
	if ($new_owner_groupname)
	{	$command_line = $command_line . ":$new_owner_groupname";
	}
	$command_line = $command_line . " '$escaped_path_to_chown_recursively'";
	
	$result = Display_and_Do_Command_Line_and_Result($command_line);
	
	
	#!!!!
	return $result;
}



sub Chown_Many_Items_Recursively
{
	my $ref_to_array_of_items_to_chown=$_[0];
	
	# Arg 1 - just for readability.  I usually set it to: "and change their owner to"
		
	my $new_owner_username=$_[2];
	my $new_owner_groupname=$_[3];
	
	#----
	my @items_to_chown__their_full_paths;
	my $item_to_chown;
	my $result;
	
	#@@@@
	@items_to_chown__their_full_paths = @$ref_to_array_of_items_to_chown;
	
	
	
	##############################
	
	foreach $item_to_chown (@items_to_chown__their_full_paths)
	{
		$result=Chown_Item_Recursively($item_to_chown, "and change its owner to", $new_owner_username, $new_owner_groupname);
	}
	
	
	#!!!!
	return $result;	
}



sub Chmod_Many_Items
{
	my $ref_to_array_of_items_to_chmod=$_[0];
	my $mode=$_[1];
	
	
	#----
	my @items_to_chmod__their_full_paths;
	my $item_to_chmod;
	my $result;
	
	#@@@@
	@items_to_chmod__their_full_paths = @$ref_to_array_of_items_to_chmod;


	
	##############################
	
	foreach $item_to_chmod (@items_to_chmod__their_full_paths)
	{
		$result=Chmod_Item($item_to_chmod, $mode);
	}
	
	
	#!!!!
	return $result;
}



sub Make_Hash_of_Full_Paths_Then_a_Dest_Parentdir
{
	my $ref_to_array_of_full_paths=$_[0];
	my $dest_parentdir=$_[1];
	
	
	#----
	my @full_paths;
	my $full_path;
	
	my %hash_of_full_paths__then_a_dest_parentdir;
	
	#@@@@
	@full_paths=@$ref_to_array_of_full_paths;
	
	
	
	##############################
		
	
	foreach $full_path (@full_paths)
	{
		$hash_of_full_paths__then_a_dest_parentdir{$full_path} = $dest_parentdir;
	}
	
	
	return \%hash_of_full_paths__then_a_dest_parentdir;
}



sub Copy_Contents_of_Folder_into_Dest_Parentdir
{
	# Arg 0 - the folder whose contents will be copied.
	#
	# Arg 1 - unused, just for readability.
	#
	# Arg 2 - the dest parentdir - the folder where things will be copied to.
	#
	# Arg 3 - whether or not files should get overwritten (also known as clobbered).
	#
	# Arg 4 - more args for the copy (cp) command, so you can "--dereference" symlinks if you prefer,
	# 	or do other things.
	
	my $folder_to_copy_from=$_[0];
	
	my $dest_parentdir=$_[2];
	
	my $should_overwrite_or_not=$_[3];
	
	my $additional_args_for_copy_command=$_[4];
	
	
	#----
	my @fullpaths_of_items_in_folder;
	my $ref_to_fullpaths_of_items_in_folder;
	my $ref_to_hash_of_full_paths__then_a_dest_parentdir;
	my %hash_of_full_paths__then_a_dest_parentdir;
	
	#@@@@
	$ref_to_fullpaths_of_items_in_folder=Make_Array_of_Fullpaths_to_Folder_Contents($folder_to_copy_from);
	@fullpaths_of_items_in_folder=@$ref_to_fullpaths_of_items_in_folder;
	
	$ref_to_hash_of_full_paths__then_a_dest_parentdir=Make_Hash_of_Full_Paths_Then_a_Dest_Parentdir( \@fullpaths_of_items_in_folder, $dest_parentdir);	
	%hash_of_full_paths__then_a_dest_parentdir=%$ref_to_hash_of_full_paths__then_a_dest_parentdir;
	
	
	
	##############################
	
	
	Copy_Many_Items_to_Their_Dest_Parentdirs(\%hash_of_full_paths__then_a_dest_parentdir, $should_overwrite_or_not, $additional_args_for_copy_command);
	
	
}




sub Copy_and_Rename_Many_Items
{	
	
	# Arg 0 - a hash (ref) of files or folders with full paths, then their dest paths.
	#
	# Arg 1 - whether or not files should get overwritten (also known as clobbered).
	#
	# Arg 2 - more args for the copy (cp) command, so you can "--dereference" symlinks if you prefer,
	# 	or do other things.
	#
	#
	# This function loops through the hash of full filepaths or folderpaths and their dest parentdirs,
	# then sends the function Copy_Item() the info it needs to copy each file/folderpath to
	# its destination parentdir, and also tells it whether or not files should get overwritten,
	# and sends along any additional args for the copy (cp) command.
	
	
	my $ref_to_hash_of_items_and_their_dest_paths=$_[0];
	
	my $should_overwrite_or_not=$_[1];
	
	my $additional_args_for_copy_command=$_[2];
	
	
	#----
	my %items_to_copy_and_rename__their_full_paths__then_their_dest_paths;
	
	my $from_path;
	my $dest_path;
	
	my $result;
	
	#@@@@
	%items_to_copy_and_rename__their_full_paths__then_their_dest_paths=
		%$ref_to_hash_of_items_and_their_dest_paths;
	
	
	
	##############################
	
	while ( ($from_path, $dest_path) = each (%items_to_copy_and_rename__their_full_paths__then_their_dest_paths))
	{	
		$result=Copy_and_Rename_Item($from_path, "and put it inside", $dest_path, $should_overwrite_or_not, $additional_args_for_copy_command);
		
	}
	
}



sub Copy_Many_Items_to_Their_Dest_Parentdirs
{	
	
	# Arg 0 - a hash (ref) of files or folders with full paths, then their dest parentdirs.
	#
	# Arg 1 - whether or not files should get overwritten (also known as clobbered).
	#
	# Arg 2 - more args for the copy (cp) command, so you can "--dereference" symlinks if you prefer,
	# 	or do other things.
	#
	#
	# This function loops through the hash of full filepaths or folderpaths and their dest parentdirs,
	# then sends the function Copy_Item() the info it needs to copy each file/folderpath to
	# its destination parentdir, and also tells it whether or not files should get overwritten,
	# and sends along any additional args for the copy (cp) command.
	
	
	my $ref_to_hash_of_items_and_their_dest_parentdirs=$_[0];
	
	my $should_overwrite_or_not=$_[1];
	
	my $additional_args_for_copy_command=$_[2];
	
	
	#----
	my %items_to_copy__their_full_paths__then_their_dest_parentdirs;
	
	my $from_path;
	my $dest_parentdirs;
	
	my $result;
	
	#@@@@
	%items_to_copy__their_full_paths__then_their_dest_parentdirs=
		%$ref_to_hash_of_items_and_their_dest_parentdirs;
	
	
	
	##############################
	
	while ( ($from_path, $dest_parentdirs) = each (%items_to_copy__their_full_paths__then_their_dest_parentdirs))
	{	
		$result=Copy_Item($from_path, "and put it inside", $dest_parentdirs, $should_overwrite_or_not, $additional_args_for_copy_command);
		
	}
	
}



1;