# ApolliaLib :: File_or_Folder_Handling :: Single_Item.pm
#
# Functions related to single files or folders.  Chmod, copy, chown.


use strict;
use warnings;


package ApolliaLib::File_or_Folder_Handling::Single_Item;



################################################################
#
#                         Exported Stuff
#
# This bit specifies which parts of this module are made available to
# scripts that use the following line:
#
# 	use ApolliaLib::File_or_Folder_Handling::Single_Item;
#
#
# I pasted much of the below from:
#
# 	http://perldoc.perl.org/perlmod.html


BEGIN 
{
	require Exporter;

	#set the version for version checking
	our $VERSION = 1.00;

	#Inherit from Exporter to export functions and variables
	our @ISA = qw(Exporter);

	#Functions and variables which are exported by default
	our @EXPORT = qw(  
		Copy_Item
		Copy_and_Rename_Item
		Chmod_Item
		Chown_Item
		Chown_to_Spot
		);

	#Functions and variables which can be optionally exported
	our @EXPORT_OK = qw(
		
		);
}



#
#                End of Exported Stuff section. 
#
#
################################################################



##################################################
#
#	Needed imports from other modules
#

use ApolliaLib::Common::Misc qw (
	true
	false
	Escape_Single_Quotes_for_Bash
	Display_and_Do_Command_Line
	Display_and_Do_Command_Line_and_Result
	Print_Octal_Value_Properly
	);

use ApolliaLib::Common::End_Script (
	"End_Script_and_Wait_for_Return"

	);

#
#	End of Needed imports from other modules
#
##################################################




################################################################
#
#	Start of stuff unique to this module
#



sub Chown_to_Spot
{
	my $path_to_chown=$_[0];
	
	
	#----
	my $succeeded;



	##############################


	$succeeded = Chown_Item( $path_to_chown, "and change its owner to", "spot", "spot" );
	
	
	#!!!!
	return $succeeded;
	
}



sub Chown_Item
{
	# Arg 1 - just for readability.  I usually use: "and change its owner to"
	
	my $path_to_chown=$_[0];
	
	my $new_owner_username=$_[2];
	my $new_owner_groupname=$_[3];
	
	#----
	my $uid;
	my $gid;
	my $succeeded;
	
	#@@@@
	
	if ( $new_owner_username)
	{
		$uid = getpwnam $new_owner_username;
	}
	else
	{	$uid = -1;
	}
	if ($new_owner_groupname)
	{	$gid = getgrnam $new_owner_groupname;
	}
	else
	{	$gid = -1;
	}
	

	if ($uid == -1 && $gid == -1)
	{	print "Can't change owner of:\n	$path_to_chown\n\nNo user name or groupname was provided!";
		
		
		#!!!!
		return false;
	}
	
		
	##############################
	

	print "\n\n\n\nAttempting to change owner of $path_to_chown...";
	
	if ($uid != -1)
	{	print "\n	New owner: $new_owner_username $uid";
	}
	if ($gid != -1)
	{	print "\n	New group: $new_owner_groupname $gid";
	}
	
	$succeeded = chown $uid, $gid, $path_to_chown;
	
	if ($succeeded)
	{	print "\n\nSucceeded!";
	}
	else
	{	print "\n\nFailed to change owner!";
	}
	
	
	#!!!!
	return $succeeded;
}



sub Chmod_Item
{
	my $full_path_of_item_to_chmod=$_[0];
	my $mode=$_[1];
	
	
	#----
	my $result;
	


	##############################
	
	$result=chmod $mode, $full_path_of_item_to_chmod;
	
	if ($result>0)
	{
		print "\n\nGranted ";
		Print_Octal_Value_Properly($mode);
		print " permissions to:\n	$full_path_of_item_to_chmod";
	}
	else
	{
		print "\n\nFAILED to grant "; 
		
		Print_Octal_Value_Properly($mode);
		
		print " permissions to:\n	$full_path_of_item_to_chmod";
	}
	
	
	#!!!!
	return $result;
}



sub Copy_and_Rename_Item
{	# Copies an item from a path to a dest parentdir, without renaming the copied item.
	# The item will have the same name at its destination that the item had at its from path.
	#
	# Arg 0: The path to the file or folder that will be copied. 
	#
	# Arg 1: Unused, can be anything.  Just for readability.
	#		Typically, I put "and put it inside".
	#
	# Arg 2: The destination where the copied file will end up.
	#
	# Arg 3 - whether or not files should get overwritten (also known as clobbered).
	#
	# Arg 4 - more args for the copy (cp) command, so you can "--dereference" symlinks if you prefer,
	# 	or do other things.
	#
	#
	# This is the function that does the actual copying of items - 
	# unlike the functions Copy_Many_Items_to_Their_Dest_Parentdirs() and 
	# Copy_Relativepath_Items_to_Their_Dest_Parentdirs() - which just are steps
	# along the way to this Copy_Item() function.

	
	my $from_path=$_[0];
	my $dest_path=$_[2];
	
	my $should_overwrite_or_not=$_[3];
	
	my $additional_args_for_copy_command=$_[4];
	
	
	#----	
	my $command_line;
	
	my $escaped_from_path;
	my $escaped_dest_path;
	
	my $result;
	
	my $from_filename;
	my $from_parentdirs;
	my $from_suffix;
	


	##############################
	
	if ( !defined $dest_path)
	{	print "\n\nAborting this script, because no dest path was specified when Copy_and_Rename_Item() was called!  (Most likely, the useless 2nd argument was missing.)";
		
		
		#!!!!
		End_Script_and_Wait_for_Return();
		
	}
	
	
	
	$command_line="cp --verbose --preserve --recursive ";
	
	
	if ( !defined $should_overwrite_or_not || $should_overwrite_or_not ne "Overwrite Existing Files")
	{	$command_line = $command_line . "--no-clobber ";
	}
	
	
	
	if ( defined $additional_args_for_copy_command )
	{	$command_line = $command_line . $additional_args_for_copy_command;
	}
	
	
	$escaped_from_path = Escape_Single_Quotes_for_Bash($from_path);
	$escaped_dest_path = Escape_Single_Quotes_for_Bash($dest_path);
	
	
	$command_line = "$command_line '$escaped_from_path' --no-target-directory '$escaped_dest_path'";
	
	$result=Display_and_Do_Command_Line($command_line);
	
}



sub Copy_Item
{	# Copies an item from a path to a dest parentdir, without renaming the copied item.
	# The item will have the same name at its destination that the item had at its from path.
	#
	# Arg 0: The path to the file or folder that will be copied. 
	#
	# Arg 1: Unused, can be anything.  Just for readability.
	#		Typically, I put "and put it inside".
	#
	# Arg 2: The parentdirs where the copied file will end up.
	#
	# Arg 3 - whether or not files should get overwritten (also known as clobbered).
	#
	# Arg 4 - more args for the copy (cp) command, so you can "--dereference" symlinks if you prefer,
	# 	or do other things.
	#
	#
	# This is the function that does the actual copying of items - 
	# unlike the functions Copy_Many_Items_to_Their_Dest_Parentdirs() and 
	# Copy_Relativepath_Items_to_Their_Dest_Parentdirs() - which just are steps
	# along the way to this Copy_Item() function.

	
	my $from_path=$_[0];
	my $dest_parentdirs=$_[2];
	
	my $should_overwrite_or_not=$_[3];
	
	my $additional_args_for_copy_command=$_[4];
	
	
	#----	
	my $command_line;
	
	my $escaped_from_path;
	my $escaped_dest_parentdirs;
	
	my $result;
	
	my $from_filename;
	my $from_parentdirs;
	my $from_suffix;
	


	##############################
	
	if ( !defined $dest_parentdirs)
	{	print "\n\nAborting this script, because no dest parentdirs were specified when Copy_Item() was called!  (Most likely, the useless 2nd argument was missing.)";
		
		
		#!!!!
		End_Script_and_Wait_for_Return();
		
	}
	
	
	
	$command_line="cp --verbose --preserve --recursive ";
	
	
	if ( !defined $should_overwrite_or_not || $should_overwrite_or_not ne "Overwrite Existing Files")
	{	$command_line = $command_line . "--no-clobber ";
	}
	
	
	
	if ( defined $additional_args_for_copy_command )
	{	$command_line = $command_line . $additional_args_for_copy_command;
	}
	
	
	$escaped_from_path = Escape_Single_Quotes_for_Bash($from_path);
	$escaped_dest_parentdirs = Escape_Single_Quotes_for_Bash($dest_parentdirs);
	
	
	$command_line = "$command_line '$escaped_from_path' --target-directory '$escaped_dest_parentdirs'";
	
	$result=Display_and_Do_Command_Line($command_line);
	
}



1;