# ApolliaLib :: Folder_Handling :: Single_Folder.pm
#
# Stuff related to single folders.  Creation.


use strict;
use warnings;


package ApolliaLib::Folder_Handling::Single_Folder;



################################################################
#
#                         Exported Stuff
#
# This bit specifies which parts of this module are made available to
# scripts that use the following line:
#
# 	use ApolliaLib::Folder_Handling::Single_Folder;
#
#
# I pasted much of the below from:
#
# 	http://perldoc.perl.org/perlmod.html


BEGIN 
{
	require Exporter;

	#set the version for version checking
	our $VERSION = 1.00;

	#Inherit from Exporter to export functions and variables
	our @ISA = qw(Exporter);

	#Functions and variables which are exported by default
	our @EXPORT = qw(  
		Make_Folder_Path
		Make_Sure_Folder_Exists_and_Create_if_it_Doesnt_and_Abort_If_Uncreateable
		Abort_If_Path_Is_Not_A_Folder
		Create_Temporary_ApolliaLib_Workspace_for_Downloads
		Create_Temporary_ApolliaLib_Workspace_for_Tarballs
		);

	#Functions and variables which can be optionally exported
	our @EXPORT_OK = qw(
	
		);
}



#
#                End of Exported Stuff section. 
#
#
################################################################



##################################################
#
#	Needed imports from other modules
#

use ApolliaLib::Common::Misc qw (
	true
	false
	
	);

use ApolliaLib::Common::End_Script qw (
	End_Script_and_Wait_for_Return
	);

use ApolliaLib::Common::Settings (
	'$ApolliaLib_Settings'
	);
	
use File::Path ("mkpath");

#
#	End of Needed imports from other modules
#
##################################################




################################################################
#
#	Start of stuff unique to this module
#


sub Create_Temporary_ApolliaLib_Workspace_for_Downloads
{
	print "\n\n\n\nNow creating temporary workspace for downloads...";
	
	Make_Folder_Path( $ApolliaLib_Settings->fullpath_to_ApolliaLib_Temp_Workspace_for_Downloads );
	
}




sub Create_Temporary_ApolliaLib_Workspace_for_Tarballs
{
	print "\n\n\n\nNow creating temporary workspace for tarballs...";
	
	Make_Folder_Path( $ApolliaLib_Settings->fullpath_to_ApolliaLib_Temp_Workspace_for_Tarballs );
		
}



sub Abort_If_Path_Is_Not_A_Folder
{
	my $folder_path_to_check=$_[0];
	my $custom_message=$_[1];

	
	
	##############################
	
	
	if (!-d $folder_path_to_check || $folder_path_to_check eq "" )
	{	
	
		if (!defined $custom_message)
		{
			print "\n\nError: ";
			print $folder_path_to_check;
			print " is not a folder!\n\n";
			
			print "For this script to work, this path must lead to a folder.  Aborting.";
		}
		else
		{	print $custom_message;
		}
		
		#!!!!
		End_Script_and_Wait_for_Return();
	}
}



sub Make_Sure_Folder_Exists_and_Create_if_it_Doesnt_and_Abort_If_Uncreateable
{
	my $fullpath_to_folder=$_[0];
	my $custom_message=$_[1];
	
	
	#----
	my $succeeded;
	
	
	
	##############################
	
	
	if ($fullpath_to_folder eq "")
	{	
		if (!defined $custom_message)
		{	print "\n\nError: no folder string provided!  Aborting.\n\n";
		}
		else
		{	print $custom_message;
		}
		
		
		#!!!!
		End_Script_and_Wait_for_Return();
	}

	
	if (!-e $fullpath_to_folder)
	{	$succeeded=Make_Folder_Path($fullpath_to_folder);
		if (!$succeeded)
		{
			if (!defined $custom_message)
			{	print "\n\nError: couldn't create the folder $fullpath_to_folder - aborting!\n\n";
			}
			else
			{	print $custom_message;
			}
			
			
			#!!!!
			End_Script_and_Wait_for_Return();
		
		}
	}
}



sub Make_Folder_Path
{	
	my $folder_path_to_make=$_[0];
	#my $silent_or_not=$_[1];
	
	#----
	my $result;


	
	##############################
		
		
	if (!-e $folder_path_to_make)
	{	$result = mkpath $folder_path_to_make;
	
		if ($result >= 1)
		{	print "\n\n\n\nMade folder path:
	$folder_path_to_make";
	
	
			#!!!!
			return (true, $result);
		}
		else
		{	print "\n\n\n\nCouldn't make folder path:
	$folder_path_to_make
Error: $!";

		
			#!!!!
			return false;
		}
	}
	else # Something exists at the folder path already.
	{	
		print "\n\n\n\nCouldn't make folder path:
	$folder_path_to_make

Something exists there already.\n";
		
		
		#!!!!
		return false;
	}
}



1;