# ApolliaLib :: Puppy_Setup_Kit :: Downloader.pm
#
# Stuff for Puppy Setup Kits related to downloading.  Affects all Puppy Setup Kits.


use strict;
use warnings;


package ApolliaLib::Puppy_Setup_Kit::Downloader;



################################################################
#
#                         Exported Stuff
#
# This bit specifies which parts of this module are made available to
# scripts that use the following line:
#
# 	use ApolliaLib::Puppy_Setup_Kit::Downloader;
#
#
# I pasted much of the below from:
#
# 	http://perldoc.perl.org/perlmod.html


BEGIN 
{
	require Exporter;

	#set the version for version checking
	our $VERSION = 1.00;

	#Inherit from Exporter to export functions and variables
	our @ISA = qw(Exporter);

	#Functions and variables which are exported by default
	our @EXPORT = qw(  
		
		$Puppy_Setup_Kit_Downloader
		);

	#Functions and variables which can be optionally exported
	our @EXPORT_OK = qw(
		
		);
}



#
#                End of Exported Stuff section. 
#
#
################################################################



##################################################
#
#	Needed imports from other modules
#


use ApolliaLib::Puppy_Setup_Kit::Settings qw ( 
	$Puppy_Setup_Kit
	);

use ApolliaLib::Common::Misc (
	"true",
	"false",
	"necho",
	"Escape_Single_Quotes_for_Bash",
	"Display_and_Do_Command_Line_and_Result",
	);
	
use ApolliaLib::Common::Settings (
	'$ApolliaLib_Settings',
	
	);

use ApolliaLib::Common::User_Prompted_Actions (
	"Ask_To_Do_Action",
	"Pause_Until_User_Presses_Enter"

	);
	
use ApolliaLib::Checksums::SHA512 (
	"SHA512_Check"
	);
	
use ApolliaLib::File_or_Folder_Handling::Many_Items (
	"Copy_Many_Items_to_Their_Dest_Parentdirs"
	);
	
use ApolliaLib::Folder_Handling::Single_Folder ("Create_Temporary_ApolliaLib_Workspace_for_Downloads");



use ApolliaLib::Folder_Handling::Deletion::Single_Folder_Recursively ("Delete_Temporary_ApolliaLib_Workspace_for_Downloads");

use ApolliaLib::Downloading::With_Wget ( "Download_Items_to_ApolliaLib_Temp_Downloads_Workspace_Using_Wget");

use ApolliaLib::Folder_Handling::Many_Folders ( "Make_Many_Folder_Paths");

use ApolliaLib::Common::End_Script (
	"End_Script"
	);

use ApolliaLib::File_or_Folder_Handling::Single_Item ( "Chmod_Item");

use Object::Tiny qw( 


	
	);
	
	
#
#	End of Needed imports from other modules
#
##################################################




################################################################
#
#	Start of stuff unique to this module
#



sub new
{
	my $class = shift;
	
	
	#----
	my $self;
	
	#@@@@
	$self = {};
			

	##############################
	
	bless $self, $class;
	
	return $self;
}



sub Initialize 
{
	my $self = $_[0];

	return $self;
}



sub Download_File
{
	my $self = $_[0];
	
	my $realm_for_file = $_[1];
	my $relative_destination_folders = $_[2];
	my $filename_of_file_to_download = $_[3];
	my $human_readable_software_name_and_maybe_desc = $_[4];
	my $URL_for_file = $_[5];
	my $optional_extra_params_for_wget = $_[6];
	
	
	#----
	my $destination_folder;
	my $full_path_to_SHA512_checksum_for_file;
	my @dirs_to_make;
	my $folder_for_wget_downloads;
	my @urls_for_wget;
	my $download_verified;
	my $sha512_check_result;
	my $printable_human_readable_software_name_and_maybe_desc;
	my $uncapitalized_human_readable_software_name_and_maybe_desc;
	my $download_was_successful;
	my $full_path_to_downloaded_file;
	
	#@@@@
	
	$uncapitalized_human_readable_software_name_and_maybe_desc=lcfirst($human_readable_software_name_and_maybe_desc);
	
	$destination_folder=$Puppy_Setup_Kit->Realm_Path($realm_for_file) . $relative_destination_folders . "/";
	
	$full_path_to_SHA512_checksum_for_file=$Puppy_Setup_Kit->Realm_Path($realm_for_file) . "SHA512 Checksums/" . $relative_destination_folders . "/" . $filename_of_file_to_download . ".sha512";
	
	$download_was_successful=false;
	
	
	##############################
	
	
	
	#---------------
	#
	# Dirs to make:

	@dirs_to_make=(
		$destination_folder
		);
		
	Make_Many_Folder_Paths(\@dirs_to_make);
	
	
	
	#---------------
	#
	# Temporary workspace to make:
	
	if ( $Puppy_Setup_Kit->should_use_temporary_workspace_for_downloads )
	{
		Create_Temporary_ApolliaLib_Workspace_for_Downloads();
		
		$folder_for_wget_downloads=$ApolliaLib_Settings->fullpath_to_ApolliaLib_Temp_Workspace_for_Downloads;
	}
	else
	{	$folder_for_wget_downloads=$destination_folder;
	}
	
	
	print "\n\nChecking once again to find out if a different instance of Wget is already downloading the URL we're about to download...";
	
	
	#!!!!
	$self->Check_Linux_Process_List_for_Wget_Already_Downloading_this_URL($URL_for_file);
	
	
	#---------------
	#
	# URLs to download:
	
	@urls_for_wget=(
		$URL_for_file
	
	);
	
	print "\n\n\n\nStarting download...";
	
	Download_Items_to_ApolliaLib_Temp_Downloads_Workspace_Using_Wget( \@urls_for_wget, $folder_for_wget_downloads, $optional_extra_params_for_wget);
	
	
	
	#---------------
	#
	# Verifying the file


	if ($Puppy_Setup_Kit->should_not_perform_first_sha512_checksum_check_after_download)
	{
		print "\n\n\n\nNow making sure the downloaded file exists, but configured not to check the SHA512 checksum at this point...";
		
		$sha512_check_result=true;
	}
	else
	{	print "\n\n\n\nNow making sure the downloaded file exists, and checking its SHA512 checksum...";
	}
	
	#!!!!
	$download_verified = $self->Verify_Downloaded_File($filename_of_file_to_download, $folder_for_wget_downloads, $full_path_to_SHA512_checksum_for_file);
	
		
	if ( $Puppy_Setup_Kit->should_use_temporary_workspace_for_downloads )
	{
		
		$sha512_check_result = $self->Copy_Downloaded_File_in_Temp_Download_Workspace_to_Final_Destination__Then_Verify($filename_of_file_to_download, $destination_folder, $full_path_to_SHA512_checksum_for_file);
		
		
		if ( $sha512_check_result == true )
		{
			#------------------
			#
			# Temporary workspace to delete:
				
			Delete_Temporary_ApolliaLib_Workspace_for_Downloads();
				
			$download_was_successful=true;
			
			
		}
		else
		{	print "\n\nWarning: The SHA512 checksum included in Apollia's Puppy Setup Kit doesn't match the downloaded then copied file!";
		
			Pause_Until_User_Presses_Enter();
			
			
			#!!!!
			return false;
		}
	}
	else # If we weren't using the temporary download workspace...
	{	
		if ($download_verified)
		{	
			$download_was_successful=true;
			
		}
	}
	
	if ($download_was_successful)
	{
		print "\n\n\nDownload succeeded!  Got $uncapitalized_human_readable_software_name_and_maybe_desc.";
			
		#print "\n\n\nDownload succeeded!  $human_readable_software_name_and_maybe_desc has been downloaded!";
		
		$full_path_to_downloaded_file = $destination_folder . $filename_of_file_to_download;


		#----------------------
		#
		#  Grant permissions
		#	0755 - "rwx for owner, rx for group and rx for the world"
		#
		#	http://www.linux.org/threads/file-permissions-chmod.4094/
		#
		#	The quoted page says 755, but only 0755 works right in Lucid Puppy 5.2.8 version 004
		
		Chmod_Item($full_path_to_downloaded_file, 0400);
		# Read only for owner, no perms for group and world
		
		
		#!!!!
		return true;
	}
	
	
	#!!!!
	return false;
}



sub Make_Sure_Download_Is_Feasible
{
	my $self = $_[0];
	
	my $realm_for_file = $_[1];
	my $relative_destination_folders = $_[2];
	my $filename_of_file_to_download = $_[3];
	my $human_readable_software_name_and_maybe_desc = $_[4];
	my $Minimum_RAM_Required_in_MB_for_file = $_[5];
	my $URL_for_file = $_[6];
	
	
	#----
	my $full_path_to_where_downloaded_file_will_be_or_already_is;
	my $destination_folder;
	
	#@@@@
	$destination_folder=$Puppy_Setup_Kit->Realm_Path($realm_for_file) . $relative_destination_folders . "/";
	
	$full_path_to_where_downloaded_file_will_be_or_already_is = $destination_folder . $filename_of_file_to_download;
	
	
	
	##############################
	
	
	##############################
	#
	# First, we check to see if the file we're about to download already exists.
	#
	# No point in continuing if it already exists.
	
	if (-e $full_path_to_where_downloaded_file_will_be_or_already_is)
	{
		print "\n\nCan't proceed, the file is already in your Puppy Setup Kit!\n\n$human_readable_software_name_and_maybe_desc is already in your $realm_for_file folder at:\n
	$full_path_to_where_downloaded_file_will_be_or_already_is";
		
		print "\n\nAborting!";
		
		
		#!!!!
		End_Script();
	}	
	
	
	##############################
	#
	# Then, we make sure there's enough RAM.  If there is, then we'll download the file to 
	# a temporary workspace in the RAM disk.
	#
	# If there's not enough RAM, then, if it's allowed, we'll download the file directly 
	# to the destination folder.
	#
	# If that's not allowed and there's not enough RAM, we have to stop here.


	#!!!!
	$self->If_Necessary__Find_Out_If_Enough_RAM_is_Available_for_Download( $Minimum_RAM_Required_in_MB_for_file );
	
	
	print "\n\n\n\nTrying to find out if a different instance of Wget is already downloading the URL we're about to download...";
	
	#!!!!
	$self->Check_Linux_Process_List_for_Wget_Already_Downloading_this_URL($URL_for_file);
	
	print "\n\n";
	
}


sub Check_Linux_Process_List_for_Wget_Already_Downloading_this_URL
{
	my $self=$_[0];
	
	my $URL_for_file=$_[1];


	#----
	my $command_line;
	my $result;
	my $escaped_URL_for_file;
	
	#@@@@
	$escaped_URL_for_file=Escape_Single_Quotes_for_Bash($URL_for_file);
	
	
	##############################
	
	#$command_line = "ps | grep -e \"wget\" -e '$escaped_URL_for_file' | grep -v \"grep\" ";
	
	$command_line = " ps | grep -v \"grep\" | grep -e \"wget\" -e '$escaped_URL_for_file' -e \"no-clobber\"";
	
	$result = Display_and_Do_Command_Line_and_Result($command_line);
	
	if ($result)
	{	#print "Result: $result ";
		print "\n\nDownload of $URL_for_file is already in progress!  Aborting!";
		
		End_Script();
	}
	
	print "\nSeems OK!  No ongoing download detected!";
}



sub Copy_Downloaded_File_in_Temp_Download_Workspace_to_Final_Destination__Then_Verify
{
	my $self=$_[0];
	
	my $filename_of_file_to_download=$_[1];
	my $destination_folder=$_[2];
	my $full_path_to_SHA512_checksum_for_file=$_[3];
	
	
	#----
	my $path_to_tmp_file;
	my $full_path_to_where_downloaded_file_will_be_or_already_is;
	my %files_to_copy__their_full_paths__then_their_dest_parentdirs;
	my $sha512_check_result;
	
	#@@@@
	$path_to_tmp_file=$ApolliaLib_Settings->fullpath_to_ApolliaLib_Temp_Workspace_for_Downloads . $filename_of_file_to_download;
	
	$full_path_to_where_downloaded_file_will_be_or_already_is = $destination_folder . $filename_of_file_to_download;
	
	
	
	##############################
	
	print "\n\n\n\nNow copying the file from the temporary download workspace to the final destination...";
	
	
	#-----------------
	#
	# Files to copy:

	%files_to_copy__their_full_paths__then_their_dest_parentdirs = (
		$path_to_tmp_file => $destination_folder,
		
		
		);

	Copy_Many_Items_to_Their_Dest_Parentdirs( \%files_to_copy__their_full_paths__then_their_dest_parentdirs, "Overwrite Existing Files" );


	if ( !-e $full_path_to_where_downloaded_file_will_be_or_already_is)
	{	print "\n\nSomehow, copying the downloaded file from...
	$path_to_tmp_file
to
	$full_path_to_where_downloaded_file_will_be_or_already_is
...failed!";

		
		#!!!!
		End_Script();
	}


	if ($Puppy_Setup_Kit->should_not_perform_second_sha512_checksum_check_after_download)
	{
		print "\n\n\n\nConfigured not to check the SHA512 checksum at this point...";
		return true;
	}
	else
	{	print "\n\n\n\nChecking the SHA512 checksum again...";
	}
	
	
	
	#------------------
	# Now we check the SHA512 checksum again to make sure the copy worked.
	#
	# If it doesn't match, the user is warned, but the file is left alone.
	
	$sha512_check_result = SHA512_Check($full_path_to_SHA512_checksum_for_file, "and run check from this folder:", $destination_folder);
	
	if ( $sha512_check_result )
	{	
		
		#!!!!
		return true;
	}
	else
	{
		
		#!!!!
		return false;

	}

	
}



sub Verify_Downloaded_File
{
	my $self=$_[0];
	
	my $filename_of_file_to_download=$_[1];
	my $folder_of_download=$_[2];
	my $full_path_to_SHA512_checksum_for_file=$_[3];
	
	#----
	my $path_to_downloaded_file;
	my $sha512_check_result;
	
	#@@@@
	$path_to_downloaded_file=$folder_of_download . $filename_of_file_to_download;
	

	
	##############################
	
	
	if (!-e $path_to_downloaded_file )
	{	print "\n\n\n\nDownload failed - nothing exists at:
	$path_to_downloaded_file";
		
		
		#!!!!
		End_Script();
	}
	
	# Now we make sure its SHA512 checksum matches the one included in Apollia's Puppy Setup Kit.
	
	if ($Puppy_Setup_Kit->should_not_perform_first_sha512_checksum_check_after_download)
	{	return true;
	}
	
	$sha512_check_result = SHA512_Check($full_path_to_SHA512_checksum_for_file, "and run check from this folder:", $folder_of_download);
	
		
	if ( $sha512_check_result == false )
	{	print "\n\n\n\nThe SHA512 checksum included in Apollia's Puppy Setup Kit doesn't match the downloaded file!";
	
	
		#!!!!
		Ask_To_Do_Action("continue anyway", "Don't display skip this step text");


		#!!!!
		return false;
	}
	
	
	#!!!!
	return true;
}



sub If_Necessary__Find_Out_If_Enough_RAM_is_Available_for_Download
{
	my $self=$_[0];
	
	my $Minimum_RAM_Required_in_MB_for_file=$_[1];
	
	#----
	my $minimum_RAM_required_in_MB;
	my $available_RAM;
	
	

	##############################
	
	if ($Puppy_Setup_Kit->should_use_temporary_workspace_for_downloads)
	{
		$minimum_RAM_required_in_MB=$Puppy_Setup_Kit->RAM_floor_in_MB + $Minimum_RAM_Required_in_MB_for_file;
		
		$available_RAM = $Puppy_Setup_Kit->Find_Out_Available_RAM_in_This_Puppys_RAM_Disk();
		
		print "\n\nAvailable RAM: $available_RAM MB";
		
		if ( $available_RAM < $minimum_RAM_required_in_MB)
		{	# If there's not enough available RAM...
			
			if (! $Puppy_Setup_Kit->should_allow_downloading_direct_to_physical_disk_if_RAM_disk_is_too_full )
			{	# If downloads to the physical disk aren't allowed...
				print "\n\nNot enough RAM available to download the file to a temporary location in the RAM disk!  There's $available_RAM MB available, and this script may not use any amount of personal storage space that would push the amount of available personal storage space below $minimum_RAM_required_in_MB MB.  Aborting!";
				
				
				#!!!!
				End_Script();
			}
			else
			{	print "\n\nMust download the file directly to the permanent location on the physical disk, because this script can't use any amount of personal storage space that would push the amount of available personal storage space below $minimum_RAM_required_in_MB MB.";
			
				$Puppy_Setup_Kit->{'should_use_temporary_workspace_for_downloads'}=false;
				
				
				#!!!!
				return false;
			}
		}
		else
		{	# There's enough RAM.
			print "   Approximate MB needed for file: $Minimum_RAM_Required_in_MB_for_file MB\n\nCan proceed with downloading the file to a temporary workspace for downloads!";


			#!!!!
			return true;
		}
	}
}



#---------------------------------------------------------------------------
#
#	Now the $Puppy_Setup_Kit_Downloader object actually gets created, 
#	but no settings are set yet.
#
#

our $Puppy_Setup_Kit_Downloader;

$Puppy_Setup_Kit_Downloader=ApolliaLib::Puppy_Setup_Kit::Downloader->new();


#---------------------------------------------------------------------------
#
# 	Aliases, equivalent to $Puppy_Setup_Kit_Downloader
#
#


#---------------------------------------------------------------------------
#
#	Now, the $Puppy_Setup_Kit_Settings get set to their defaults.
#
#	The defaults don't suit every script, but that's OK, because 
#	individual scripts can (and often should) adjust the defaults.


$Puppy_Setup_Kit_Downloader->Initialize();



1;