# ApolliaLib :: Puppy_Setup_Kit :: Settings.pm
#
# Settings for Puppy Setup Kits.  Affects all Puppy Setup Kits.


use strict;
use warnings;


package ApolliaLib::Puppy_Setup_Kit::Settings;



################################################################
#
#                         Exported Stuff
#
# This bit specifies which parts of this module are made available to
# scripts that use the following line:
#
# 	use ApolliaLib::Puppy_Setup_Kit::Settings;
#
#
# I pasted much of the below from:
#
# 	http://perldoc.perl.org/perlmod.html


BEGIN 
{
	require Exporter;

	#set the version for version checking
	our $VERSION = 1.00;

	#Inherit from Exporter to export functions and variables
	our @ISA = qw(Exporter);

	#Functions and variables which are exported by default
	our @EXPORT = qw(  
		
		$Puppy_Setup_Kit_Settings 
		$Puppy_Setup_Kit_Settings_say_we 
		$Puppy_Setup_Kit
		
		);

	#Functions and variables which can be optionally exported
	our @EXPORT_OK = qw(
	
		);
}



#
#                End of Exported Stuff section. 
#
#
################################################################



##################################################
#
#	Needed imports from other modules
#


use ApolliaLib::Common::Misc qw ( 
	true 
	false 
	Escape_Single_Quotes_for_Bash
	Print_Hash_Indentedly
	necho
	Display_and_Do_Command_Line_and_Result
	$dollar_sign
	);


use ApolliaLib::File_Handling::Overwriting::Single_File (
	"Overwrite_Filepath_with_Text"
	);
	
use ApolliaLib::Puppy_Setup_Kit::Arguments qw (
	Determine_Using_Argv_Whether_To_Install_Without_Asking
	);

use ApolliaLib::Common::Script_Details qw ( 
	$Launched_Script_Details
	$Launched_Script
	);

use ApolliaLib::File_Handling::Contents (
	"Make_String_out_of_File_Contents"
);

	
use ApolliaLib::Common::Settings qw (
	$ApolliaLib_Settings
);

use ApolliaLib::Common::End_Script qw ( 
	
	End_Script_and_Wait_for_Return
	End_Script_For_Problem_Using_Whatever_Ending_Type_The_Settings_Specify
	);

	
use Object::Tiny qw( 
	Main_Disk_of_Puppy_Setup_Kit
	Main_Mountpoint_Parentdir_of_Puppy_Setup_Kit
	Abbrev_of_Puppy_Distro
	
	Parentdirs_of_Puppy_Setup_Kit
	
	grandparentdirs_for_Main_Disk_of_Puppy_Setup_Kit_note_files

	Puppy_Distro_Abbrev
	
	List_of_Setup_Kit_Realm_Names
	List_of_Realm_Hash_Names
	
	realm_paths
	
	Name_of_Puppy_Setup_Kit
	
	
	should_or_should_not_install_without_asking	
	should_allow_downloading_direct_to_physical_disk_if_RAM_disk_is_too_full
	RAM_floor_in_MB
	should_use_temporary_workspace_for_downloads
	should_allow_downloads
	should_not_perform_first_sha512_checksum_check_after_download
	should_not_perform_second_sha512_checksum_check_after_download
	
	List_of_Args_for_ApolliaLib_to_Ignore

	
	);

use ApolliaLib::File_or_Folder_Handling::Many_Items (
	"Copy_Many_Items_to_Their_Dest_Parentdirs"
	);


#
#	End of Needed imports from other modules
#
##################################################




################################################################
#
#	Start of stuff unique to this module
#



sub new
{
	my $class = shift;
	
	
	#----
	my $self;
	
	#@@@@
	$self = {};
			

	##############################
	
	bless $self, $class;
	
	return $self;
}



sub Initialize 
{
	my $self = $_[0];

	
	#----
	my $realm;

	
	
	#########################################################################
	#
	#   Default settings you can adjust if desired:
	#
	
	
	my $Main_Disk_of_Puppy_Setup_Kit = "Not Yet Known";
										$self->{'Main_Disk_of_Puppy_Setup_Kit'} = $Main_Disk_of_Puppy_Setup_Kit;
	
	my $Main_Mountpoint_Parentdir_of_Puppy_Setup_Kit= "Not Yet Known";
										$self->{'Main_Mountpoint_Parentdir_of_Puppy_Setup_Kit'} = $Main_Mountpoint_Parentdir_of_Puppy_Setup_Kit;

	my $Parentdirs_of_Puppy_Setup_Kit="Not Yet Known";
										$self->{'Parentdirs_of_Puppy_Setup_Kit'} = $Parentdirs_of_Puppy_Setup_Kit;
										
	my $grandparentdirs_for_Main_Disk_of_Puppy_Setup_Kit_note_files="/root/zzzzz-tmp/Apollia's Puppy Setup Kit Settings";
										$self->{'grandparentdirs_for_Main_Disk_of_Puppy_Setup_Kit_note_files'} = $grandparentdirs_for_Main_Disk_of_Puppy_Setup_Kit_note_files;
	# Don't put a slash on the end.
	# Default: /root/zzzzz-tmp/Apollia's Puppy Setup Kit Settings
										
	my $Puppy_Distro_Abbrev="Not Yet Specified";
										$self->{'Puppy_Distro_Abbrev'} = $Puppy_Distro_Abbrev;
		# This string - example: "Lupu5284" - is used in both realm folders and to access the
		# Puppy Setup Kit module for that specific kit.
		
	
	my @List_of_Args_for_ApolliaLib_to_Ignore=("Dont_Install_Without_Asking", "Install_Without_Asking");
										@{$self->{'List_of_Args_for_ApolliaLib_to_Ignore'} } = @List_of_Args_for_ApolliaLib_to_Ignore;
	

	my @List_of_Setup_Kit_Realm_Names=("Woodland", "Waterfall", "Spring", "Hot Spring", "Weir", "Fjord", "Forest");
										@{$self->{'List_of_Setup_Kit_Realm_Names'} } = @List_of_Setup_Kit_Realm_Names;
	
	my @List_of_Realm_Hash_Names=( "full_realm_paths", "full_realm_paths_for_any_pup");
										@{$self->{'List_of_Realm_Hash_Names'} }= @List_of_Realm_Hash_Names;
										
	my $Name_of_Puppy_Setup_Kit="Not Yet Specified Puppy Setup Kit";
										$self->{'Name_of_Puppy_Setup_Kit'} = $Name_of_Puppy_Setup_Kit;
	
	my $should_or_should_not_install_without_asking=false;
										$self->{'should_or_should_not_install_without_asking'} = $should_or_should_not_install_without_asking;

	my $should_allow_downloading_direct_to_physical_disk_if_RAM_disk_is_too_full=true;
										$self->{'should_allow_downloading_direct_to_physical_disk_if_RAM_disk_is_too_full'} = $should_allow_downloading_direct_to_physical_disk_if_RAM_disk_is_too_full;
										
	my $RAM_floor_in_MB=200;
										$self->{'RAM_floor_in_MB'} = $RAM_floor_in_MB;
	
	my $should_use_temporary_workspace_for_downloads=true;
										$self->{'should_use_temporary_workspace_for_downloads'} = $should_use_temporary_workspace_for_downloads;
	
	my $should_allow_downloads=true;
										$self->{'should_allow_downloads'} = $should_allow_downloads;
	
	my $should_not_perform_first_sha512_checksum_check_after_download=false;
										$self->{'should_not_perform_first_sha512_checksum_check_after_download'} = $should_not_perform_first_sha512_checksum_check_after_download;
	
	my $should_not_perform_second_sha512_checksum_check_after_download=false;
										$self->{'should_not_perform_second_sha512_checksum_check_after_download'} = $should_not_perform_second_sha512_checksum_check_after_download;
																		
	
	
						
		# The Puppy Setup Kit may not use RAM space for downloads if the amount of
		# available RAM is less than the RAM floor number.
										
										
	#
	#   End of Default settings you can adjust if desired
	#
	#########################################################################
	

#########################################################################
#
#   Settings you shouldn't mess with, which are calculated or observed, 
#	or which are set later inside individual scripts.
#

	
	foreach $realm (@List_of_Setup_Kit_Realm_Names)
	{	
		$self->{full_realm_paths}{$realm}="Not Yet Specified";
		$self->{full_realm_paths_for_any_pup}{$realm}="Not Yet Specified";
	}
	


#
#   End of Settings you shouldn't mess with, which are calculated or observed, 
#	or which are set later inside individual scripts.
#
#########################################################################

	
	return $self;
}

#
#	End of Initialize() method
#	
#	of 
#	
#	$Puppy_Setup_Kit_Settings
#
#
####################################################




sub Puppy_Distro_is_
{	
	# This does more than the name implies.  Sets the Puppy distro abbreviated name,
	# _and_ also does some other necessary setup.
	
	my $self = $_[0];
	my $distro_abbrev=$_[1];



	##############################
	
	if ( !Is_Valid_Puppy_Puppy_Distro_Abbrev( $distro_abbrev ) )
	{	
		print "\n\nAborting this script, because the Puppy distro abbreviation specified - $distro_abbrev - was unrecognized!  If this is a valid new distro, please edit the source code of the Puppy Setup Kit to make it recognize this distro.";
		
		
		#!!!!
		End_Script_and_Wait_for_Return();
	}
	
	$self->Set_Distro_Abbrev($distro_abbrev);
	$self->Set_Long_Name_of_Puppy_Distro();
	
	$self->Find_Out_Parentdirs_of_Puppy_Setup_Kit_from_File_in_Zzzzz_Tmp();
	$self->Find_Out_Main_Disk_of_Puppy_Setup_Kit_from_File_in_Zzzzz_Tmp();
	$self->Find_Out_Abbrev_of_Puppy_Distro_from_File_in_Zzzzz_Tmp();
	
	$self->Find_Out_Main_Mountpoint_Parentdir_of_Puppy_Setup_Kit_from_File_in_Zzzzz_Tmp();
	
	$self->Glue_the_Realm_Paths();
	
	
	
	$ApolliaLib_Settings->Add_Non_ApolliaLib_Args_to_Ignore( \@{$self->{'List_of_Args_for_ApolliaLib_to_Ignore'} } );
	
	#Determine_Using_Argv_Whether_To_Install_Without_Asking(\@ARGV);	
}




############################################################################
#
#
# 	If query functions - used in if statements
#
#




sub Find_Out_Available_RAM_in_This_Puppys_RAM_Disk
{
	my $self=$_[0];
	
	#----
	my $command_line;
	my $command_line_result;
	my $result;
	
	
	
	##############################
	
	print "\n\nFinding out how much RAM is available in " . $self->Abbrev_of_Puppy_Distro . "'s RAM disk...";
	
	if ($self->Abbrev_of_Puppy_Distro eq "Lupu5284")
	{	
		$command_line = "df --block-size M --type aufs | tail -1 | awk 'NF && NF-1 { print ( $dollar_sign(NF-2) ) }'";
		
		$command_line_result=Display_and_Do_Command_Line_and_Result($command_line);
		
		$result=$command_line_result;
		
		chomp $result;	# Gets rid of a whitespace at the end
		chop $result;	# Gets rid of M
		
		return $result;
	}
	else
	{	print "\n\nNot sure how to do that for this Puppy!";


		#----
		return 0;
	}
}



sub Is_Valid_Puppy_Puppy_Distro_Abbrev
{
	my $distro_abbrev=$_[0];

	
	
	##############################
	
	if ($distro_abbrev eq "Lupu5284")
	{	
		
		#!!!!
		return true;
	}
	elsif ($distro_abbrev eq "LH64_602_B2")
	{
	
		#!!!!
		return true;
	}
	elsif ($distro_abbrev eq "Multiple_Pups")
	{
	
		#!!!!
		return true;
	}
	elsif ($distro_abbrev eq "Any_Pup")
	{
	
		#!!!!
		return true;
	}
	elsif ($distro_abbrev eq "Any_32_bit_Pup")
	{
	
		#!!!!
		return true;
	}
	elsif ($distro_abbrev eq "Any_64_bit_Pup")
	{
	
		#!!!!
		return true;
	}
	
	
	#!!!!
	return false;
}



#
#
# 	End of If query functions - used in if statements
#
#
############################################################################




############################################################################
#
#
# 	Boring setter functions
#
#



sub Set_Distro_Abbrev
{	my $self = $_[0];
	my $distro_abbrev=$_[1];
	

	
	##############################
	
	$self->{'Puppy_Distro_Abbrev'}=$distro_abbrev;

}



sub Set_Long_Name_of_Puppy_Distro
{
	my $self = $_[0];
	
	
	#----
	my $distro_abbrev;
	
	#@@@@
	$distro_abbrev=$self->Puppy_Distro_Abbrev;
	
	
	
	##############################

	
	if ( $distro_abbrev eq "Lupu5284")
	{	
		$self->{'Name_of_Puppy_Setup_Kit'}="Lucid Puppy 5.2.8 version 004";
	}
	elsif ($distro_abbrev eq "LH64_602_B2")
	{
		$self->{'Name_of_Puppy_Setup_Kit'}="Lighthouse 64 6.02 Beta 2";
	}
	elsif ($distro_abbrev eq "Any_Pup")
	{
		$self->{'Name_of_Puppy_Setup_Kit'}="Any Puppy";
	}
	elsif ($distro_abbrev eq "Any_32_bit_Pup")
	{
		$self->{'Name_of_Puppy_Setup_Kit'}="Any 32-bit Puppy";
	}
	elsif ($distro_abbrev eq "Any_64_bit_Pup")
	{
		$self->{'Name_of_Puppy_Setup_Kit'}="Any 64-bit Puppy";
	}
	else
	{	$self->{'Name_of_Puppy_Setup_Kit'}="Unrecognized Puppy Setup Kit";
	}
}



#
#
# 	End of Boring setter functions
#
#
############################################################################





############################################################################
#
#
# 	String assembly functions
#
#



sub Glue_the_Realm_Paths
{	my $self = $_[0];
	
	
	#----
	my $disk;
	my $mountpoint_parentdir;
	my $distro_abbrev;
	my $realm;
	my $parentdirs_of_puppy_setup_kit;
	
	my @List_of_Setup_Kit_Realm_Names;
	
	
	#@@@@
	$distro_abbrev=$self->Puppy_Distro_Abbrev;
	$disk=$Launched_Script->disk;	
	$mountpoint_parentdir=$Launched_Script->mountpoint_parentdir;
	@List_of_Setup_Kit_Realm_Names=@{ $self->{List_of_Setup_Kit_Realm_Names} };
	
	$parentdirs_of_puppy_setup_kit=$self->Parentdirs_of_Puppy_Setup_Kit;
	
	
	##############################
	
	foreach $realm (@List_of_Setup_Kit_Realm_Names)
	{	
		$self->{full_realm_paths}{$realm}="/$mountpoint_parentdir/$disk/$parentdirs_of_puppy_setup_kit/$realm/$distro_abbrev";
		
		$self->{full_realm_paths_for_any_pup}{$realm}="/$mountpoint_parentdir/$disk/$parentdirs_of_puppy_setup_kit/$realm/Any_Pup";
		
		#print "Waterfall path: " . $self->{relative_realm_paths}{"Waterfall"}; # This line works

	}
	
}


sub Path_of_Text_File_Stating_Abbrev_of_Puppy_Distro
{
	my $self = $_[0];
	
	#----
	my $result;
	
	
	
	##############################
		
	$result = $self->grandparentdirs_for_Main_Disk_of_Puppy_Setup_Kit_note_files . "/Abbrev of Puppy Distro.txt";
	
	
	#!!!!
	return $result;
	
}


sub Path_of_Text_File_Stating_Parentdirs_of_Puppy_Setup_Kit
{	
	my $self = $_[0];

	#----
	my $result;


	
	##############################
	
	$result = $self->grandparentdirs_for_Main_Disk_of_Puppy_Setup_Kit_note_files . "/Parentdirs of Puppy Setup Kit.txt";
	
	
	#!!!!
	return $result;
}



sub Path_of_Text_File_Stating_Main_Mountpoint_Parentdir_of_Puppy_Setup_Kit
{	
	my $self = $_[0];

	#----
	my $result;


	
	##############################
	
	$result = $self->grandparentdirs_for_Main_Disk_of_Puppy_Setup_Kit_note_files . "/Mountpoint Parentdir of Puppy Setup Kit.txt";
	
	
	#!!!!
	return $result;
}



sub Path_of_Text_File_Stating_Main_Disk_of_Puppy_Setup_Kit
{	
	my $self = $_[0];

	#----
	my $result;


	
	##############################
	
	$result = $self->grandparentdirs_for_Main_Disk_of_Puppy_Setup_Kit_note_files . "/Main Disk of Puppy Setup Kit.txt";
	
	
	#!!!!
	return $result;
}


#
#
# 	End of String assembly functions
#
#
############################################################################




############################################################################
#
#
# 	File reading function
#
#


sub Find_Out_Abbrev_of_Puppy_Distro_from_File_in_Zzzzz_Tmp
{
	my $self = $_[0];
	
	
	#----
	my $fullpath_to_file_in_tmp;
	my $file_contents;


	##############################
	
	$fullpath_to_file_in_tmp = $self->Path_of_Text_File_Stating_Abbrev_of_Puppy_Distro();

	$file_contents = Make_String_out_of_File_Contents($fullpath_to_file_in_tmp);

	
	if ($file_contents)
	{
		$self->{'Abbrev_of_Puppy_Distro'}=$file_contents;
	}
	
}


	
sub Find_Out_Parentdirs_of_Puppy_Setup_Kit_from_File_in_Zzzzz_Tmp
{	
	my $self = $_[0];
	
	
	#----
	my $fullpath_to_file_in_tmp;
	my $FILEHANDLE;
	my $file_contents;
	
	
	
	##############################
	
	$fullpath_to_file_in_tmp = $self->Path_of_Text_File_Stating_Parentdirs_of_Puppy_Setup_Kit();

	$file_contents = Make_String_out_of_File_Contents($fullpath_to_file_in_tmp);

	
	if ($file_contents)
	{
		$self->{'Parentdirs_of_Puppy_Setup_Kit'}=$file_contents;
	}
	
}


sub Find_Out_Main_Mountpoint_Parentdir_of_Puppy_Setup_Kit_from_File_in_Zzzzz_Tmp
{	
	my $self = $_[0];
	
	
	#----
	my $fullpath_to_file_in_tmp;
	my $FILEHANDLE;
	my $file_contents;
	
	
	
	##############################
	
	$fullpath_to_file_in_tmp = $self->Path_of_Text_File_Stating_Main_Mountpoint_Parentdir_of_Puppy_Setup_Kit();

	$file_contents = Make_String_out_of_File_Contents($fullpath_to_file_in_tmp);

	
	if ($file_contents)
	{
		$self->{'Main_Mountpoint_Parentdir_of_Puppy_Setup_Kit'}=$file_contents;
	}
	
}



sub Find_Out_Main_Disk_of_Puppy_Setup_Kit_from_File_in_Zzzzz_Tmp
{	
	my $self = $_[0];
	
	
	#----
	my $fullpath_to_main_disk_file_in_tmp;
	my $FILEHANDLE;
	my $file_contents;
	
	
	
	##############################
	
	$fullpath_to_main_disk_file_in_tmp = $self->Path_of_Text_File_Stating_Main_Disk_of_Puppy_Setup_Kit();

	$file_contents = Make_String_out_of_File_Contents($fullpath_to_main_disk_file_in_tmp);

	
	if ($file_contents)
	{
		$self->{'Main_Disk_of_Puppy_Setup_Kit'}=$file_contents;
	}
	
}


#
#
# 	End of File reading function
#
#
############################################################################



############################################################################
#
#
# 	Get Realm Path functions
#
#


sub Realm_Path
{	
	my $self = $_[0];
	my $realm= $_[1];

	
	
	##############################
	
	
	#!!!!
	return $self->{full_realm_paths}{$realm} . "/";	
}



sub Realm_Path_on_Other_Disk
{	

	my $self = $_[0];
	my $disk = $_[1];
	my $realm=$_[2];
	
	
	#----
	my $mountpoint_parentdir;
	my $distro_abbrev;
	my $result;
	my $parentdirs_of_puppy_setup_kit;
	
	#@@@@
	$mountpoint_parentdir=$self->Main_Mountpoint_Parentdir_of_Puppy_Setup_Kit;
	$distro_abbrev=$self->Puppy_Distro_Abbrev;
	$parentdirs_of_puppy_setup_kit=$self->Parentdirs_of_Puppy_Setup_Kit;



	##############################

	$result = "/$mountpoint_parentdir/$disk/$parentdirs_of_puppy_setup_kit/$realm/$distro_abbrev/";
	
	return $result;
	
}



sub Realm_Path_on_Main_Disk
{	

	my $self = $_[0];
	my $realm=$_[1];
	
	
	#----
	my $disk;
	my $mountpoint_parentdir;
	my $distro_abbrev;
	my $result;
	my $parentdirs_of_puppy_setup_kit;
	
	#@@@@
	$disk=$self->Main_Disk_of_Puppy_Setup_Kit;
	$mountpoint_parentdir=$self->Main_Mountpoint_Parentdir_of_Puppy_Setup_Kit;
	$distro_abbrev=$self->Puppy_Distro_Abbrev;
	$parentdirs_of_puppy_setup_kit=$self->Parentdirs_of_Puppy_Setup_Kit;



	##############################

	$result = "/$mountpoint_parentdir/$disk/$parentdirs_of_puppy_setup_kit/$realm/$distro_abbrev/";
	
	return $result;
	
}



sub Realm_Path_for_Any_Pup
{	
	my $self = $_[0];
	my $realm=$_[1];
	
	
	#----
	my $result;
	
	
	
	##############################
	
	$result = $self->{full_realm_paths_for_any_pup}{$realm} . "/";
	
	#necho $result;
	
	
	#!!!!
	return $result;
	
}



sub Realm_Path_for_Any_Pup_on_Main_Disk
{	
	my $self = $_[0];
	my $realm=$_[1];
	
	
	#----
	my $main_disk;
	my $mountpoint_parentdir;
	my $result;
	my $parentdirs_of_puppy_setup_kit;
	
	#@@@@
	$main_disk=$self->Main_Disk_of_Puppy_Setup_Kit;
	$mountpoint_parentdir=$self->Main_Mountpoint_Parentdir_of_Puppy_Setup_Kit;
	$parentdirs_of_puppy_setup_kit=$self->Parentdirs_of_Puppy_Setup_Kit;


	##############################

	$result = "/$mountpoint_parentdir/$main_disk/$parentdirs_of_puppy_setup_kit/$realm/Any_Pup/";
	
	
	#!!!!
	return $result;
	
}



sub Pupless_Realm_Path_on_Other_Disk
{	
	my $self = $_[0];
	my $disk =$_[1];
	my $realm= $_[2];
	
	#----
	my $mountpoint_parentdir;
	my $result;
	my $parentdirs_of_puppy_setup_kit;
	
	#@@@@
	$mountpoint_parentdir=$self->Main_Mountpoint_Parentdir_of_Puppy_Setup_Kit;
	$parentdirs_of_puppy_setup_kit=$self->Parentdirs_of_Puppy_Setup_Kit;



	##############################
	
	$result = "/$mountpoint_parentdir/$disk/$parentdirs_of_puppy_setup_kit/$realm/";
	
	
	#!!!!
	return $result;
}



sub Pupless_Realm_Path_on_Main_Disk
{	
	my $self = $_[0];
	my $realm= $_[1];
	
	#----
	my $main_disk;
	my $mountpoint_parentdir;
	my $result;
	my $parentdirs_of_puppy_setup_kit;
	
	#@@@@
	$main_disk=$self->Main_Disk_of_Puppy_Setup_Kit;
	$mountpoint_parentdir=$self->Main_Mountpoint_Parentdir_of_Puppy_Setup_Kit;
	$parentdirs_of_puppy_setup_kit=$self->Parentdirs_of_Puppy_Setup_Kit;



	##############################
	
	$result = "/$mountpoint_parentdir/$main_disk/$parentdirs_of_puppy_setup_kit/$realm/";
	
	
	#!!!!
	return $result;
}



sub Realm_Path_Without_Pup
{	
	my $self = $_[0];
	my $realm= $_[1];
	
	#----
	my $disk;
	my $mountpoint_parentdir;
	my $result;
	my $parentdirs_of_puppy_setup_kit;
	
	#@@@@
	$disk=$Launched_Script->disk;
	$mountpoint_parentdir=$self->Main_Mountpoint_Parentdir_of_Puppy_Setup_Kit;
	$parentdirs_of_puppy_setup_kit=$self->Parentdirs_of_Puppy_Setup_Kit;



	##############################
	
	$result = "/$mountpoint_parentdir/$disk/$parentdirs_of_puppy_setup_kit/$realm/";
	
	
	#!!!!
	return $result;
}



sub Realm_Path_for_Other_Pup_on_Main_Disk
{	
	my $self = $_[0];
	my $other_pup_distro_abbrev=$_[1];
	my $realm=$_[2];
	
	
	#----
	my $main_disk;
	my $mountpoint_parentdir;
	my $result;
	my $parentdirs_of_puppy_setup_kit;
	
	#@@@@
	$main_disk=$self->Main_Disk_of_Puppy_Setup_Kit;
	$mountpoint_parentdir=$self->Main_Mountpoint_Parentdir_of_Puppy_Setup_Kit;
	$parentdirs_of_puppy_setup_kit=$self->Parentdirs_of_Puppy_Setup_Kit;



	##############################
	
	
	if ( Is_Valid_Puppy_Puppy_Distro_Abbrev ( $other_pup_distro_abbrev ) )
	{	
		$result = "/$mountpoint_parentdir/$main_disk/$parentdirs_of_puppy_setup_kit/$realm/$other_pup_distro_abbrev/";
		
		
		#!!!!
		return $result;
	}
	
	
	#!!!!
	End_Script_For_Problem_Using_Whatever_Ending_Type_The_Settings_Specify("\n\n\n\nAn invalid Puppy distro abbreviation - \"$other_pup_distro_abbrev\" - was supplied to the function Realm_Path_for_Other_Pup_on_Main_Disk()!  Aborting!");
		
}



sub Realm_Path_for_Other_Pup
{	
	my $self = $_[0];
	my $other_pup_distro_abbrev=$_[1];
	my $realm=$_[2];
	
	
	#----
	my $disk;
	my $mountpoint_parentdir;
	my $result;
	my $parentdirs_of_puppy_setup_kit;
	
	#@@@@
	$disk=$Launched_Script->disk;
	$mountpoint_parentdir=$Launched_Script->mountpoint_parentdir;
	$parentdirs_of_puppy_setup_kit=$self->Parentdirs_of_Puppy_Setup_Kit;


	##############################

	if ( Is_Valid_Puppy_Puppy_Distro_Abbrev ( $other_pup_distro_abbrev ) )
	{	
		$result = "/$mountpoint_parentdir/$disk/$parentdirs_of_puppy_setup_kit/$realm/$other_pup_distro_abbrev/";
		
		
		#!!!!
		return $result;
	}
	
	
	#!!!!
	End_Script_For_Problem_Using_Whatever_Ending_Type_The_Settings_Specify("\n\n\n\nAn invalid Puppy distro abbreviation - \"$other_pup_distro_abbrev\" - was supplied to the function Realm_Path_for_Other_Pup()!  Aborting!");
}



sub Realm_Path_for_Other_Pup_on_Other_Disk
{	
	my $self = $_[0];
	my $disk = $_[1];
	my $other_pup_distro_abbrev=$_[1];
	my $realm=$_[2];
	
	
	#----
	my $mountpoint_parentdir;
	my $result;
	my $parentdirs_of_puppy_setup_kit;
	
	#@@@@
	$mountpoint_parentdir=$Launched_Script->mountpoint_parentdir;
	$parentdirs_of_puppy_setup_kit=$self->Parentdirs_of_Puppy_Setup_Kit;


	##############################

	if ( Is_Valid_Puppy_Puppy_Distro_Abbrev ( $other_pup_distro_abbrev ) )
	{	
		$result = "/$mountpoint_parentdir/$disk/$parentdirs_of_puppy_setup_kit/$realm/$other_pup_distro_abbrev/";
		
		
		#!!!!
		return $result;
	}
	
	
	#!!!!
	End_Script_For_Problem_Using_Whatever_Ending_Type_The_Settings_Specify("\n\n\n\nAn invalid Puppy distro abbreviation - \"$other_pup_distro_abbrev\" - was supplied to the function Realm_Path_for_Other_Pup_on_Other_Disk()!  Aborting!");
}


#
#
# 	Get Realm Path functions
#
#
############################################################################




############################################################################
#
#
# 	File output functions
#
#


sub Output_Text_File_Stating_Main_Disk_of_Puppy_Setup_Kit
{	
	my $self=$_[0];
	
	
	
	##############################
		
	Overwrite_Filepath_with_Text( $self->Path_of_Text_File_Stating_Main_Disk_of_Puppy_Setup_Kit, "and put the following text:", $Launched_Script->disk);	
}



sub Output_Text_File_Stating_Mountpoint_Parentdir_of_Puppy_Setup_Kit
{	
	my $self=$_[0];
	
	
	
	##############################
		
	Overwrite_Filepath_with_Text( $self->Path_of_Text_File_Stating_Main_Mountpoint_Parentdir_of_Puppy_Setup_Kit, "and put the following text:", $Launched_Script->mountpoint_parentdir);	
}



sub Output_Text_File_Stating_Disk_of_Legacy_Lib_Apollia
{	
	my $self=$_[0];
	
	
	
	##############################
		
	Overwrite_Filepath_with_Text( 
		"/tmp/apollia_perl_library_disk.txt", 
		
		"and put the following text:", 
		
		$self->Main_Disk_of_Puppy_Setup_Kit,

	);
	
}

sub Output_Text_File_Stating_Fullpath_to_Legacy_Lib_Apollia
{	
	my $self=$_[0];


	
	##############################
	
	Overwrite_Filepath_with_Text( 
		"/tmp/apollia_perl_library_path.txt", 
		
		"and put the following text:", 
		
		"/root/zzzzz-tmp/Apollias_Perl_Modules/lib/perl5/ApolliaLib/Legacy/Lib-Apollia.pl"
	);
	
}


#
#
# 	End of File output functions
#
#
############################################################################




############################################################################
#
#
# 	Display functions
#
#


sub Print_All_Realm_Hashes
{	
	my $self = $_[0];
	
	
	#----
	my $hash_name;
	my %hash;
	my @List_of_Realm_Hash_Names;
	
	#@@@@
	@List_of_Realm_Hash_Names=@{ $self->{List_of_Realm_Hash_Names} };


	
	##############################

		
	foreach $hash_name (@List_of_Realm_Hash_Names)
	{	
		%hash=%{ $self->{$hash_name} };
		
		print "\n\nRealm hash name: $hash_name\n\n";
		
		Print_Hash_Indentedly(\%hash);
	}
	
}



#
#
# 	End of Display functions
#
#
############################################################################



#---------------------------------------------------------------------------
#
#	Now the $Puppy_Setup_Kit_Settings object actually gets created, 
#	but no settings are set yet.
#
#

our $Puppy_Setup_Kit_Settings;

$Puppy_Setup_Kit_Settings=ApolliaLib::Puppy_Setup_Kit::Settings->new();


#---------------------------------------------------------------------------
#
# 	Aliases, equivalent to $Puppy_Setup_Kit_Settings
#
#

our $Puppy_Setup_Kit_Settings_say_we;
	# Makes if statements very readable.

$Puppy_Setup_Kit_Settings_say_we=$Puppy_Setup_Kit_Settings;


our $Puppy_Setup_Kit;

$Puppy_Setup_Kit=$Puppy_Setup_Kit_Settings;


#---------------------------------------------------------------------------
#
#	Now, the $Puppy_Setup_Kit_Settings get set to their defaults.
#
#	The defaults don't suit every script, but that's OK, because 
#	individual scripts can (and often should) adjust the defaults.


$Puppy_Setup_Kit_Settings->Initialize();



1;