# ApolliaLib :: Wallpaper :: Misc.pm
#
# Stuff related to desktop wallpaper.
#
# It assumes that wallpaper can be set by the shell command "wallpaper",
# followed by a full path to the wallpaper image.


use strict;
use warnings;


package ApolliaLib::Wallpaper::Misc;



################################################################
#
#                         Exported Stuff
#
# This bit specifies which parts of this module are made available to
# scripts that use the following line:
#
# 	use ApolliaLib::Wallpaper::Misc;
#
#
# I pasted much of the below from:
#
# 	http://perldoc.perl.org/perlmod.html


BEGIN 
{
	require Exporter;

	#set the version for version checking
	our $VERSION = 1.00;

	#Inherit from Exporter to export functions and variables
	our @ISA = qw(Exporter);

	#Functions and variables which are exported by default
	our @EXPORT = qw(  
		Switch_Wallpaper_to_
		Make_Array_of_Wallpapers_Found_in_Folder
		Get_Random_Wallpaper_from_Array_
		Make_Filename_Array_of_Wallpapers_Found_in_Folder
		Use_A_Random_Wallpaper_from_Folder
		);

	#Functions and variables which can be optionally exported
	our @EXPORT_OK = qw(
	
		);
}



#
#                End of Exported Stuff section. 
#
#
################################################################



##################################################
#
#	Needed imports from other modules
#


use ApolliaLib::Common::Misc qw (
	true
	false
	Display_and_Do_Command_Line_and_Result
	Escape_Single_Quotes_for_Bash
	Get_Random_Item_from_Array_
	Add_Slash_If_Needed_To_
	);

use ApolliaLib::Common::Path_String_Handling qw (
	Get_File_Suffix_from_Path
	);
	
use ApolliaLib::Folder_Handling::Contents (
	"Make_Array_of_Folder_Contents"
	);


#
#	End of Needed imports from other modules
#
##################################################




################################################################
#
#	Start of stuff unique to this module
#



sub Use_A_Random_Wallpaper_from_Folder
{
	my $fullpath_to_folder_of_wallpaper=$_[0];
	
	
	#----
	my $ref_to_wallpaper_filenames_array;
	my @wallpapers;
	my $random_wallpaper;
	my $full_path_to_random_wallpaper;
	
	
	
	##############################
	
	$ref_to_wallpaper_filenames_array=Make_Filename_Array_of_Wallpapers_Found_in_Folder($fullpath_to_folder_of_wallpaper);
	
	$fullpath_to_folder_of_wallpaper=Add_Slash_If_Needed_To_($fullpath_to_folder_of_wallpaper);
	
	@wallpapers=@$ref_to_wallpaper_filenames_array;
	
	$random_wallpaper=Get_Random_Item_from_Array_(\@wallpapers);
	
	$full_path_to_random_wallpaper=$fullpath_to_folder_of_wallpaper . $random_wallpaper;
	
	Switch_Wallpaper_to_($full_path_to_random_wallpaper);
	
}



sub Make_Filename_Array_of_Wallpapers_Found_in_Folder
{	
	# Assumes wallpapers are all in JPG or PNG format.
	my $fullpath_to_folder=$_[0];
	
	
	#----
	my $ref_to_array_of_items_in_folder;

	my @wallpapers;
	my @items_in_folder;
	
	my $item;
		
	my $fullpath_to_wallpaper;
	my $file_suffix;



	##############################
	
	$ref_to_array_of_items_in_folder = Make_Array_of_Folder_Contents($fullpath_to_folder);
	
	@items_in_folder = @$ref_to_array_of_items_in_folder;
	
	$fullpath_to_folder=Add_Slash_If_Needed_To_($fullpath_to_folder);
	
	chdir $fullpath_to_folder;
	
	foreach $item (@items_in_folder)
	{	
		if (-f $item)
		{
			$file_suffix=Get_File_Suffix_from_Path($item);
			$file_suffix=lc($file_suffix); # Lowercases it
			
			if ($file_suffix eq ".jpg" || $file_suffix eq ".png")
			{	
				
				push(@wallpapers, $item);
			}
		}
		else
		{	next;
		}
	}
	
	
	#!!!!
	return \@wallpapers;
}



sub Make_Fullpath_Array_of_Wallpapers_Found_in_Folder
{	
	# Assumes wallpapers are all in JPG or PNG format.
	my $fullpath_to_folder=$_[0];
	
	
	#----
	my $ref_to_array_of_items_in_folder;
	
	my @items_in_folder;
	my @wallpapers;
	
	my $item;
		
	my $fullpath_to_wallpaper;
	my $file_suffix;
	
	
	
	##############################
		
	$ref_to_array_of_items_in_folder = Make_Array_of_Folder_Contents($fullpath_to_folder);
	
	@items_in_folder = @$ref_to_array_of_items_in_folder;
	
	$fullpath_to_folder=Add_Slash_If_Needed_To_($fullpath_to_folder);
	
	chdir $fullpath_to_folder;
	
	foreach $item (@items_in_folder)
	{	
		if (-f $item)
		{
			$file_suffix=Get_File_Suffix_from_Path($item);
			$file_suffix=lc($file_suffix); # Lowercases it
			

			if ($file_suffix eq ".jpg" || $file_suffix eq ".png")
			{	$fullpath_to_wallpaper=$fullpath_to_folder . $item;

				push(@wallpapers, $fullpath_to_wallpaper);
			}
		}
		else
		{	next;
		}
	}
	
	
	#!!!!	
	return \@wallpapers;
}



sub Get_Random_Wallpaper_from_Array_
{	# Almost just a more readable alias.  Get_Random_Item_from_Array_() does the real work.

	my $ref_to_array_of_wallpaper_fullpaths=$_[0];
	

	#----
	my @wallpapers;
	my $chosen_wallpaper;
	
	#@@@@
	@wallpapers=@$ref_to_array_of_wallpaper_fullpaths;
	
	

	##############################
	
	$chosen_wallpaper=Get_Random_Item_from_Array_(\@wallpapers);
	
	
	#!!!!
	return $chosen_wallpaper;
}



sub Switch_Wallpaper_to_
{	my $full_path_of_wallpaper_to_switch_to=$_[0];


	#----
	my $escaped_full_path_of_wallpaper_to_switch_to;
	my $command_line;
	my $result;
	
	
	
	##############################
	
	$escaped_full_path_of_wallpaper_to_switch_to=Escape_Single_Quotes_for_Bash($full_path_of_wallpaper_to_switch_to);
	
	$command_line = "wallpaper '$escaped_full_path_of_wallpaper_to_switch_to' 2>&1";
	
	$result=Display_and_Do_Command_Line_and_Result($command_line);

}



1;