#!/bin/sh

#	Script:                  Puppy Setup Kit Blueprint:               
#
#                 Default for Bruno 2 - with some non-libre things
#
#                    For Apollia's Lucid Puppy 5.2.8-4 Setup Kit
#
#
#	Version 1.0
#	by Apollia of Astroblahhh.Com.  http://astroblahhh.com/
#
#
#	Mostly completed May 16, 2014.
#
#	Polished a bit August 15 and August 25, 2014.
#
#
#
#------------------------------------------------------------------------------
# 
# Launches various scripts which launch lots of other scripts which install
# all sorts of different things, and sometimes ask you what you'd like 
# installed.
#
#------------------------------------------------------------------------------
#
#
#
# Under the GNU General Public License v3.0.
#
# Copyright (C) 2014  Apollia
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# Contact info:     http://astroblahhh.com/contact-apollia.shtml
#
# ----



#######################################################################
#
#   A thing you can change
#

terminal_window_title="Apollia's Lucid Puppy 5.2.8-4 Setup Kit - Default for Bruno 2 with some non-libre things"

#
#   End of A thing you can change
#
#######################################################################



############################################################
############################################################
#
#
#	Don't change this:
#
#


##########################################
#
#
# 	Start of function 
#
#
#	Escape_Single_Quotes_for_Bash_and_Enclose_in_Single_Quotes()
#
#

Escape_Single_Quotes_for_Bash_and_Enclose_in_Single_Quotes()
{	
	local quotes_escaped

	quotes_escaped=$(echo "$1" | sed -e "s/'/'\"'\"'/g")

	echo "'$quotes_escaped'"
}

#
#
#	End of function 
#
#	Escape_Single_Quotes_for_Bash_and_Enclose_in_Single_Quotes()
#
#
###########################################################



##########################################################
#
#	Now, we set a bunch of variables related to the script's
#	path and filename.
#


Get_Script_Path_and_Filename()
{
	THIS_SCRIPT_FILE_NAME="$(basename "${BASH_SOURCE[0]}" )"

	THIS_SCRIPT_FOLDER="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

	full_path_to_this_script="$THIS_SCRIPT_FOLDER/$THIS_SCRIPT_FILE_NAME"

	escaped_enclosed_full_path_to_this_script=$(Escape_Single_Quotes_for_Bash_and_Enclose_in_Single_Quotes "$full_path_to_this_script")
}



Get_Script_Path_and_Filename



##########################################################
#
#	Then, we announce this script's name, path, and what it is.
#


Announce_Script_Now_Running()
{
	echo -e "\n\n*(*)*\n\nBash Script Now Running:\n"
	
	echo "	$THIS_SCRIPT_FILE_NAME

Full path:
	$full_path_to_this_script"

}


Announce_Script_Now_Running

echo -e "\n\nA launcher for $terminal_window_title."




cd "$THIS_SCRIPT_FOLDER"

source "Not Standalone/00-Script Needed By All Lupu5284 Blueprints.sh"





##########################################################
#
#	We open a terminal window at this point and restart 
#	the script, if there was no Argument 1.
#
#	The script ends here if there was no Argument 1.
#


If_Necessary__Open_New_Terminal_Window_and_Relaunch_Self()
{	if [ -z "$1" ]
	# If argument 1 is empty...
	then

		# Need to escape the terminal window title for the command line.
		escaped_enclosed_terminal_window_title=$(Escape_Single_Quotes_for_Bash_and_Enclose_in_Single_Quotes "$terminal_window_title")

		command_line="rxvt -g 80x24 -rv -sb -sr -sl 99999 -title $escaped_enclosed_terminal_window_title -e sh $escaped_enclosed_full_path_to_this_script \"Terminal_Window_Was_Opened_Automatically\""
		
		echo -e "Relaunching this script in a new terminal window:\n\n# $command_line" 

		eval "$command_line"
		exit
	fi
}

If_Necessary__Open_New_Terminal_Window_and_Relaunch_Self "$1"



#
#
#	End of some stuff you shouldn't change.
# 
#
############################################################
############################################################



#######################################################################
#
#   Some stuff you can change
#



###################################
#
# Later on, the below relative paths will be prefixed with the
# $full_path_to_blueprint_scripts.
#
#
# The resultant path (not what you should input below) will probably look something like:
#
# /[mountpoint parentdir of the disk this script is on]/[disk this script is on]/[parentdirs of the Puppy Setup Kits]/Woodland/Lupu5284/Apollia's Puppy Setup Kit/Blueprints/For All Blueprints/Install Perl module local-lib.sh"
#
#
# Don't begin paths with a slash.
#
# Also, don't try to put the first parenthesis on the line after the equal sign.
# Bash gets confused if you do that.

relative_paths_to_bash_setup_scripts_for_this_blueprint=(
	#"Bash/Install Perl.sh"
	#"Install Perl module local-lib.sh"
	
)


relative_paths_to_perl_setup_scripts_for_this_blueprint=(
	"Step 0 - Preliminary Setup.pl"
	
	"Step 1 - Setup for Setup Kit, etc.pl" 
		"For Bruno 2/Step 2 - Some Setup for Bruno 2.pl"
	
	"Step 3 - Basics.pl"
	
	"Step 4 - Typical Things.pl"
		"For Bruno 2/Step 5 - More Setup for Bruno 2.pl"
	
	"Step 6 - Optional Things - with some non-libre things.pl"
	
		"For Bruno 2/Step 7 - Yet More Setup for Bruno 2.pl"
	
)



############################################################
############################################################
#
#   Paths you can probably leave alone.
#
#	Don't put a slash on the end.
#
#

full_path_to_blueprint_scripts="/$setup_kit_mountpoint_parentdir/$setup_kit_disk/$parentdirs_of_puppy_setup_kit/Woodland/$setup_kit_folder_name/Apollia's Puppy Setup Kit/Blueprints/Default"

#
#   End of Paths you can probably leave alone
#
#
############################################################
############################################################



#
#
#   End of Some stuff you can change
#
#   Below this point, nothing else in this script needs to be changed.
#
#
####################################################################################################
####################################################################################################
####################################################################################################




##########################################################
#
#	If the user specified any additional Bash scripts to run,
#	we loop through the list of them and run them.
#


#Run_Array_of_Scripts "sh" "$full_path_to_blueprint_scripts" relative_paths_to_bash_setup_scripts_for_this_blueprint[@]




##########################################################
#
#	Now, we can start running Puppy Setup Kit scripts
#	written in Perl, which will all be fully able to use
#	the Perl modules just installed.
#

Run_Array_of_Scripts "perl" "$full_path_to_blueprint_scripts" relative_paths_to_perl_setup_scripts_for_this_blueprint[@]


##########################################################
#
#	After all the Perl scripts are run, this Puppy Setup Kit
#	is just about finished running - everything should be 
#	installed now.
#

echo -n "." > "/root/zzzzz-tmp/Apollia's Puppy Setup Kit Settings/Puppy Setup Kit Launcher Can Create Note Stating That a Puppy Setup Kit Blueprint Was Already Installed.txt"



wmswitcher &
	# This just launches the window manager switcher, so I can quickly
	# switch to my preferred window manager, IceWM, whenever I'm ready.



End_Script_and_Wait_for_Return()
{
	local wait_for_return
	
	echo -e "\n\n\n\n**********";
	
	echo -e "\n\nScript finished!";
	
	echo -e "\n\n$THIS_SCRIPT_FILE_NAME";
	
	echo -e "\n\n$full_path_to_this_script";
	
	echo -e "\n\nYou can press Return (or Enter) to close this window.\n";
	read wait_for_return
	
	exit
}

End_Script_and_Wait_for_Return

#
#
#  End of script.
#
############################