#!/bin/sh

# 	Script:             Install Perl Module local-lib.sh
#
#
#	Version 1.0
#	by Apollia of Astroblahhh.Com.  http://astroblahhh.com/
#
#	Mostly completed May 18, 2014.  Improved a bit Sept. 11, 2014.
#
#
#
#------------------------------------------------------------------------------
#
# Installs the local::lib Perl module into:
#
#		/usr/share/perl/5.10.1/ 
#
# That makes it possible for the ApolliaLib modules to be used despite them 
# being in an odd location: /root/zzzzz-tmp/Apollias_Perl_Modules
#
# This script assumes the local::lib module is stored on the same disk as 
# this script.
#
#------------------------------------------------------------------------------
#
#
#
# Under the GNU General Public License v3.0.
#
# Copyright (C) 2014  Apollia
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# Contact info:     http://astroblahhh.com/contact-apollia.shtml
#
# ----



#######################################################################
#
#   Some stuff you can change
#

terminal_window_title="Bash Script: Install Apollia's Perl Modules"

should_make_symlink_to_folder_of_apollialib_modules_instead_of_copying=true
	# true|false
	

############################################################
#
#   Paths and folder names you can probably leave alone:
#
#

full_path_to_perl_module_folder="/usr/share/perl/5.10.1"

full_path_to_lib_module_dest_parentdirs="$full_path_to_perl_module_folder/local/"

full_path_to_local_lib_module="/root/zzzzz-tmp/Apollias_Perl_Modules/lib/perl5/local/lib.pm"


#
#
#   End of Some stuff you can change
#
#   Below this point, nothing else in this script needs to be changed.
#
#######################################################################



##########################################
#
#
# 	Start of function 
#
#
#	Escape_Single_Quotes_for_Bash_and_Enclose_in_Single_Quotes()
#
#

Escape_Single_Quotes_for_Bash_and_Enclose_in_Single_Quotes()
{	
	local quotes_escaped

	quotes_escaped=$(echo "$1" | sed -e "s/'/'\"'\"'/g")

	echo "'$quotes_escaped'"
}

#
#
#	End of function
#
#	Escape_Single_Quotes_for_Bash_and_Enclose_in_Single_Quotes()
#
#
###########################################################



##########################################################
#
#	Now, we set a bunch of variables related to the script's
#	path and filename.
#


Get_Script_Path_and_Filename()
{
	THIS_SCRIPT_FILE_NAME="$(basename "${BASH_SOURCE[0]}" )"

	THIS_SCRIPT_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

	full_path_to_this_script="$THIS_SCRIPT_DIR/$THIS_SCRIPT_FILE_NAME"

	escaped_enclosed_full_path_to_this_script=$(Escape_Single_Quotes_for_Bash_and_Enclose_in_Single_Quotes "$full_path_to_this_script")
}

Get_Script_Path_and_Filename



##########################################################
#
#	Then, we announce this script's name, path, and what it is.
#

Announce_Script_Now_Running()
{
	echo -e "\n\n*(*)*\n\nBash Script Now Running:\n"
	
	echo "	$THIS_SCRIPT_FILE_NAME

Full path:
	$full_path_to_this_script"

}

Announce_Script_Now_Running

echo -e "\n\nThe installer for the local::lib Perl module, which is necessary to make it possible for the ApolliaLib modules to be used despite them being in an odd location."



##########################################################
#
#	We open a terminal window at this point and restart 
#	the script, if there was no Argument 1.
#
#	The script ends here if there was no Argument 1.
#

If_Necessary__Open_New_Terminal_Window_and_Relaunch_Self()
{	if [ -z "$1" ]
	# If argument 1 is empty...
	then

		# Need to escape the terminal window title for the command line.
		escaped_enclosed_terminal_window_title=$(Escape_Single_Quotes_for_Bash_and_Enclose_in_Single_Quotes "$terminal_window_title")

		command_line="rxvt -g 80x44 -rv -sb -sr -sl 99999 -title $escaped_enclosed_terminal_window_title -e sh $escaped_enclosed_full_path_to_this_script \"Terminal_Window_Was_Opened_Automatically\" \"$2\" \"$3\" \"$4\""
		
		echo -e "Relaunching this script in a new terminal window:\n\n# $command_line" 

		eval "$command_line"
		exit
	fi
}

If_Necessary__Open_New_Terminal_Window_and_Relaunch_Self "$1" "$2" "$3" "$4"


##########################################################
#
#	If there was an Argument 1, the script can proceed.
#
#

echo -e "\n\n"



echo -e "Going to copy local::lib Perl module into:\n	$full_path_to_lib_module_dest_parentdirs"



##########################################################
#
#	Next, we create the folder for the Perl module local::lib.


escaped_enclosed_full_path_to_lib_module_dest_parentdirs=$(Escape_Single_Quotes_for_Bash_and_Enclose_in_Single_Quotes "$full_path_to_lib_module_dest_parentdirs")

command_line="mkdir --parents $escaped_enclosed_full_path_to_lib_module_dest_parentdirs"

echo -e "\n\n# $command_line\n\n"

eval "$command_line"
	
echo -e "\nHopefully created the needed folder!"
	
##########################################################
#
#	Now, we copy the local::lib module to its proper place.
#

escaped_enclosed_full_path_to_local_lib_module=$(Escape_Single_Quotes_for_Bash_and_Enclose_in_Single_Quotes "$full_path_to_local_lib_module")


echo -e "\n\nTrying to copy file..."
	
	command_line="cp $escaped_enclosed_full_path_to_local_lib_module --target-directory $escaped_enclosed_full_path_to_lib_module_dest_parentdirs 2>&1"
	
	echo -e "\n\n# $command_line\n\n"
	
	eval "$command_line"
	
	echo -e "\nHopefully copied files!"




End_Script_and_Wait_for_Return()
{
	local wait_for_return
	
	echo -e "\n\n\n\n---";
	
	echo -e "\n\nScript finished!";
	
	echo -e "\n\n$THIS_SCRIPT_FILE_NAME";
	
	echo -e "\n\n$full_path_to_this_script";
	
	echo -e "\n\nYou can press Return (or Enter) to close this window.\n";
	read wait_for_return
	
	exit
}


End_Script_Immediately()
{
	echo -e "\n\n\n\n---\n\n\nImmediate end of $THIS_SCRIPT_FILE_NAME!\n\n"
	echo -e "*(*)*"
	exit
}



if [ -z $2 ] || [ "$2" = "Wait_for_Return_At_Scripts_End" ] # If no 2nd argument, or it's "Wait_for_Return"...
then
	End_Script_and_Wait_for_Return
else 
	End_Script_Immediately
fi

End_Script_and_Wait_for_Return
exit

#  End of script.
#
############################