#!/bin/sh

#   Script:    Puppy Setup Kit - Switch Meadow Repository and Blueprint
#
#
#          Configured to switch to a different Meadow repository folder and
#                                 blueprint for:
#
#                 Lupu5284 - Lucid Puppy Linux 5.2.8 version 004                
#
#
#                                Meadow folder:
#
#                                    Meadow
#
#
#                                  Blueprint:                             
#
#                 Default for Bruno 2 - with some non-libre things 
#
#
#
# by Apollia of Astroblahhh.Com.  http://astroblahhh.com/
#
# Completed September 17, 2014.
#
#
#------------------------------------------------------------------------------
#
# Similar to a Launch Blueprint script, but this only switches the Meadow
# repository folder in use, and the current blueprint.
#
# Probably mainly useful to developers who have used Mercurial to make a 
# cloned repository.
#
#------------------------------------------------------------------------------
#
#
#
# Under the GNU General Public License v3.0.
#
# Copyright (C) 2014  Apollia
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
# Contact info:     http://astroblahhh.com/contact-apollia.shtml
#
# ----



#######################################################################
#
#   Some stuff you can change
#

PUPPY_DISTRO_ABBREV="Lupu5284"
	# The Puppy distro abbreviated name is used in many locations as 
	# a subfolder name.
	#
	#
	# Examples:
	#
	#
	# Lupu5284
	#      (Lucid Puppy Linux 5.2.8 version 004)
	#
	# LH64_602_B2
	#      (Lighthouse 64 Puppy Linux 6.02 Beta 2)
	#
	#
	# Some folders contain stuff that should work in multiple distros.
	#
	# Examples:
	#
	# Any_Pup, Any_32_bit_Pup, Any_64_bit_Pup
	
	

SETUP_KIT_PARENTDIRS="Meadow"
	# The setup kit's parent directories are the part of the setup kit's 
	# path after the mountpoint parentdir and disk name (such as /mnt/sda1/
	# or /media/Pup/), and before going into any of the Puppy Setup Kit 
	# realm folders (such as Woodland, Spring, Waterfall, or Weir).
	#
	# Don't put a slash at the beginning or end, but the middle is OK.
	#
	#
	#
	# Useable examples:
	#
	#
	# Meadow
	#       (Use if the setup kit is inside someplace like /mnt/sda1/Meadow)
	#
	# Puppy Setup Kits/Meadow
	#       (Use if the setup kit is inside someplace like /mnt/sda1/Puppy Setup Kits/Meadow)
	#
	#
	#
	# Examples of what NOT to do:
	#
	#
	# /Meadow
	#       (Bad because it begins with a slash.)
	#
	# Meadow/
	#       (Bad because it ends with a slash.)
	
	


BLUEPRINT_TO_FOLLOW="Default for Bruno 2 - with some non-libre things"
	# Each Puppy Setup Kit consists of many separate, independent scripts, 
	# most of which can be run in any order you like (and which can even 
	# be double-clicked on to run), and you don't even have to run them all.
	#
	# In fact, you shouldn't indiscriminately run them all, because some would
	# install settings which aren't compatible with your computer, or settings
	# which would conflict with each other.  Or, installing everything might 
	# use up too much of your available RAM.
	#
	#
	# The scripts which decide what other setup kit scripts to run, and when,
	# are what I call blueprints.
	#
	# Imagine the Puppy Setup Kit is an architect.  You could 
	# theoretically have this architect build you absolutely everything he 
	# knows how to build - but, you probably have limited patience and space, 
	# so, you're most likely to ask him to only build you a smaller subset 
	# of everything that he knows how to build.
	#
	# Likewise, at a restaurant, you generally wouldn't order absolutely 
	# everything available on the menu, but instead, only what you really want 
	# at the moment.
	#
	#
	#
	# Example blueprints:
	#
	# Default for Any Computer
	#
	# Default for Bruno 2
	#       (Bruno 2 is what I call one of my main computers.)
	#
	#
	# Blueprints are Bash scripts with .sh on the end.  However, you shouldn't
	# include .sh at the end of the $BLUEPRINT_TO_FOLLOW string - it will
	# be added automatically later.
	

should_make_symlink_to_folder_of_apollialib_modules_instead_of_copying=true
	# true|false
	#
	# Symlinking is useful if you're debugging and don't want to have to
	# frequently copy newly-updated modules to /root/zzzzz-tmp/ .
	#
	# If you're not debugging, then copying is better, so the modules
	# will be read from your RAM disk instead of a physical disk.


should_abort_if_a_setup_kit_was_already_installed=false
	# (This setting is ignored in a "Switch Meadow and Blueprint" script.)
	#
	# true|false
	#
	# This setting should always be set to true, unless you're debugging
	# and/or actually want to be able to re-run your Puppy Setup Kit 
	# launcher script multiple times.

#
#
#   End of Some stuff you can change
#
#   Below this point, nothing else in this script needs to be changed.
#
#
####################################################################################################
####################################################################################################
####################################################################################################



THIS_SCRIPT_FOLDER="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"



Get_Disk_and_Mountpoint_Parentdir_from_Path()
{
	
	local this_path=$1
	

	##########################################################
	#
	#	Next, we use the "read" command to make an array
	#	containing the disk name and the mountpoint parentdir.
	

	IFS="/"
		# Sets the delimiter for the read command.
		
	read -a pathparts this_path <<< "$this_path"
	
	mountpoint_parentdir="${pathparts[1]}"
	disk="${pathparts[2]}"
	
}


Get_Disk_and_Mountpoint_Parentdir_from_Path "$THIS_SCRIPT_FOLDER"



Escape_Single_Quotes_for_Bash_and_Enclose_in_Single_Quotes()
{	
	local quotes_escaped

	quotes_escaped=$(echo "$1" | sed -e "s/'/'\"'\"'/g")

	echo "'$quotes_escaped'"
}


full_path_to_use_source_command_on="/$mountpoint_parentdir/$disk/$SETUP_KIT_PARENTDIRS/Woodland/Any_Pup/Puppy Setup Kit Launcher/Not Standalone/Switch Meadow and Blueprint.sh"

escaped_full_path_to_use_source_command_on=$(Escape_Single_Quotes_for_Bash_and_Enclose_in_Single_Quotes "$full_path_to_use_source_command_on")


command_line="source $escaped_full_path_to_use_source_command_on"

eval "$command_line"


#
#
#  End of script.
#
############################